@@ -3,7 +3,7 @@
 echo "EXECUTING: Jasmine tests"
 echo
 
-cd $WORKSPACE/web/
+cd ${WORKSPACE}/web/
 
 /usr/bin/yarn install || { echo 'ERROR: Failed to install the required Javascript modules.' ; exit 1; }
 /usr/bin/yarn run linter  || { echo 'ERROR: Failed to lint the Javascript code.' ; exit 1; }
