@@ -60,7 +60,7 @@ done
 yarn install
 yarn run bundle
 
-for FILE in `ls -d pgAdmin/static/js/generated/*`
+for FILE in `ls -d pgadmin/static/js/generated/*`
 do
     echo Adding $FILE
     tar cf - $FILE | (cd ../pip-build/pgadmin4; tar xf -)
