@@ -86,8 +86,28 @@ setup(
 
     extras_require={
         # ...
-        ":python_version<'2.7'": ["ordereddict"],
-        ":python_version<='2.7'": ["backports.csv", "importlib"],
+        ":python_version<'2.7'": [
+            "psycopg2==2.7.3.2",
+            "Flask-Script==2.0.5",
+            "ordereddict",
+            "python-dateutil==2.5.0",
+            "SQLAlchemy==1.0.14",
+            "Flask-Security==1.7.5",
+            "Flask-BabelEx==0.9.3"
+        ],
+        ":python_version<='2.7'": [
+            "backports.csv==1.0.5",
+            "importlib==1.0.3"
+        ],
+        ":python_version>='2.7'": [
+            "psycopg2>=2.7.4",
+            "python-dateutil>=2.7.1",
+            "htmlmin==0.1.12",
+            "Flask-HTMLmin==1.3.2",
+            "SQLAlchemy>=1.2.5",
+            "Flask-Security>=3.0.0",
+            "sshtunnel>=0.1.3"
+        ]
     },
 
     # Specify data files to be included. For Python 2.6 include them in MANIFEST.in
