@@ -36,5 +36,9 @@ const QString PGA_APP_NAME = QString("pgAdmin 4");
 // Global function prototypes
 int main(int argc, char * argv[]);
 bool PingServer(QUrl url);
+void delay(int milliseconds);
+void cleanup();
+unsigned long sdbm(unsigned char *str);
+bool shutdownServer(QUrl url);
 
 #endif // PGADMIN4_H
