@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -10,28 +10,29 @@
 """The main pgAdmin module. This handles the application initialisation tasks,
 such as setup of logging, dynamic loading of modules etc."""
 import logging
-import os, sys
+import os
+import sys
 from collections import defaultdict
 from importlib import import_module
 
-from flask import Flask, abort, request, current_app, session
-from flask_babel import Babel, gettext
-from flask_htmlmin import HTMLMIN
-from flask_login import user_logged_in
-from flask_security import Security, SQLAlchemyUserDatastore
+from flask import Flask, abort, request, current_app, session, url_for
+from flask_babelex import Babel, gettext
+from flask_login import user_logged_in, user_logged_out
 from flask_mail import Mail
+from flask_paranoid import Paranoid
+from flask_security import Security, SQLAlchemyUserDatastore, current_user
 from flask_security.utils import login_user
 from werkzeug.datastructures import ImmutableDict
-
-from pgadmin.utils import PgAdminModule, driver
-from pgadmin.utils.versioned_template_loader import VersionedTemplateLoader
-from pgadmin.utils.session import create_session_interface
 from werkzeug.local import LocalProxy
 from werkzeug.utils import find_modules
 
+from pgadmin.model import db, Role, Server, ServerGroup, \
+    User, Keys, Version, SCHEMA_VERSION as CURRENT_SCHEMA_VERSION
+from pgadmin.utils import PgAdminModule, driver
 from pgadmin.utils.preferences import Preferences
-
-from pgadmin.model import db, Role, Server, ServerGroup, User, Keys
+from pgadmin.utils.session import create_session_interface, pga_unauthorised
+from pgadmin.utils.versioned_template_loader import VersionedTemplateLoader
+from datetime import timedelta
 
 # If script is running under python3, it will not have the xrange function
 # defined
@@ -88,13 +89,39 @@ class PgAdmin(Flask):
 
     @property
     def exposed_endpoint_url_map(self):
+        #############################################################
+        # To handle WSGI paths
+        # If user has setup application under WSGI alias
+        # like 'localhost/pgadmin4' then we have to append '/pgadmin4'
+        # into endpoints
+        #############################################################
+        import config
+        is_wsgi_root_present = False
+        if config.SERVER_MODE:
+            pgadmin_root_path = url_for('browser.index')
+            if pgadmin_root_path != '/browser/':
+                is_wsgi_root_present = True
+                wsgi_root_path = pgadmin_root_path.replace(
+                    '/browser/', ''
+                )
+
+        def get_full_url_path(url):
+            """
+            Generate endpoint URL at per WSGI alias
+            """
+            if is_wsgi_root_present and url:
+                return wsgi_root_path + url
+            else:
+                return url
+
+        # Fetch all endpoints and their respective url
         for rule in current_app.url_map.iter_rules('static'):
-            yield rule.endpoint, rule.rule
+            yield rule.endpoint, get_full_url_path(rule.rule)
 
         for module in self.submodules:
             for endpoint in module.exposed_endpoints:
                 for rule in current_app.url_map.iter_rules(endpoint):
-                    yield rule.endpoint, rule.rule
+                    yield rule.endpoint, get_full_url_path(rule.rule)
 
     @property
     def javascripts(self):
@@ -140,12 +167,28 @@ current_blueprint = LocalProxy(_find_blueprint)
 
 
 def create_app(app_name=None):
-
     # Configuration settings
     import config
     if not app_name:
         app_name = config.APP_NAME
 
+    # Only enable password related functionality in server mode.
+    if config.SERVER_MODE is True:
+        # Some times we need to access these config params where application
+        # context is not available (we can't use current_app.config in those
+        # cases even with current_app.app_context())
+        # So update these params in config itself.
+        # And also these updated config values will picked up by application
+        # since we are updating config before the application instance is
+        # created.
+
+        config.SECURITY_RECOVERABLE = True
+        config.SECURITY_CHANGEABLE = True
+        # Now we'll open change password page in alertify dialog
+        # we don't want it to redirect to main page after password
+        # change operation so we will open the same password change page again.
+        config.SECURITY_POST_CHANGE_VIEW = 'browser.change_password'
+
     """Create the Flask application, startup logging and dynamically load
     additional modules (blueprints) that are found in this directory."""
     app = PgAdmin(__name__, static_url_path='/static')
@@ -171,8 +214,10 @@ def create_app(app_name=None):
     logger.setLevel(logging.INFO)
 
     # Set SQLITE_PATH to TEST_SQLITE_PATH while running test cases
-    if "PGADMIN_TESTING_MODE" in os. environ and \
-                    os.environ["PGADMIN_TESTING_MODE"] == "1":
+    if (
+        'PGADMIN_TESTING_MODE' in os.environ and
+        os.environ['PGADMIN_TESTING_MODE'] == '1'
+    ):
         config.SQLITE_PATH = config.TEST_SQLITE_PATH
 
     # Ensure the various working directories exist
@@ -214,13 +259,17 @@ def create_app(app_name=None):
         language = 'en'
         if config.SERVER_MODE is False:
             # Get the user language preference from the miscellaneous module
-            misc_preference = Preferences.module('miscellaneous', False)
-            if misc_preference:
-                user_languages = misc_preference.preference(
-                    'user_language'
-                )
-                if user_languages:
-                    language = user_languages.get() or language
+            if current_user.is_authenticated:
+                user_id = current_user.id
+            else:
+                user = user_datastore.get_user(config.DESKTOP_USER)
+                if user is not None:
+                    user_id = user.id
+            user_language = Preferences.raw_value(
+                'miscellaneous', 'user_language', None, user_id
+            )
+            if user_language is not None:
+                language = user_language
         else:
             # If language is available in get request then return the same
             # otherwise check the session or cookie
@@ -231,7 +280,9 @@ def create_app(app_name=None):
             elif hasattr(session, 'PGADMIN_LANGUAGE'):
                 language = getattr(session, 'PGADMIN_LANGUAGE', language)
             elif hasattr(request.cookies, 'PGADMIN_LANGUAGE'):
-                language = getattr(request.cookies, 'PGADMIN_LANGUAGE', language)
+                language = getattr(
+                    request.cookies, 'PGADMIN_LANGUAGE', language
+                )
 
         return language
 
@@ -239,16 +290,10 @@ def create_app(app_name=None):
     # Setup authentication
     ##########################################################################
 
-    app.config['SQLALCHEMY_DATABASE_URI'] = u'sqlite:///{0}?timeout={1}'.format(
-        config.SQLITE_PATH.replace(u'\\', u'/'),
-        getattr(config, 'SQLITE_TIMEOUT', 500)
-    )
-
-    # Only enable password related functionality in server mode.
-    if config.SERVER_MODE is True:
-        # TODO: Figure out how to disable /logout and /login
-        app.config['SECURITY_RECOVERABLE'] = True
-        app.config['SECURITY_CHANGEABLE'] = True
+    app.config['SQLALCHEMY_DATABASE_URI'] = u'sqlite:///{0}?timeout={1}' \
+        .format(config.SQLITE_PATH.replace(u'\\', u'/'),
+                getattr(config, 'SQLITE_TIMEOUT', 500)
+                )
 
     # Create database connection object and mailer
     db.init_app(app)
@@ -256,7 +301,25 @@ def create_app(app_name=None):
     ##########################################################################
     # Upgrade the schema (if required)
     ##########################################################################
-    db_upgrade(app)
+    with app.app_context():
+        # Run migration for the first time i.e. create database
+        from config import SQLITE_PATH
+        if not os.path.exists(SQLITE_PATH):
+            db_upgrade(app)
+        else:
+            version = Version.query.filter_by(name='ConfigDB').first()
+            schema_version = version.value
+
+            # Run migration if current schema version is greater than the
+            # schema version stored in version table
+            if CURRENT_SCHEMA_VERSION >= schema_version:
+                db_upgrade(app)
+
+            # Update schema version to the latest
+            if CURRENT_SCHEMA_VERSION > schema_version:
+                version = Version.query.filter_by(name='ConfigDB').first()
+                version.value = CURRENT_SCHEMA_VERSION
+                db.session.commit()
 
     Mail(app)
 
@@ -271,19 +334,39 @@ def create_app(app_name=None):
     # Setup security
     ##########################################################################
     with app.app_context():
-        config.CSRF_SESSION_KEY = Keys.query.filter_by(name = 'CSRF_SESSION_KEY').first().value
-        config.SECRET_KEY = Keys.query.filter_by(name = 'SECRET_KEY').first().value
-        config.SECURITY_PASSWORD_SALT = Keys.query.filter_by(name = 'SECURITY_PASSWORD_SALT').first().value
+        config.CSRF_SESSION_KEY = Keys.query.filter_by(
+            name='CSRF_SESSION_KEY').first().value
+        config.SECRET_KEY = Keys.query.filter_by(
+            name='SECRET_KEY').first().value
+        config.SECURITY_PASSWORD_SALT = Keys.query.filter_by(
+            name='SECURITY_PASSWORD_SALT').first().value
 
     # Update the app.config with proper security keyes for signing CSRF data,
     # signing cookies, and the SALT for hashing the passwords.
-    app.config.update(dict(CSRF_SESSION_KEY=config.CSRF_SESSION_KEY))
-    app.config.update(dict(SECRET_KEY=config.SECRET_KEY))
-    app.config.update(dict(SECURITY_PASSWORD_SALT=config.SECURITY_PASSWORD_SALT))
+    app.config.update(dict({
+        'CSRF_SESSION_KEY': config.CSRF_SESSION_KEY,
+        'SECRET_KEY': config.SECRET_KEY,
+        'SECURITY_PASSWORD_SALT': config.SECURITY_PASSWORD_SALT,
+        'SESSION_COOKIE_DOMAIN': config.SESSION_COOKIE_DOMAIN
+    }))
 
-    security.init_app(app)
+    security.init_app(app, user_datastore)
 
-    app.session_interface = create_session_interface(app)
+    # register custom unauthorised handler.
+    app.login_manager.unauthorized_handler(pga_unauthorised)
+
+    # Set the permanent session lifetime to the specified value in config file.
+    app.permanent_session_lifetime = timedelta(
+        days=config.SESSION_EXPIRATION_TIME)
+
+    app.session_interface = create_session_interface(
+        app, config.SESSION_SKIP_PATHS
+    )
+
+    # Make the Session more secure against XSS & CSRF when running in web mode
+    if config.SERVER_MODE:
+        paranoid = Paranoid(app)
+        paranoid.redirect_view = 'browser.index'
 
     ##########################################################################
     # Load all available server drivers
@@ -310,13 +393,11 @@ def create_app(app_name=None):
             if user_languages and language:
                 language = user_languages.set(language)
 
-
     ##########################################################################
     # Register any local servers we can discover
     ##########################################################################
     @user_logged_in.connect_via(app)
     def on_user_logged_in(sender, user):
-
         # Keep hold of the user ID
         user_id = user.id
 
@@ -331,26 +412,28 @@ def create_app(app_name=None):
             servergroup_id = servergroup.id
 
         '''Add a server to the config database'''
-        def add_server(user_id, servergroup_id, name, superuser, port, discovery_id, comment):
-        # Create a server object if needed, and store it.
+
+        def add_server(user_id, servergroup_id, name, superuser, port,
+                       discovery_id, comment):
+            # Create a server object if needed, and store it.
             servers = Server.query.filter_by(
                 user_id=user_id,
                 discovery_id=svr_discovery_id
             ).order_by("id")
 
             if servers.count() > 0:
-                return;
+                return
 
             svr = Server(user_id=user_id,
-                            servergroup_id=servergroup_id,
-                            name=name,
-                            host='localhost',
-                            port=port,
-                            maintenance_db='postgres',
-                            username=superuser,
-                            ssl_mode='prefer',
-                            comment=svr_comment,
-                            discovery_id=discovery_id)
+                         servergroup_id=servergroup_id,
+                         name=name,
+                         host='localhost',
+                         port=port,
+                         maintenance_db='postgres',
+                         username=superuser,
+                         ssl_mode='prefer',
+                         comment=svr_comment,
+                         discovery_id=discovery_id)
 
             db.session.add(svr)
             db.session.commit()
@@ -362,7 +445,7 @@ def create_app(app_name=None):
 
             try:
                 proc_arch64 = os.environ['PROCESSOR_ARCHITEW6432'].lower()
-            except:
+            except Exception as e:
                 proc_arch64 = None
 
             if proc_arch == 'x86' and not proc_arch64:
@@ -392,7 +475,8 @@ def create_app(app_name=None):
                             svr_port = winreg.QueryValueEx(inst_key, 'Port')[0]
                             svr_discovery_id = inst_id
                             svr_comment = gettext(
-                                "Auto-detected %s installation with the data directory at %s" % (
+                                "Auto-detected %s installation with the data "
+                                "directory at %s" % (
                                     winreg.QueryValueEx(
                                         inst_key, 'Display Name'
                                     )[0],
@@ -409,7 +493,7 @@ def create_app(app_name=None):
                             )
 
                             inst_key.Close()
-                    except:
+                    except Exception as e:
                         pass
         else:
             # We use the postgres-winreg.ini file on non-Windows
@@ -426,25 +510,52 @@ def create_app(app_name=None):
 
             # Loop the sections, and get the data from any that are PG or PPAS
             for section in sections:
-                if section.startswith('PostgreSQL/') or section.startswith('EnterpriseDB/'):
+                if (
+                    section.startswith('PostgreSQL/') or
+                    section.startswith('EnterpriseDB/')
+                ):
                     svr_name = registry.get(section, 'Description')
                     svr_superuser = registry.get(section, 'Superuser')
-                    svr_port = registry.getint(section, 'Port')
+
+                    # getint function throws exception if value is blank.
+                    # Ex: Port=
+                    # In such case we should handle the exception and continue
+                    # to read the next section of the config file.
+                    try:
+                        svr_port = registry.getint(section, 'Port')
+                    except ValueError:
+                        continue
+
                     svr_discovery_id = section
                     description = registry.get(section, 'Description')
                     data_directory = registry.get(section, 'DataDirectory')
                     if hasattr(str, 'decode'):
                         description = description.decode('utf-8')
                         data_directory = data_directory.decode('utf-8')
-                    svr_comment = gettext(u"Auto-detected %s installation with the data directory at %s" % (
-                        description,
-                        data_directory
-                    ))
-                    add_server(user_id, servergroup_id, svr_name, svr_superuser, svr_port, svr_discovery_id, svr_comment)
+                    svr_comment = gettext(u"Auto-detected %s installation "
+                                          u"with the data directory at %s" % (
+                                              description,
+                                              data_directory
+                                          )
+                                          )
+                    add_server(user_id, servergroup_id, svr_name,
+                               svr_superuser, svr_port, svr_discovery_id,
+                               svr_comment)
 
-        except:
+        except Exception as e:
             pass
 
+    @user_logged_in.connect_via(app)
+    @user_logged_out.connect_via(app)
+    def force_session_write(app, user):
+        session.force_write = True
+
+    @user_logged_out.connect_via(app)
+    def clear_current_user_connections(app, user):
+        from config import PG_DEFAULT_DRIVER
+        from pgadmin.utils.driver import get_driver
+        _driver = get_driver(PG_DEFAULT_DRIVER)
+        _driver.gc_own()
 
     ##########################################################################
     # Load plugin modules
@@ -465,39 +576,80 @@ def create_app(app_name=None):
         # mode, and it's not a help file request.
         if not config.SERVER_MODE and app.PGADMIN_KEY != '':
             if (
-                (not 'key' in request.args or request.args['key'] != app.PGADMIN_KEY) and
+                ('key' not in request.args or
+                 request.args['key'] != app.PGADMIN_KEY) and
                 request.cookies.get('PGADMIN_KEY') != app.PGADMIN_KEY and
                 request.endpoint != 'help.static'
             ):
                 abort(401)
 
-        if not config.SERVER_MODE:
+        if not config.SERVER_MODE and not current_user.is_authenticated:
             user = user_datastore.get_user(config.DESKTOP_USER)
-
             # Throw an error if we failed to find the desktop user, to give
             # the sysadmin a hint. We'll continue to try to login anyway as
             # that'll through a nice 500 error for us.
             if user is None:
                 app.logger.error(
-                    'The desktop user %s was not found in the configuration database.'
+                    'The desktop user %s was not found in the configuration '
+                    'database.'
                     % config.DESKTOP_USER
                 )
                 abort(401)
-
             login_user(user)
 
     @app.after_request
     def after_request(response):
         if 'key' in request.args:
-            response.set_cookie('PGADMIN_KEY', value=request.args['key'])
+            domain = dict()
+            if config.COOKIE_DEFAULT_DOMAIN and \
+                    config.COOKIE_DEFAULT_DOMAIN != 'localhost':
+                domain['domain'] = config.COOKIE_DEFAULT_DOMAIN
+            response.set_cookie('PGADMIN_KEY', value=request.args['key'],
+                                path=config.COOKIE_DEFAULT_PATH,
+                                **domain)
 
         return response
 
+    ##########################################################################
+    # Cache busting
+    ##########################################################################
+
+    # Version number to be added to all static file url requests
+    # This is used by url_for function when generating urls
+    # This will solve caching issues when application is upgrading
+    # This is called - Cache Busting
+    @app.url_defaults
+    def add_internal_version(endpoint, values):
+        extensions = config.APP_VERSION_EXTN
+
+        # Add the internal version only if it is set
+        if config.APP_VERSION_PARAM is not None and \
+           config.APP_VERSION_PARAM != '':
+            # If there is a filename, add the version
+            if 'filename' in values \
+               and values['filename'].endswith(extensions):
+                values[config.APP_VERSION_PARAM] = config.APP_VERSION_INT
+            else:
+                # Sometimes there may be direct endpoint for some files
+                # There will be only one rule for such endpoints
+                urls = [url for url in app.url_map.iter_rules(endpoint)]
+                if len(urls) == 1 and urls[0].rule.endswith(extensions):
+                    values[config.APP_VERSION_PARAM] = \
+                        config.APP_VERSION_INT
+
+    # Strip away internal version param before sending further to app as it was
+    # required for cache busting only
+    @app.url_value_preprocessor
+    def strip_version_number(endpoint, values):
+        if values and config.APP_VERSION_PARAM in values:
+            values.pop(config.APP_VERSION_PARAM)
+
     ##########################################################################
     # Minify output
     ##########################################################################
     # HTMLMIN doesn't work with Python 2.6.
-    if not config.DEBUG and sys.version_info >= (2,7):
+    if not config.DEBUG and sys.version_info >= (2, 7):
+        from flask_htmlmin import HTMLMIN
         HTMLMIN(app)
 
     @app.context_processor
@@ -511,6 +663,5 @@ def create_app(app_name=None):
     ##########################################################################
     # All done!
     ##########################################################################
-    app.logger.debug('URL map: %s' % app.url_map)
 
     return app
