@@ -12,7 +12,7 @@ import re
 import pgadmin.browser.server_groups as sg
 from flask import render_template, request, make_response, jsonify, \
     current_app, url_for
-from flask_babel import gettext
+from flask_babelex import gettext
 from flask_security import current_user, login_required
 from pgadmin.browser.server_groups.servers.types import ServerType
 from pgadmin.browser.utils import PGChildNodeView
@@ -26,6 +26,7 @@ from config import PG_DEFAULT_DRIVER
 from pgadmin.model import db, Server, ServerGroup, User
 from pgadmin.utils.driver import get_driver
 
+
 def has_any(data, keys):
     """
     Checks any one of the keys present in the data given
@@ -42,8 +43,10 @@ def has_any(data, keys):
 
     return False
 
+
 def recovery_state(connection, postgres_version):
-    recovery_check_sql = render_template("connect/sql/#{0}#/check_recovery.sql".format(postgres_version))
+    recovery_check_sql = render_template(
+        "connect/sql/#{0}#/check_recovery.sql".format(postgres_version))
 
     status, result = connection.execute_dict(recovery_check_sql)
     if status and 'rows' in result and len(result['rows']) > 0:
@@ -54,6 +57,7 @@ def recovery_state(connection, postgres_version):
         wal_paused = None
     return in_recovery, wal_paused
 
+
 def server_icon_and_background(is_connected, manager, server):
     """
 
@@ -133,7 +137,11 @@ class ServerModule(sg.ServerGroupPluginModule):
                 db=manager.db,
                 user=manager.user_info if connected else None,
                 in_recovery=in_recovery,
-                wal_pause=wal_paused
+                wal_pause=wal_paused,
+                is_password_saved=True if server.password is not None
+                else False,
+                is_tunnel_password_saved=True
+                if server.tunnel_password is not None else False
             )
 
     @property
@@ -160,16 +168,16 @@ class ServerModule(sg.ServerGroupPluginModule):
 
         scripts.extend([{
             'name': 'pgadmin.browser.server.privilege',
-            'path': url_for('%s.static'% self.name, filename='js/privilege'),
+            'path': url_for('%s.static' % self.name, filename='js/privilege'),
             'when': self.node_type,
             'is_template': False,
             'deps': ['pgadmin.browser.node.ui']
         }, {
             'name': 'pgadmin.browser.server.variable',
-            'path': url_for('%s.static'% self.name, filename='js/variable'),
+            'path': url_for('%s.static' % self.name, filename='js/variable'),
             'when': self.node_type,
             'is_template': False
-        },{
+        }, {
             'name': 'pgadmin.server.supported_servers',
             'path': url_for('browser.index') + 'server/supported_servers',
             'is_template': True,
@@ -202,6 +210,9 @@ class ServerModule(sg.ServerGroupPluginModule):
         """
         ServerType.register_preferences()
 
+    def get_exposed_url_endpoints(self):
+        return ['NODE-server.connect_id']
+
 
 class ServerMenuItem(MenuItem):
     def __init__(self, **kwargs):
@@ -241,28 +252,36 @@ class ServerNode(PGChildNodeView):
         'wal_replay': [{
             'delete': 'pause_wal_replay', 'put': 'resume_wal_replay'
         }],
-        'check_pgpass': [{'get': 'check_pgpass'}]
+        'check_pgpass': [{'get': 'check_pgpass'}],
+        'clear_saved_password': [{'put': 'clear_saved_password'}],
+        'clear_sshtunnel_password': [{'put': 'clear_sshtunnel_password'}]
     })
-    EXP_IP4 = "^\s*((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."\
-            "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."\
-            "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."\
-            "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))\s*$"
-    EXP_IP6 = '^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|'\
-           '(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|'\
-           '2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|'\
-           '(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|'\
-           ':((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|'\
-           '(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|'\
-           '2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|'\
-           '(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|'\
-           '[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|'\
-           '((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|'\
-           '(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|'\
-           '1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|'\
-           '((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$'
+    EXP_IP4 = "^\s*((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\." \
+              "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\." \
+              "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\." \
+              "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))\s*$"
+    EXP_IP6 = '^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|' \
+              '(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|' \
+              '2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d))' \
+              '{3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|' \
+              ':((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d' \
+              '|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]' \
+              '{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|' \
+              '[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|' \
+              '(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-' \
+              'Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25' \
+              '[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:)' \
+              '{2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:(' \
+              '(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|' \
+              '[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]' \
+              '{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|'\
+              '1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))' \
+              '|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((' \
+              '25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|' \
+              '[1-9]?\d)){3}))|:)))(%.+)?\s*$'
     pat4 = re.compile(EXP_IP4)
     pat6 = re.compile(EXP_IP6)
-    SSL_MODES = ['prefer','require', 'verify-ca', 'verify-full']
+    SSL_MODES = ['prefer', 'require', 'verify-ca', 'verify-full']
 
     def check_ssl_fields(self, data):
         """
@@ -292,13 +311,13 @@ class ServerNode(PGChildNodeView):
                     # prevent using default SSL certificates from web servers
 
                     if config.SERVER_MODE and \
-                                    field in required_ssl_fields_server_mode:
+                            field in required_ssl_fields_server_mode:
                         # Set file manager directory from preference
                         import os
                         file_extn = '.key' if field.endswith('key') else '.crt'
                         dummy_ssl_file = os.path.join(
-                                '<STORAGE_DIR>', '.postgresql',
-                                'postgresql' + file_extn
+                            '<STORAGE_DIR>', '.postgresql',
+                            'postgresql' + file_extn
                         )
                         data[field] = dummy_ssl_file
                     # For Desktop mode, we will allow to default
@@ -344,14 +363,18 @@ class ServerNode(PGChildNodeView):
                     db=manager.db,
                     user=manager.user_info if connected else None,
                     in_recovery=in_recovery,
-                    wal_pause=wal_paused
+                    wal_pause=wal_paused,
+                    is_password_saved=True if server.password is not None
+                    else False,
+                    is_tunnel_password_saved=True
+                    if server.tunnel_password is not None else False
                 )
             )
 
         if not len(res):
             return gone(errormsg=gettext(
                 'The specified server group with id# {0} could not be found.'
-                ))
+            ))
 
         return make_json_response(result=res)
 
@@ -397,7 +420,11 @@ class ServerNode(PGChildNodeView):
                 db=manager.db,
                 user=manager.user_info if connected else None,
                 in_recovery=in_recovery,
-                wal_pause=wal_paused
+                wal_pause=wal_paused,
+                is_password_saved=True if server.password is not None
+                else False,
+                is_tunnel_password_saved=True
+                if server.tunnel_password is not None else False
             )
         )
 
@@ -465,7 +492,15 @@ class ServerNode(PGChildNodeView):
             'sslcrl': 'sslcrl',
             'sslcompression': 'sslcompression',
             'bgcolor': 'bgcolor',
-            'fgcolor': 'fgcolor'
+            'fgcolor': 'fgcolor',
+            'service': 'service',
+            'connect_timeout': 'connect_timeout',
+            'use_ssh_tunnel': 'use_ssh_tunnel',
+            'tunnel_host': 'tunnel_host',
+            'tunnel_port': 'tunnel_port',
+            'tunnel_username': 'tunnel_username',
+            'tunnel_authentication': 'tunnel_authentication',
+            'tunnel_identity_file': 'tunnel_identity_file',
         }
 
         disp_lbl = {
@@ -486,13 +521,13 @@ class ServerNode(PGChildNodeView):
         if 'db_res' in data:
             data['db_res'] = ','.join(data['db_res'])
 
-        if 'hostaddr' in data and data['hostaddr'] != '':
+        if 'hostaddr' in data and data['hostaddr'] and data['hostaddr'] != '':
             if not self.pat4.match(data['hostaddr']):
                 if not self.pat6.match(data['hostaddr']):
                     return make_json_response(
-                    success=0,
-                    status=400,
-                    errormsg=gettext('Host address not valid')
+                        success=0,
+                        status=400,
+                        errormsg=gettext('Host address not valid')
                     )
 
         manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(sid)
@@ -501,12 +536,14 @@ class ServerNode(PGChildNodeView):
 
         if connected:
             for arg in (
-                    'host', 'hostaddr', 'port', 'db', 'username', 'sslmode', 'role'
+                    'host', 'hostaddr', 'port', 'db', 'username', 'sslmode',
+                    'role', 'service'
             ):
                 if arg in data:
                     return forbidden(
                         errormsg=gettext(
-                            "'{0}' is not allowed to modify, when server is connected."
+                            "'{0}' is not allowed to modify, "
+                            "when server is connected."
                         ).format(disp_lbl[arg])
                     )
 
@@ -548,7 +585,8 @@ class ServerNode(PGChildNodeView):
                 server_icon_and_background(connected, manager, server),
                 True,
                 self.node_type,
-                connected=False,
+                connected=connected,
+                user=manager.user_info if connected else None,
                 server_type='pg'  # default server type
             )
         )
@@ -648,7 +686,22 @@ class ServerNode(PGChildNodeView):
                 'sslrootcert': server.sslrootcert if is_ssl else None,
                 'sslcrl': server.sslcrl if is_ssl else None,
                 'sslcompression': True if is_ssl and server.sslcompression
-                else False
+                else False,
+                'service': server.service if server.service else None,
+                'connect_timeout':
+                    server.connect_timeout if server.connect_timeout else 0,
+                'use_ssh_tunnel': server.use_ssh_tunnel
+                if server.use_ssh_tunnel else 0,
+                'tunnel_host': server.tunnel_host if server.tunnel_host
+                else None,
+                'tunnel_port': server.tunnel_port if server.tunnel_port
+                else 22,
+                'tunnel_username': server.tunnel_username
+                if server.tunnel_username else None,
+                'tunnel_identity_file': server.tunnel_identity_file
+                if server.tunnel_identity_file else None,
+                'tunnel_authentication': server.tunnel_authentication
+                if server.tunnel_authentication else 0
             }
         )
 
@@ -657,18 +710,22 @@ class ServerNode(PGChildNodeView):
         """Add a server node to the settings database"""
         required_args = [
             u'name',
-            u'host',
             u'port',
-            u'db',
-            u'username',
             u'sslmode',
-            u'role'
+            u'username'
         ]
 
         data = request.form if request.form else json.loads(
             request.data, encoding='utf-8'
         )
 
+        # Some fields can be provided with service file so they are optional
+        if 'service' in data and not data['service']:
+            required_args.extend([
+                u'host',
+                u'db',
+                u'role'
+            ])
         for arg in required_args:
             if arg not in data:
                 return make_json_response(
@@ -679,7 +736,7 @@ class ServerNode(PGChildNodeView):
                     )
                 )
 
-        if 'hostaddr' in data and data['hostaddr'] != '':
+        if 'hostaddr' in data and data['hostaddr'] and data['hostaddr'] != '':
             if not self.pat4.match(data['hostaddr']):
                 if not self.pat6.match(data['hostaddr']):
                     return make_json_response(
@@ -696,27 +753,33 @@ class ServerNode(PGChildNodeView):
         try:
             server = Server(
                 user_id=current_user.id,
-                servergroup_id=data[u'gid'] if u'gid' in data else gid,
-                name=data[u'name'],
-                host=data[u'host'],
-                hostaddr=data[u'hostaddr'] if u'hostaddr' in data else None,
-                port=data[u'port'],
-                maintenance_db=data[u'db'],
-                username=data[u'username'],
-                ssl_mode=data[u'sslmode'],
-                comment=data[u'comment'] if u'comment' in data else None,
-                role=data[u'role'] if u'role' in data else None,
-                db_res=','.join(data[u'db_res']) if u'db_res' in data else None,
-                sslcert=data['sslcert'] if is_ssl else None,
-                sslkey=data['sslkey'] if is_ssl else None,
-                sslrootcert=data['sslrootcert'] if is_ssl else None,
-                sslcrl=data['sslcrl'] if is_ssl else None,
+                servergroup_id=data.get('gid', gid),
+                name=data.get('name'),
+                host=data.get('host', None),
+                hostaddr=data.get('hostaddr', None),
+                port=data.get('port'),
+                maintenance_db=data.get('db', None),
+                username=data.get('username'),
+                ssl_mode=data.get('sslmode'),
+                comment=data.get('comment', None),
+                role=data.get('role', None),
+                db_res=','.join(data[u'db_res'])
+                if u'db_res' in data else None,
+                sslcert=data.get('sslcert', None),
+                sslkey=data.get('sslkey', None),
+                sslrootcert=data.get('sslrootcert', None),
+                sslcrl=data.get('sslcrl', None),
                 sslcompression=1 if is_ssl and data['sslcompression'] else 0,
-                bgcolor=data['bgcolor'] if u'bgcolor' in data
-                else None,
-                fgcolor = data['fgcolor'] if u'fgcolor' in data
-                else None
-
+                bgcolor=data.get('bgcolor', None),
+                fgcolor=data.get('fgcolor', None),
+                service=data.get('service', None),
+                connect_timeout=data.get('connect_timeout', 0),
+                use_ssh_tunnel=data.get('use_ssh_tunnel', 0),
+                tunnel_host=data.get('tunnel_host', None),
+                tunnel_port=data.get('tunnel_port', 22),
+                tunnel_username=data.get('tunnel_username', None),
+                tunnel_authentication=data.get('tunnel_authentication', 0),
+                tunnel_identity_file=data.get('tunnel_identity_file', None)
             )
             db.session.add(server)
             db.session.commit()
@@ -726,13 +789,16 @@ class ServerNode(PGChildNodeView):
             manager = None
 
             if 'connect_now' in data and data['connect_now']:
-                manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(server.id)
+                manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(
+                    server.id)
                 manager.update(server)
                 conn = manager.connection()
 
                 have_password = False
+                have_tunnel_password = False
                 password = None
                 passfile = None
+                tunnel_password = ''
                 if 'password' in data and data["password"] != '':
                     # login with password
                     have_password = True
@@ -743,9 +809,16 @@ class ServerNode(PGChildNodeView):
                     setattr(server, 'passfile', passfile)
                     db.session.commit()
 
+                if 'tunnel_password' in data and data["tunnel_password"] != '':
+                    have_tunnel_password = True
+                    tunnel_password = data['tunnel_password']
+                    tunnel_password = \
+                        encrypt(tunnel_password, current_user.password)
+
                 status, errmsg = conn.connect(
                     password=password,
                     passfile=passfile,
+                    tunnel_password=tunnel_password,
                     server_types=ServerType.types()
                 )
                 if hasattr(str, 'decode') and errmsg is not None:
@@ -756,7 +829,8 @@ class ServerNode(PGChildNodeView):
                     return make_json_response(
                         status=401,
                         success=0,
-                        errormsg=gettext(u"Unable to connect to server:\n\n%s" % errmsg)
+                        errormsg=gettext(
+                            u"Unable to connect to server:\n\n%s" % errmsg)
                     )
                 else:
                     if 'save_password' in data and data['save_password'] and \
@@ -764,6 +838,13 @@ class ServerNode(PGChildNodeView):
                         setattr(server, 'password', password)
                         db.session.commit()
 
+                    if 'save_tunnel_password' in data and \
+                        data['save_tunnel_password'] and \
+                        have_tunnel_password and \
+                            config.ALLOW_SAVE_TUNNEL_PASSWORD:
+                        setattr(server, 'tunnel_password', tunnel_password)
+                        db.session.commit()
+
                     user = manager.user_info
                     connected = True
 
@@ -776,7 +857,9 @@ class ServerNode(PGChildNodeView):
                     self.node_type,
                     user=user,
                     connected=connected,
-                    server_type=manager.server_type if manager and manager.server_type else 'pg'
+                    server_type=manager.server_type
+                    if manager and manager.server_type
+                    else 'pg'
                 )
             )
 
@@ -853,10 +936,11 @@ class ServerNode(PGChildNodeView):
         res = conn.connected()
 
         if res:
-            from pgadmin.utils.exception import ConnectionLost
+            from pgadmin.utils.exception import ConnectionLost, \
+                SSHTunnelConnectionLost
             try:
                 conn.execute_scalar('SELECT 1')
-            except ConnectionLost:
+            except (ConnectionLost, SSHTunnelConnectionLost):
                 res = False
 
         return make_json_response(data={'connected': res})
@@ -886,9 +970,12 @@ class ServerNode(PGChildNodeView):
         if server is None:
             return bad_request(gettext("Server not found."))
 
-        # Fetch User Details.
-        user = User.query.filter_by(id=current_user.id).first()
-        if user is None:
+        if current_user and hasattr(current_user, 'id'):
+            # Fetch User Details.
+            user = User.query.filter_by(id=current_user.id).first()
+            if user is None:
+                return unauthorized(gettext("Unauthorized request."))
+        else:
             return unauthorized(gettext("Unauthorized request."))
 
         data = request.form if request.form else json.loads(
@@ -897,39 +984,55 @@ class ServerNode(PGChildNodeView):
 
         password = None
         passfile = None
+        tunnel_password = None
         save_password = False
+        save_tunnel_password = False
+        prompt_password = False
+        prompt_tunnel_password = False
 
         # Connect the Server
         manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(sid)
         conn = manager.connection()
 
+        # If server using SSH Tunnel
+        if server.use_ssh_tunnel:
+            if 'tunnel_password' not in data:
+                if server.tunnel_password is None:
+                    prompt_tunnel_password = True
+                else:
+                    tunnel_password = server.tunnel_password
+            else:
+                tunnel_password = data['tunnel_password'] \
+                    if 'tunnel_password'in data else ''
+                save_tunnel_password = data['save_tunnel_password'] \
+                    if tunnel_password and 'save_tunnel_password' in data \
+                    else False
+                # Encrypt the password before saving with user's login
+                # password key.
+                try:
+                    tunnel_password = encrypt(tunnel_password, user.password) \
+                        if tunnel_password is not None else \
+                        server.tunnel_password
+                except Exception as e:
+                    current_app.logger.exception(e)
+                    return internal_server_error(errormsg=e.message)
+
         if 'password' not in data:
             conn_passwd = getattr(conn, 'password', None)
             if conn_passwd is None and server.password is None and \
-                    server.passfile is None:
-                # Return the password template in case password is not
-                # provided, or password has not been saved earlier.
-                return make_json_response(
-                    success=0,
-                    status=428,
-                    result=render_template(
-                        'servers/password.html',
-                        server_label=server.name,
-                        username=server.username,
-                        _=gettext
-                    )
-                )
+                    server.passfile is None and server.service is None:
+                prompt_password = True
             elif server.passfile and server.passfile != '':
                 passfile = server.passfile
             else:
                 password = conn_passwd or server.password
         else:
             password = data['password'] if 'password' in data else None
-            save_password = \
-                data['save_password'] if password and \
-                                         'save_password' in data else False
+            save_password = data['save_password']\
+                if password and 'save_password' in data else False
 
-            # Encrypt the password before saving with user's login password key.
+            # Encrypt the password before saving with user's login
+            # password key.
             try:
                 password = encrypt(password, user.password) \
                     if password is not None else server.password
@@ -937,49 +1040,36 @@ class ServerNode(PGChildNodeView):
                 current_app.logger.exception(e)
                 return internal_server_error(errormsg=e.message)
 
+        # Check do we need to prompt for the database server or ssh tunnel
+        # password or both. Return the password template in case password is
+        # not provided, or password has not been saved earlier.
+        if prompt_password or prompt_tunnel_password:
+            return self.get_response_for_password(server, 428, prompt_password,
+                                                  prompt_tunnel_password)
+
         status = True
         try:
             status, errmsg = conn.connect(
                 password=password,
                 passfile=passfile,
+                tunnel_password=tunnel_password,
                 server_types=ServerType.types()
             )
         except Exception as e:
             current_app.logger.exception(e)
-
-            return make_json_response(
-                success=0,
-                status=401,
-                result=render_template(
-                    'servers/password.html',
-                    server_label=server.name,
-                    username=server.username,
-                    errmsg=getattr(e, 'message', str(e)),
-                    _=gettext
-                )
-            )
+            return self.get_response_for_password(
+                server, 401, True, True, getattr(e, 'message', str(e)))
 
         if not status:
             if hasattr(str, 'decode'):
                 errmsg = errmsg.decode('utf-8')
 
             current_app.logger.error(
-                "Could not connected to server(#{0}) - '{1}'.\nError: {2}".format(
-                    server.id, server.name, errmsg
-                )
-            )
-
-            return make_json_response(
-                success=0,
-                status=401,
-                result=render_template(
-                    'servers/password.html',
-                    server_label=server.name,
-                    username=server.username,
-                    errmsg=errmsg,
-                    _=gettext
-                )
+                "Could not connected to server(#{0}) - '{1}'.\nError: {2}"
+                .format(server.id, server.name, errmsg)
             )
+            return self.get_response_for_password(server, 401, True,
+                                                  True, errmsg)
         else:
             if save_password and config.ALLOW_SAVE_PASSWORD:
                 try:
@@ -995,9 +1085,23 @@ class ServerNode(PGChildNodeView):
 
                     return internal_server_error(errormsg=e.message)
 
+            if save_tunnel_password and config.ALLOW_SAVE_TUNNEL_PASSWORD:
+                try:
+                    # Save the encrypted tunnel password.
+                    setattr(server, 'tunnel_password', tunnel_password)
+                    db.session.commit()
+                except Exception as e:
+                    # Release Connection
+                    current_app.logger.exception(e)
+                    manager.release(database=server.maintenance_db)
+                    conn = None
+
+                    return internal_server_error(errormsg=e.message)
+
             current_app.logger.info('Connection Established for server: \
                 %s - %s' % (server.id, server.name))
-            # Update the recovery and wal pause option for the server if connected successfully
+            # Update the recovery and wal pause option for the server
+            # if connected successfully
             in_recovery, wal_paused = recovery_state(conn, manager.version)
 
             return make_json_response(
@@ -1012,7 +1116,11 @@ class ServerNode(PGChildNodeView):
                     'db': manager.db,
                     'user': manager.user_info,
                     'in_recovery': in_recovery,
-                    'wal_pause': wal_paused
+                    'wal_pause': wal_paused,
+                    'is_password_saved': True if server.password is not None
+                    else False,
+                    'is_tunnel_password_saved': True
+                    if server.tunnel_password is not None else False,
                 }
             )
 
@@ -1056,13 +1164,17 @@ class ServerNode(PGChildNodeView):
                     gettext("Could not reload the server configuration.")
                 )
             else:
-                return make_json_response(data={'status': True,
-                                                'result': gettext('Server configuration reloaded.')})
+                return make_json_response(data={
+                    'status': True,
+                    'result': gettext('Server configuration reloaded.')
+                })
 
         else:
-            return make_json_response(data={'status': False,
-                                            'result': gettext(
-                                                'Not connected to the server or the connection to the server has been closed.')})
+            return make_json_response(data={
+                'status': False,
+                'result': gettext(
+                    'Not connected to the server or the connection to the'
+                    ' server has been closed.')})
 
     def create_restore_point(self, gid, sid):
         """
@@ -1137,8 +1249,9 @@ class ServerNode(PGChildNodeView):
             # If there is no password found for the server
             # then check for pgpass file
             if not server.password and not manager.password:
-                if server.passfile and manager.passfile and \
-                    server.passfile == manager.passfile:
+                if server.passfile and \
+                        manager.passfile and \
+                        server.passfile == manager.passfile:
                     is_passfile = True
 
             # Check for password only if there is no pgpass file used
@@ -1187,10 +1300,20 @@ class ServerNode(PGChildNodeView):
                     return unauthorized(gettext("Incorrect password."))
 
             # Hash new password before saving it.
-            password = pqencryptpassword(data['newPassword'], manager.user)
+            if manager.sversion >= 100000:
+                password = conn.pq_encrypt_password_conn(data['newPassword'],
+                                                         manager.user)
+                if password is None:
+                    # Unable to encrypt the password so used the
+                    # old method of encryption
+                    password = pqencryptpassword(data['newPassword'],
+                                                 manager.user)
+            else:
+                password = pqencryptpassword(data['newPassword'], manager.user)
 
             SQL = render_template(
-                "/servers/sql/#{0}#/change_password.sql".format(manager.version),
+                "/servers/sql/#{0}#/change_password.sql".format(
+                    manager.version),
                 conn=conn, _=gettext,
                 user=manager.user, encrypted_password=password)
 
@@ -1329,9 +1452,10 @@ class ServerNode(PGChildNodeView):
                 )
 
             if not server.password or not manager.password:
-                if server.passfile and manager.passfile and \
-                    server.passfile == manager.passfile:
-                        is_pgpass = True
+                if server.passfile and \
+                        manager.passfile and \
+                        server.passfile == manager.passfile:
+                    is_pgpass = True
             return make_json_response(
                 success=1,
                 data=dict({'is_pgpass': is_pgpass}),
@@ -1342,4 +1466,108 @@ class ServerNode(PGChildNodeView):
             )
             return internal_server_error(errormsg=str(e))
 
+    def get_response_for_password(self, server, status, prompt_password=False,
+                                  prompt_tunnel_password=False, errmsg=None):
+        if server.use_ssh_tunnel:
+            return make_json_response(
+                success=0,
+                status=status,
+                result=render_template(
+                    'servers/tunnel_password.html',
+                    server_label=server.name,
+                    username=server.username,
+                    tunnel_username=server.tunnel_username,
+                    tunnel_host=server.tunnel_host,
+                    tunnel_identity_file=server.tunnel_identity_file,
+                    errmsg=errmsg,
+                    _=gettext,
+                    prompt_tunnel_password=prompt_tunnel_password,
+                    prompt_password=prompt_password
+                )
+            )
+        else:
+            return make_json_response(
+                success=0,
+                status=status,
+                result=render_template(
+                    'servers/password.html',
+                    server_label=server.name,
+                    username=server.username,
+                    errmsg=errmsg,
+                    _=gettext
+                )
+            )
+
+    def clear_saved_password(self, gid, sid):
+        """
+        This function is used to remove database server password stored into
+        the pgAdmin's db file.
+
+        :param gid:
+        :param sid:
+        :return:
+        """
+        try:
+            server = Server.query.filter_by(
+                user_id=current_user.id, id=sid
+            ).first()
+
+            if server is None:
+                return make_json_response(
+                    success=0,
+                    info=gettext("Could not find the required server.")
+                )
+
+            setattr(server, 'password', None)
+            db.session.commit()
+        except Exception as e:
+            current_app.logger.error(
+                "Unable to clear saved password.\nError: {0}".format(str(e))
+            )
+
+            return internal_server_error(errormsg=str(e))
+
+        return make_json_response(
+            success=1,
+            info=gettext("The saved password cleared successfully."),
+            data={'is_password_saved': False}
+        )
+
+    def clear_sshtunnel_password(self, gid, sid):
+        """
+        This function is used to remove sshtunnel password stored into
+        the pgAdmin's db file.
+
+        :param gid:
+        :param sid:
+        :return:
+        """
+        try:
+            server = Server.query.filter_by(
+                user_id=current_user.id, id=sid
+            ).first()
+
+            if server is None:
+                return make_json_response(
+                    success=0,
+                    info=gettext("Could not find the required server.")
+                )
+
+            setattr(server, 'tunnel_password', None)
+            db.session.commit()
+        except Exception as e:
+            current_app.logger.error(
+                "Unable to clear ssh tunnel password."
+                "\nError: {0}".format(str(e))
+            )
+
+            return internal_server_error(errormsg=str(e))
+
+        return make_json_response(
+            success=1,
+            info=gettext("The saved password cleared successfully."),
+            data={'is_tunnel_password_saved': False}
+        )
+
+
 ServerNode.register_node_view(blueprint)
