@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,7 +14,7 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as databases
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, \
@@ -60,6 +60,7 @@ class ExtensionModule(CollectionNodeModule):
         Initialising the base class
         """
         super(ExtensionModule, self).__init__(*args, **kwargs)
+        self.min_gpdbver = 1000000000
 
     def get_nodes(self, gid, sid, did):
         """
@@ -77,8 +78,8 @@ class ExtensionModule(CollectionNodeModule):
     @property
     def script_load(self):
         """
-        Load the module script for extension, when any of the database nodes are
-        initialized.
+        Load the module script for extension, when any of the database nodes
+        are initialized.
         """
         return databases.DatabaseModule.NODE_TYPE
 
@@ -122,9 +123,9 @@ class ExtensionView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
         'msql': [{'get': 'msql'}, {'get': 'msql'}],
@@ -326,49 +327,53 @@ class ExtensionView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, eid):
+    def delete(self, gid, sid, did, eid=None):
         """
         This function will drop/drop cascade a extension object
         """
+
+        if eid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [eid]}
+
         cascade = True if self.cmd == 'delete' else False
+
         try:
-            # check if extension with eid exists
-            SQL = render_template("/".join(
-                [self.template_path, 'delete.sql']), eid=eid)
-            status, name = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=name)
+            for eid in data['ids']:
+                # check if extension with eid exists
+                SQL = render_template("/".join(
+                    [self.template_path, 'delete.sql']), eid=eid)
+                status, name = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=name)
 
-            if name is None:
-                return make_json_response(
-                    status=410,
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified extension could not be found.\n'
+                if name is None:
+                    return make_json_response(
+                        status=410,
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified extension could not be found.\n'
+                        )
                     )
-                )
 
-            # drop extension
-            SQL = render_template("/".join(
-                [self.template_path, 'delete.sql']
-            ), name=name, cascade=cascade)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                # drop extension
+                SQL = render_template("/".join(
+                    [self.template_path, 'delete.sql']
+                ), name=name, cascade=cascade)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Extension dropped"),
-                data={
-                    'id': did,
-                    'sid': sid,
-                    'gid': gid,
-                }
+                info=gettext("Extension dropped")
             )
-
         except Exception as e:
             return internal_server_error(errormsg=str(e))
 
