@@ -2,18 +2,20 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-import simplejson as json
 import re
 from functools import wraps
 
-import pgadmin.browser.server_groups.servers as servers
+import simplejson as json
 from flask import render_template, request, jsonify, current_app
-from flask_babel import gettext
+from flask_babelex import gettext
+
+import pgadmin.browser.server_groups.servers as servers
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.collection import CollectionNodeModule, PGChildModule
 from pgadmin.browser.server_groups.servers.utils import parse_priv_from_db, \
     parse_priv_to_db
@@ -22,8 +24,6 @@ from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone, bad_request
 from pgadmin.utils.driver import get_driver
 
-from config import PG_DEFAULT_DRIVER
-
 """
     This module is responsible for generating two nodes
     1) Schema
@@ -139,11 +139,13 @@ def check_precondition(f):
 
         self.conn = self.manager.connection(did=kwargs['did'])
         # Set the template path for the SQL scripts
-        self.template_path = self.template_initial + '/' + (
-            self.ppas_template_path(self.manager.version)
-            if self.manager.server_type == 'ppas' else
-            self.pg_template_path(self.manager.version)
-        )
+        if self.manager.server_type == 'gpdb':
+            _temp = self.gpdb_template_path(self.manager.version)
+        elif self.manager.server_type == 'ppas':
+            _temp = self.ppas_template_path(self.manager.version)
+        else:
+            _temp = self.pg_template_path(self.manager.version)
+        self.template_path = self.template_initial + '/' + _temp
 
         return f(*args, **kwargs)
 
@@ -212,7 +214,7 @@ class SchemaView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'nodes'}, {'get': 'nodes'}],
@@ -221,7 +223,8 @@ class SchemaView(PGChildNodeView):
         'stats': [{'get': 'statistics'}],
         'dependency': [{'get': 'dependencies'}],
         'dependent': [{'get': 'dependents'}],
-        'delete': [{'delete': 'delete'}]
+        'delete': [{'delete': 'delete'},
+                   {'delete': 'delete'}]
     })
 
     def __init__(self, *args, **kwargs):
@@ -250,6 +253,13 @@ class SchemaView(PGChildNodeView):
         """
         return 'pg/#{0}#'.format(ver)
 
+    @staticmethod
+    def gpdb_template_path(ver):
+        """
+        Returns the template path for GreenPlum servers.
+        """
+        return '#gpdb#{0}#'.format(ver)
+
     def format_request_acls(self, data, modified=False, specific=None):
         acls = {}
         try:
@@ -274,7 +284,8 @@ class SchemaView(PGChildNodeView):
                                 data[aclcol][modifier], allowedacl['acl']
                             )
                 else:
-                    data[aclcol] = parse_priv_to_db(data[aclcol], allowedacl['acl'])
+                    data[aclcol] = parse_priv_to_db(data[aclcol],
+                                                    allowedacl['acl'])
 
         return acls
 
@@ -357,7 +368,8 @@ class SchemaView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid, did):
         """
-        This function is used to list all the schema nodes within the collection.
+        This function is used to list all the schema nodes within the
+        collection.
 
         Args:
             gid: Server group ID
@@ -482,13 +494,13 @@ It may have been removed by another user.
         for row in rset['rows']:
             return make_json_response(
                 data=self.blueprint.generate_browser_node(
-                        row['oid'],
-                        did,
-                        row['name'],
-                        icon=icon,
-                        can_create=row['can_create'],
-                        has_usage=row['has_usage']
-                    ),
+                    row['oid'],
+                    did,
+                    row['name'],
+                    icon=icon,
+                    can_create=row['can_create'],
+                    has_usage=row['has_usage']
+                ),
                 status=200
             )
 
@@ -519,9 +531,10 @@ It may have been removed by another user.
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(
-                gettext("Could not find the schema in the database. It may have been removed by another user."
-                ))
+            return gone(gettext(
+                "Could not find the schema in the database. "
+                "It may have been removed by another user."
+            ))
 
         # Making copy of output for future use
         copy_data = dict(res['rows'][0])
@@ -571,8 +584,7 @@ It may have been removed by another user.
                 return make_json_response(
                     status=410,
                     success=0,
-                    errormsg=res + '\n' +
-                             'Operation failed while running create statement'
+                    errormsg=res
                 )
 
             # we need oid to to add object in tree at browser,
@@ -634,7 +646,7 @@ It may have been removed by another user.
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid):
+    def delete(self, gid, sid, did, scid=None):
         """
         This function will delete an existing schema object
 
@@ -645,54 +657,56 @@ It may have been removed by another user.
            scid: Schema ID
         """
 
-        try:
-            # Get name for schema from did
-            SQL = render_template(
-                "/".join([self.template_path, 'sql/get_name.sql']),
-                _=gettext,
-                scid=scid
+        if scid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
             )
+        else:
+            data = {'ids': [scid]}
 
-            status, name = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=name)
-
-            if name is None:
-                return make_json_response(
-                    status=410,
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified schema could not be found.\n'
-                    )
+        for scid in data['ids']:
+            try:
+                # Get name for schema from did
+                SQL = render_template(
+                    "/".join([self.template_path, 'sql/get_name.sql']),
+                    _=gettext,
+                    scid=scid
                 )
 
-            # drop schema
-            SQL = render_template(
-                "/".join([self.template_path, 'sql/delete.sql']),
-                _=gettext, name=name, conn=self.conn,
-                cascade=True if self.cmd == 'delete' else False
-            )
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                status, name = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=name)
 
-            return make_json_response(
-                success=1,
-                info=gettext("Schema dropped"),
-                data={
-                    'id': scid,
-                    'sid': sid,
-                    'gid': gid,
-                    'did': did
-                }
-            )
+                if name is None:
+                    return make_json_response(
+                        status=410,
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified schema could not be found.\n'
+                        )
+                    )
 
-        except Exception as e:
-            current_app.logger.exception(e)
-            return internal_server_error(errormsg=str(e))
+                # drop schema
+                SQL = render_template(
+                    "/".join([self.template_path, 'sql/delete.sql']),
+                    _=gettext, name=name, conn=self.conn,
+                    cascade=True if self.cmd == 'delete' else False
+                )
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
+
+            except Exception as e:
+                current_app.logger.exception(e)
+                return internal_server_error(errormsg=str(e))
+
+        return make_json_response(
+            success=1,
+            info=gettext("Schema dropped")
+        )
 
     @check_precondition
     def msql(self, gid, sid, did, scid=None):
@@ -793,7 +807,9 @@ It may have been removed by another user.
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the schema in the database. It may have been removed by another user."""))
+            return gone(gettext(
+                'Could not find the schema in the database.'
+                ' It may have been removed by another user.'))
 
         data = res['rows'][0]
         data = self._formatter(data, scid)
@@ -884,10 +900,12 @@ It may have been removed by another user.
         nodes = []
         for module in self.blueprint.submodules:
             if isinstance(module, PGChildModule):
-                if self.manager is not None and \
-                        module.BackendSupported(
-                            self.manager, **backend_support_keywords
-                        ):
+                if (
+                    self.manager is not None and
+                    module.BackendSupported(
+                        self.manager, **backend_support_keywords
+                    )
+                ):
                     nodes.extend(module.get_nodes(**kwargs))
             else:
                 nodes.extend(module.get_nodes(**kwargs))
@@ -999,5 +1017,6 @@ It may have been removed by another user.
 
         return ajax_response(response=SQL.strip("\n"))
 
+
 SchemaView.register_node_view(schema_blueprint)
 CatalogView.register_node_view(catalog_blueprint)
