@@ -2,28 +2,33 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 """ Implements Collation Node """
 
-import simplejson as json
 from functools import wraps
 
-import pgadmin.browser.server_groups.servers.databases as database
+import simplejson as json
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
+
+import pgadmin.browser.server_groups.servers.databases as database
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.server_groups.servers.databases.schemas.utils \
     import SchemaChildModule
 from pgadmin.browser.utils import PGChildNodeView
+from pgadmin.utils import IS_PY2
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
-    make_response as ajax_response
+    make_response as ajax_response, gone
+from pgadmin.utils.compile_template_name import compile_template_path
 from pgadmin.utils.driver import get_driver
-from pgadmin.utils.ajax import gone
 
-from config import PG_DEFAULT_DRIVER
+# If we are in Python3
+if not IS_PY2:
+    unicode = str
 
 
 class CollationModule(SchemaChildModule):
@@ -63,6 +68,7 @@ class CollationModule(SchemaChildModule):
         super(CollationModule, self).__init__(*args, **kwargs)
         self.min_ver = 90100
         self.max_ver = None
+        self.min_gpdbver = 1000000000
 
     def get_nodes(self, gid, sid, did, scid):
         """
@@ -155,9 +161,9 @@ class CollationView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -186,15 +192,21 @@ class CollationView(PGChildNodeView):
             )
             self.conn = self.manager.connection(did=kwargs['did'])
             # Set the template path for the SQL scripts
-            self.template_path = 'collation/sql/#{0}#'.format(self.manager.version)
+            self.template_path = compile_template_path(
+                'collation/sql/',
+                self.manager.server_type,
+                self.manager.version
+            )
 
             return f(*args, **kwargs)
+
         return wrap
 
     @check_precondition
     def list(self, gid, sid, did, scid):
         """
-        This function is used to list all the collation nodes within that collection.
+        This function is used to list all the collation nodes within that
+        collection.
 
         Args:
             gid: Server group ID
@@ -220,7 +232,8 @@ class CollationView(PGChildNodeView):
     @check_precondition
     def nodes(self, gid, sid, did, scid):
         """
-        This function will used to create all the child node within that collection.
+        This function will used to create all the child node within that
+        collection.
         Here it will create all the collation node.
 
         Args:
@@ -315,7 +328,9 @@ class CollationView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the collation object in the database. It may have been removed by another user."""))
+            return gone(
+                gettext("Could not find the collation object in the database.")
+            )
 
         return ajax_response(
             response=res['rows'][0],
@@ -370,22 +385,28 @@ class CollationView(PGChildNodeView):
         missing_definition_flag = False
 
         for arg in definition_args:
-            if arg == 'locale' and \
-                    (arg not in data or data[arg] == ''):
+            if (
+                arg == 'locale' and
+                (arg not in data or data[arg] == '')
+            ):
                 if 'copy_collation' not in data and (
-                                'lc_collate' not in data and 'lc_type' not in data
+                    'lc_collate' not in data and 'lc_type' not in data
                 ):
                     missing_definition_flag = True
 
-            if arg == 'copy_collation' and \
-                    (arg not in data or data[arg] == ''):
+            if (
+                arg == 'copy_collation' and
+                (arg not in data or data[arg] == '')
+            ):
                 if 'locale' not in data and (
-                                'lc_collate' not in data and 'lc_type' not in data
+                    'lc_collate' not in data and 'lc_type' not in data
                 ):
                     missing_definition_flag = True
 
-            if (arg == 'lc_collate' or arg == 'lc_type') and \
-                    (arg not in data or data[arg] == ''):
+            if (
+                (arg == 'lc_collate' or arg == 'lc_type') and
+                (arg not in data or data[arg] == '')
+            ):
                 if 'copy_collation' not in data and 'locale' not in data:
                     missing_definition_flag = True
 
@@ -425,7 +446,8 @@ class CollationView(PGChildNodeView):
                 status=410,
                 success=0,
                 errormsg=gettext(
-                    "Definition incomplete. Please provide Locale OR Copy Collation OR LC_TYPE/LC_COLLATE"
+                    "Definition incomplete. Please provide Locale OR Copy "
+                    "Collation OR LC_TYPE/LC_COLLATE."
                 )
             )
 
@@ -460,7 +482,7 @@ class CollationView(PGChildNodeView):
         )
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, coid):
+    def delete(self, gid, sid, did, scid, coid=None):
         """
         This function will delete existing the collation object
 
@@ -471,6 +493,12 @@ class CollationView(PGChildNodeView):
            scid: Schema ID
            coid: Collation ID
         """
+        if coid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [coid]}
 
         # Below will decide if it's simple drop or drop with cascade call
         if self.cmd == 'delete':
@@ -480,40 +508,36 @@ class CollationView(PGChildNodeView):
             cascade = False
 
         try:
-            SQL = render_template("/".join([self.template_path,
-                                            'get_name.sql']),
-                                  scid=scid, coid=coid)
-            status, name = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=name)
+            for coid in data['ids']:
+                SQL = render_template("/".join([self.template_path,
+                                                'get_name.sql']),
+                                      scid=scid, coid=coid)
+                status, name = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=name)
 
-            if name is None:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified collation could not be found.\n'
+                if name is None:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified collation could not be found.\n'
+                        )
                     )
-                )
 
-            SQL = render_template("/".join([self.template_path,
-                                            'delete.sql']),
-                                  name=name, cascade=cascade,
-                                  conn=self.conn)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      name=name, cascade=cascade,
+                                      conn=self.conn)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Collation dropped"),
-                data={
-                    'id': coid,
-                    'scid': scid,
-                    'did': did
-                }
+                info=gettext("Collation dropped")
             )
 
         except Exception as e:
@@ -535,6 +559,9 @@ class CollationView(PGChildNodeView):
             request.data, encoding='utf-8'
         )
         SQL, name = self.get_sql(gid, sid, data, scid, coid)
+        # Most probably this is due to error
+        if not isinstance(SQL, (str, unicode)):
+            return SQL
         SQL = SQL.strip('\n').strip(' ')
         status, res = self.conn.execute_scalar(SQL)
 
@@ -581,13 +608,16 @@ class CollationView(PGChildNodeView):
 
         try:
             SQL, name = self.get_sql(gid, sid, data, scid, coid)
+            # Most probably this is due to error
+            if not isinstance(SQL, (str, unicode)):
+                return SQL
             if SQL == '':
                 SQL = "--modified SQL"
 
             return make_json_response(
                 data=SQL,
                 status=200
-                )
+            )
         except Exception as e:
             return internal_server_error(errormsg=str(e))
 
@@ -602,12 +632,19 @@ class CollationView(PGChildNodeView):
             status, res = self.conn.execute_dict(SQL)
             if not status:
                 return internal_server_error(errormsg=res)
+            if len(res['rows']) == 0:
+                return gone(
+                    gettext(
+                        "Could not find the collation object in the database.")
+                )
+
             old_data = res['rows'][0]
             SQL = render_template(
                 "/".join([self.template_path, 'update.sql']),
                 data=data, o_data=old_data, conn=self.conn
             )
-            return SQL.strip('\n'), data['name'] if 'name' in data else old_data['name']
+            return SQL.strip('\n'), data['name'] if 'name' in data else \
+                old_data['name']
         else:
             required_args = [
                 'name'
@@ -628,7 +665,8 @@ class CollationView(PGChildNodeView):
     @check_precondition
     def sql(self, gid, sid, did, scid, coid):
         """
-        This function will generates reverse engineered sql for collation object
+        This function will generates reverse engineered sql for collation
+        object
 
          Args:
            gid: Server Group ID
@@ -643,6 +681,10 @@ class CollationView(PGChildNodeView):
         status, res = self.conn.execute_dict(SQL)
         if not status:
             return internal_server_error(errormsg=res)
+        if len(res['rows']) == 0:
+            return gone(
+                gettext("Could not find the collation object in the database.")
+            )
 
         data = res['rows'][0]
 
