@@ -9,19 +9,20 @@
 
 """Implements the Domain Constraint Module."""
 
-import simplejson as json
 from functools import wraps
 
+import simplejson as json
+from flask import render_template, request, jsonify
+from flask_babelex import gettext
+
 import pgadmin.browser.server_groups.servers.databases.schemas.domains \
     as domains
-from flask import render_template, request, jsonify
-from flask_babel import gettext
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-from config import PG_DEFAULT_DRIVER
 
 
 class DomainConstraintModule(CollectionNodeModule):
@@ -166,9 +167,8 @@ class DomainConstraintView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -211,7 +211,8 @@ class DomainConstraintView(PGChildNodeView):
                             status=410,
                             success=0,
                             errormsg=gettext(
-                                "Could not find the required parameter (%s)." % arg
+                                "Could not find the required parameter (%s)." %
+                                arg
                             )
                         )
 
@@ -247,9 +248,11 @@ class DomainConstraintView(PGChildNodeView):
             self.qtIdent = driver.qtIdent
 
             # Set the template path for the SQL scripts
-            self.template_path = 'domain_constraints/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'domain_constraints/sql/#{0}#'.format(
+                self.manager.version)
 
             return f(*args, **kwargs)
+
         return wrap
 
     @check_precondition
@@ -383,7 +386,7 @@ class DomainConstraintView(PGChildNodeView):
         if len(res['rows']) == 0:
             return gone(gettext(
                 "Could not find the specified domain constraint."
-                )
+            )
             )
 
         data = res['rows'][0]
@@ -446,7 +449,7 @@ class DomainConstraintView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, doid, coid):
+    def delete(self, gid, sid, did, scid, doid, coid=None):
         """
         Drops the Domain Constraint object.
 
@@ -458,45 +461,47 @@ class DomainConstraintView(PGChildNodeView):
             doid: Domain Id
             coid: Domain Constraint Id
         """
+        if coid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [coid]}
+
         try:
-            SQL = render_template("/".join([self.template_path,
-                                            'properties.sql']),
-                                  doid=doid, coid=coid)
-            status, res = self.conn.execute_dict(SQL)
+            for coid in data['ids']:
+                SQL = render_template("/".join([self.template_path,
+                                                'properties.sql']),
+                                      doid=doid, coid=coid)
+                status, res = self.conn.execute_dict(SQL)
 
-            if not status:
-                return internal_server_error(errormsg=res)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            if not res['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified domain constraint could not be found.\n'
+                if not res['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified domain constraint '
+                            'could not be found.\n'
+                        )
                     )
-                )
 
-            data = res['rows'][0]
+                data = res['rows'][0]
 
-            SQL = render_template("/".join([self.template_path,
-                                            'delete.sql']),
-                                  data=data)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      data=data)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Domain Constraint dropped"),
-                data={
-                    'id': doid,
-                    'scid': scid,
-                    'sid': sid,
-                    'gid': gid,
-                    'did': did
-                }
+                info=gettext("Domain Constraint dropped")
             )
 
         except Exception as e:
@@ -585,7 +590,7 @@ class DomainConstraintView(PGChildNodeView):
         if len(res['rows']) == 0:
             return gone(gettext(
                 "Could not find the specified domain constraint."
-                )
+            )
             )
 
         data = res['rows'][0]
@@ -657,7 +662,7 @@ class DomainConstraintView(PGChildNodeView):
                 if len(res['rows']) == 0:
                     return False, gone(gettext(
                         "Could not find the specified domain constraint."
-                        )
+                    )
                     )
 
                 old_data = res['rows'][0]
@@ -672,7 +677,10 @@ class DomainConstraintView(PGChildNodeView):
                 SQL = render_template("/".join([self.template_path,
                                                 'create.sql']),
                                       data=data, domain=domain, schema=schema)
-            return True, SQL.strip('\n'), data['name'] if 'name' in data else old_data['name']
+            if 'name' in data:
+                return True, SQL.strip('\n'), data['name']
+            else:
+                return True, SQL.strip('\n'), old_data['name']
         except Exception as e:
             return False, internal_server_error(errormsg=str(e)), None
 
