@@ -2,26 +2,29 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 """Defines views for management of Fts Template node"""
 
-import simplejson as json
 from functools import wraps
 
+import simplejson as json
 from flask import render_template, make_response, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
+
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.server_groups.servers.databases import DatabaseModule
-from pgadmin.browser.server_groups.servers.databases.schemas.utils import SchemaChildModule
+from pgadmin.browser.server_groups.servers.databases.schemas.utils import \
+    SchemaChildModule
 from pgadmin.browser.utils import PGChildNodeView
+from pgadmin.utils import IS_PY2
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-from config import PG_DEFAULT_DRIVER
-from pgadmin.utils import IS_PY2
+
 # If we are in Python3
 if not IS_PY2:
     unicode = str
@@ -36,7 +39,8 @@ class FtsTemplateModule(SchemaChildModule):
     Methods:
     -------
     * __init__(*args, **kwargs)
-      - Method is used to initialize the FtsTemplateModule and it's base module.
+      - Method is used to initialize the FtsTemplateModule and it's base
+      module.
 
     * get_nodes(gid, sid, did, scid)
       - Method is used to generate the browser collection node.
@@ -55,6 +59,7 @@ class FtsTemplateModule(SchemaChildModule):
         self.min_ver = None
         self.max_ver = None
         super(FtsTemplateModule, self).__init__(*args, **kwargs)
+        self.min_gpdbver = 1000000000
 
     def get_nodes(self, gid, sid, did, scid):
         """
@@ -89,10 +94,11 @@ class FtsTemplateView(PGChildNodeView):
     """
     class FtsTemplateView(PGChildNodeView)
 
-        A view class for FTS Tempalte node derived from PGChildNodeView. This class is
-        responsible for all the stuff related to view like create/update/delete
-        responsible for all the stuff related to view like create/update/delete
-        FTS template, showing properties of node, showing sql in sql pane.
+        A view class for FTS Tempalte node derived from PGChildNodeView. This
+        class is responsible for all the stuff related to view like
+        create/update/delete responsible for all the stuff related to view
+        like create/update/delete FTS template, showing properties of node,
+        showing sql in sql pane.
 
     Methods:
     -------
@@ -112,11 +118,13 @@ class FtsTemplateView(PGChildNodeView):
       - This function is used to list all the  nodes within that collection.
 
     * nodes()
-      - This function will used to create all the child node within that collection.
+      - This function will used to create all the child node within that
+      collection.
         Here it will create all the FTS Template nodes.
 
     * properties(gid, sid, did, scid, tid)
-      - This function will show the properties of the selected FTS Template node
+      - This function will show the properties of the selected FTS Template
+      node
 
     * create(gid, sid, did, scid)
       - This function will create the new FTS Template object
@@ -128,22 +136,27 @@ class FtsTemplateView(PGChildNodeView):
       - This function will drop the FTS Template object
 
     * msql(gid, sid, did, scid, tid)
-      - This function is used to return modified SQL for the selected FTS Template node
+      - This function is used to return modified SQL for the selected FTS
+      Template node
 
     * get_sql(data, tid)
       - This function will generate sql from model data
 
     * sql(gid, sid, did, scid,  tid):
-      - This function will generate sql to show it in sql pane for the selected FTS Template node.
+      - This function will generate sql to show it in sql pane for the selected
+      FTS Template node.
 
     * get_type():
-      - This function will fetch all the types for source and target types select control.
+      - This function will fetch all the types for source and target types
+      select control.
 
     * dependents(gid, sid, did, scid, tid):
-      - This function get the dependents and return ajax response for the Fts Tempalte node.
+      - This function get the dependents and return ajax response for the
+      Fts Template node.
 
     * dependencies(self, gid, sid, did, scid, tid):
-      - This function get the dependencies and return ajax response for the FTS Tempalte node.
+      - This function get the dependencies and return ajax response for the
+      FTS Template node.
 
     """
 
@@ -162,12 +175,12 @@ class FtsTemplateView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
         'children': [{
             'get': 'children'
         }],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
         'msql': [{'get': 'msql'}, {'get': 'msql'}],
@@ -211,9 +224,11 @@ class FtsTemplateView(PGChildNodeView):
             self.manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(
                 kwargs['sid'])
             self.conn = self.manager.connection(did=kwargs['did'])
-            self.template_path = 'fts_template/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'fts_template/sql/#{0}#'.format(
+                self.manager.version)
 
             return f(*args, **kwargs)
+
         return wrap
 
     @check_precondition
@@ -418,7 +433,7 @@ class FtsTemplateView(PGChildNodeView):
         )
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, tid):
+    def delete(self, gid, sid, did, scid, tid=None):
         """
         This function will drop the fts_template object
         :param gid: group id
@@ -427,6 +442,13 @@ class FtsTemplateView(PGChildNodeView):
         :param scid: schema id
         :param tid: fts tempate id
         """
+        if tid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [tid]}
+
         # Below will decide if it's simple drop or drop with cascade call
         if self.cmd == 'delete':
             # This is a cascade operation
@@ -434,46 +456,40 @@ class FtsTemplateView(PGChildNodeView):
         else:
             cascade = False
 
-        # Get name for template from tid
-        sql = render_template("/".join([self.template_path, 'delete.sql']),
-                              tid=tid)
-        status, res = self.conn.execute_dict(sql)
-        if not status:
-            return internal_server_error(errormsg=res)
+        for tid in data['ids']:
+            # Get name for template from tid
+            sql = render_template("/".join([self.template_path, 'delete.sql']),
+                                  tid=tid)
+            status, res = self.conn.execute_dict(sql)
+            if not status:
+                return internal_server_error(errormsg=res)
 
-        if not res['rows']:
-            return make_json_response(
-                success=0,
-                errormsg=gettext(
-                    'Error: Object not found.'
-                ),
-                info=gettext(
-                    'The specified FTS template could not be found.\n'
+            if not res['rows']:
+                return make_json_response(
+                    success=0,
+                    errormsg=gettext(
+                        'Error: Object not found.'
+                    ),
+                    info=gettext(
+                        'The specified FTS template could not be found.\n'
+                    )
                 )
-            )
 
-        # Drop fts template
-        result = res['rows'][0]
-        sql = render_template("/".join([self.template_path, 'delete.sql']),
-                              name=result['name'],
-                              schema=result['schema'],
-                              cascade=cascade
-                              )
+            # Drop fts template
+            result = res['rows'][0]
+            sql = render_template("/".join([self.template_path, 'delete.sql']),
+                                  name=result['name'],
+                                  schema=result['schema'],
+                                  cascade=cascade
+                                  )
 
-        status, res = self.conn.execute_scalar(sql)
-        if not status:
-            return internal_server_error(errormsg=res)
+            status, res = self.conn.execute_scalar(sql)
+            if not status:
+                return internal_server_error(errormsg=res)
 
         return make_json_response(
             success=1,
-            info=gettext("FTS Template dropped"),
-            data={
-                'id': tid,
-                'sid': sid,
-                'gid': gid,
-                'did': did,
-                'scid': scid
-            }
+            info=gettext("FTS Template dropped")
         )
 
     @check_precondition
@@ -501,7 +517,7 @@ class FtsTemplateView(PGChildNodeView):
         return make_json_response(
             data=SQL,
             status=200
-            )
+        )
 
     def get_sql(self, gid, sid, did, scid, data, tid=None):
         """
@@ -532,7 +548,8 @@ class FtsTemplateView(PGChildNodeView):
             old_data = res['rows'][0]
 
             # If user has changed the schema then fetch new schema directly
-            # using its oid otherwise fetch old schema name using fts template oid
+            # using its oid otherwise fetch old schema name using
+            # fts template oid
             sql = render_template(
                 "/".join([self.template_path, 'schema.sql']),
                 data=data)
@@ -563,7 +580,9 @@ class FtsTemplateView(PGChildNodeView):
                 data=new_data, o_data=old_data
             )
             # Fetch sql query for modified data
-            return sql.strip('\n'), data['name'] if 'name' in data else old_data['name']
+            if 'name' in data:
+                return sql.strip('\n'), data['name']
+            return sql.strip('\n'), old_data['name']
         else:
             # Fetch schema name from schema oid
             sql = render_template("/".join([self.template_path, 'schema.sql']),
@@ -577,9 +596,11 @@ class FtsTemplateView(PGChildNodeView):
             new_data = data.copy()
             new_data['schema'] = schema
 
-            if 'tmpllexize' in new_data and \
-                            'name' in new_data and \
-                            'schema' in new_data:
+            if (
+                'tmpllexize' in new_data and
+                'name' in new_data and
+                'schema' in new_data
+            ):
                 sql = render_template("/".join([self.template_path,
                                                 'create.sql']),
                                       data=new_data,
@@ -666,15 +687,15 @@ class FtsTemplateView(PGChildNodeView):
         if not status:
             return internal_server_error(
                 gettext(
-                    "Could not generate reversed engineered query for the FTS Template.\n{0}").format(
-                    res
-                )
+                    "Could not generate reversed engineered query for the "
+                    "FTS Template.\n{0}").format(res)
             )
 
         if res is None:
             return gone(
                 gettext(
-                    "Could not generate reversed engineered query for FTS Template node.")
+                    "Could not generate reversed engineered query for "
+                    "FTS Template node.")
             )
 
         return ajax_response(response=res)
