@@ -19,7 +19,7 @@ from functools import wraps
 import pgadmin.browser.server_groups.servers.databases as databases
 from flask import render_template, make_response, request, jsonify, \
     current_app
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.server_groups.servers.databases.schemas.utils import \
     SchemaChildModule, DataTypeReader
 from pgadmin.browser.server_groups.servers.databases.utils import \
@@ -71,10 +71,10 @@ class FunctionModule(SchemaChildModule):
         """
         super(FunctionModule, self).__init__(*args, **kwargs)
 
-        self.min_ver = 90100
+        self.min_ver = None
         self.max_ver = None
         self.server_type = None
-        self.min_gpdbver = 1000000000
+        self.min_gpdbver = None
 
     def get_nodes(self, gid, sid, did, scid):
         """
@@ -199,9 +199,9 @@ class FunctionView(PGChildNodeView, DataTypeReader):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'nodes'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -257,16 +257,22 @@ class FunctionView(PGChildNodeView, DataTypeReader):
                 req = request.args or request.form
 
             if 'fnid' not in kwargs:
+                req_args = self.required_args
+                # We need to remove 'prosrc' from the required arguments list
+                # if language is 'c'.
+                if req['lanname'] == 'c' and 'prosrc' in req_args:
+                    req_args.remove('prosrc')
 
-                for arg in self.required_args:
+                for arg in req_args:
                     if (arg not in req or req[arg] == '') or \
-                            (arg == 'probin' and req['lanname'] == 'c'
-                             and (arg not in req or req[arg] == '')):
+                        (arg == 'probin' and req['lanname'] == 'c' and
+                         (arg not in req or req[arg] == '')):
                         return make_json_response(
                             status=410,
                             success=0,
                             errormsg=gettext(
-                                "Could not find the required parameter (%s)." % arg
+                                "Could not find the required parameter (%s)." %
+                                arg
                             )
                         )
 
@@ -276,14 +282,16 @@ class FunctionView(PGChildNodeView, DataTypeReader):
                                'seclabels', 'acl', 'args']
 
             for key in req:
-                if key in list_params and req[key] != '' \
-                        and req[key] is not None:
+                if (
+                    key in list_params and req[key] != '' and
+                    req[key] is not None
+                ):
                     # Coverts string into python list as expected.
                     data[key] = json.loads(req[key], encoding='utf-8')
                 elif (
-                                            key == 'proretset' or key == 'proisstrict' or
-                                        key == 'prosecdef' or key == 'proiswindow' or
-                                key == 'proleakproof'
+                    key == 'proretset' or key == 'proisstrict' or
+                    key == 'prosecdef' or key == 'proiswindow' or
+                    key == 'proleakproof'
                 ):
                     data[key] = True if (
                         req[key] == 'true' or req[key] is True) \
@@ -392,7 +400,8 @@ class FunctionView(PGChildNodeView, DataTypeReader):
         if fnid is not None:
             if len(rset['rows']) == 0:
                 return gone(
-                     _("Could not find the specified %s.").format(self.node_type)
+                    _("Could not find the specified %s.").format(
+                        self.node_type)
                 )
 
             row = rset['rows'][0]
@@ -572,7 +581,7 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             proargnames[i] if len(proargnames) > i else '',
             proargdefaultvals[i] if len(proargdefaultvals) > i else ''
         )
-                   for i in range(len(proargtypes))]
+            for i in range(len(proargtypes))]
 
         proargs = {"proargs": ", ".join(proargs)}
 
@@ -654,12 +663,15 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             fnid: Function Id
         """
 
-        condition = "(typtype IN ('b', 'c', 'd', 'e', 'p', 'r') AND typname NOT IN ('any', 'trigger', 'language_handler', 'event_trigger'))"
+        condition = "(typtype IN ('b', 'c', 'd', 'e', 'p', 'r') AND " \
+                    "typname NOT IN ('any', 'trigger', 'language_handler', " \
+                    "'event_trigger'))"
         if self.blueprint.show_system_objects:
-            condition += " AND nspname NOT LIKE E'pg\\\\_toast%' AND nspname NOT LIKE E'pg\\\\_temp%'"
+            condition += " AND nspname NOT LIKE E'pg\\\\_toast%' AND " \
+                         "nspname NOT LIKE E'pg\\\\_temp%'"
 
         # Get Types
-        status, types = self.get_types(self.conn, condition)
+        status, types = self.get_types(self.conn, condition, False, scid)
 
         if not status:
             return internal_server_error(errormsg=types)
@@ -697,7 +709,7 @@ class FunctionView(PGChildNodeView, DataTypeReader):
                 data=res,
                 status=200
             )
-        except:
+        except Exception:
             exc_type, exc_value, exc_traceback = sys.exc_info()
             current_app.logger.error(traceback.print_exception(
                 exc_type,
@@ -789,7 +801,7 @@ class FunctionView(PGChildNodeView, DataTypeReader):
         )
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, fnid):
+    def delete(self, gid, sid, did, scid, fnid=None):
         """
         Drop the Function.
 
@@ -800,6 +812,12 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             scid: Schema Id
             fnid: Function Id
         """
+        if fnid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [fnid]}
 
         if self.cmd == 'delete':
             # This is a cascade operation
@@ -808,44 +826,39 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             cascade = False
 
         try:
-            # Fetch Name and Schema Name to delete the Function.
-            SQL = render_template("/".join([self.sql_template_path,
-                                            'delete.sql']), scid=scid, fnid=fnid)
-            status, res = self.conn.execute_2darray(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+            for fnid in data['ids']:
+                # Fetch Name and Schema Name to delete the Function.
+                SQL = render_template("/".join([self.sql_template_path,
+                                                'delete.sql']), scid=scid,
+                                      fnid=fnid)
+                status, res = self.conn.execute_2darray(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            if not res['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified function could not be found.\n'
+                if not res['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified function could not be found.\n'
+                        )
                     )
-                )
 
-            SQL = render_template("/".join([self.sql_template_path,
-                                            'delete.sql']),
-                                  name=res['rows'][0]['name'],
-                                  func_args=res['rows'][0]['func_args'],
-                                  nspname=res['rows'][0]['nspname'],
-                                  cascade=cascade)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.sql_template_path,
+                                                'delete.sql']),
+                                      name=res['rows'][0]['name'],
+                                      func_args=res['rows'][0]['func_args'],
+                                      nspname=res['rows'][0]['nspname'],
+                                      cascade=cascade)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Function dropped."),
-                data={
-                    'id': fnid,
-                    'scid': scid,
-                    'sid': sid,
-                    'gid': gid,
-                    'did': did
-                }
+                info=gettext("Function dropped.")
             )
 
         except Exception as e:
@@ -941,13 +954,18 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             resp_data['args'] = resp_data['arguments']
 
         for a in args_list:
-            if (('argmode' in a and a['argmode'] != 'OUT' and
+            if (
+                (
+                    'argmode' in a and a['argmode'] != 'OUT' and
                     a['argmode'] is not None
-                 ) or 'argmode' not in a):
+                ) or 'argmode' not in a
+            ):
                 if 'argmode' in a:
                     args += a['argmode'] + " "
-                if 'argname' in a and a['argname'] != '' \
-                        and a['argname'] is not None:
+                if (
+                    'argname' in a and a['argname'] != '' and
+                    a['argname'] is not None
+                ):
                     args += self.qtIdent(
                         self.conn, a['argname']) + " "
                 if 'argtype' in a:
@@ -970,8 +988,8 @@ class FunctionView(PGChildNodeView, DataTypeReader):
 
             # Get Schema Name from its OID.
             if 'pronamespace' in resp_data:
-                resp_data['pronamespace'] = self._get_schema(resp_data[
-                    'pronamespace'])
+                resp_data['pronamespace'] = self._get_schema(
+                    resp_data['pronamespace'])
 
             SQL = render_template("/".join([self.sql_template_path,
                                             'get_definition.sql']
@@ -982,21 +1000,28 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             if not status:
                 return internal_server_error(errormsg=res)
 
+            name_with_default_args = self.qtIdent(
+                self.conn,
+                res['rows'][0]['nspname'],
+                res['rows'][0]['proname']
+            ) + '(' + res['rows'][0]['func_args'] + ')'
             # Add newline and tab before each argument to format
-            name_with_default_args = res['rows'][0]['name_with_default_args'].replace(', ', ',\r\t').replace('(', '(\r\t')
+            name_with_default_args = name_with_default_args.replace(
+                ', ', ',\r\t').replace('(', '(\r\t')
 
             # Parse privilege data
             if 'acl' in resp_data:
                 resp_data['acl'] = parse_priv_to_db(resp_data['acl'], ['X'])
 
                 # Check Revoke all for public
-                resp_data['revoke_all'] = self._set_revoke_all(resp_data['acl'])
+                resp_data['revoke_all'] = self._set_revoke_all(
+                    resp_data['acl'])
 
             # Generate sql for "SQL panel"
             # func_def is procedure signature with default arguments
             # query_for - To distinguish the type of call
             func_def = render_template("/".join([self.sql_template_path,
-                                                'create.sql']),
+                                                 'create.sql']),
                                        data=resp_data, query_type="create",
                                        func_def=name_with_default_args,
                                        query_for="sql_panel")
@@ -1005,15 +1030,17 @@ class FunctionView(PGChildNodeView, DataTypeReader):
 
             # Get Schema Name from its OID.
             if 'pronamespace' in resp_data:
-                resp_data['pronamespace'] = self._get_schema(resp_data[
-                    'pronamespace'])
+                resp_data['pronamespace'] = self._get_schema(
+                    resp_data['pronamespace']
+                )
 
             # Parse privilege data
             if 'acl' in resp_data:
                 resp_data['acl'] = parse_priv_to_db(resp_data['acl'], ['X'])
 
                 # Check Revoke all for public
-                resp_data['revoke_all'] = self._set_revoke_all(resp_data['acl'])
+                resp_data['revoke_all'] = self._set_revoke_all(
+                    resp_data['acl'])
 
             SQL = render_template("/".join([self.sql_template_path,
                                             'get_definition.sql']
@@ -1024,8 +1051,16 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             if not status:
                 return internal_server_error(errormsg=res)
 
+            name_with_default_args = self.qtIdent(
+                self.conn,
+                res['rows'][0]['nspname'],
+                res['rows'][0]['proname']
+            ) + '(' + res['rows'][0]['func_args'] + ')'
             # Add newline and tab before each argument to format
-            name_with_default_args = res['rows'][0]['name_with_default_args'].replace(', ', ',\r\t').replace('(', '(\r\t')
+            name_with_default_args = name_with_default_args.replace(
+                ', ',
+                ',\r\t'
+            ).replace('(', '(\r\t')
 
             # Generate sql for "SQL panel"
             # func_def is function signature with default arguments
@@ -1098,8 +1133,9 @@ class FunctionView(PGChildNodeView, DataTypeReader):
 
         # Get Schema Name from its OID.
         if 'pronamespace' in data:
-            data['pronamespace'] = self._get_schema(data[
-                                                        'pronamespace'])
+            data['pronamespace'] = self._get_schema(
+                data['pronamespace']
+            )
         if 'provolatile' in data:
             data['provolatile'] = vol_dict[data['provolatile']]
 
@@ -1118,14 +1154,18 @@ class FunctionView(PGChildNodeView, DataTypeReader):
                 )
 
             # Get Schema Name
-            old_data['pronamespace'] = self._get_schema(old_data[
-                                                            'pronamespace'])
+            old_data['pronamespace'] = self._get_schema(
+                old_data['pronamespace']
+            )
 
-            if 'provolatile' in old_data:
+            if 'provolatile' in old_data and \
+                    old_data['provolatile'] is not None:
                 old_data['provolatile'] = vol_dict[old_data['provolatile']]
 
-            if 'proparallel' in old_data:
-                old_data['proparallel'] = parallel_dict[old_data['proparallel']]
+            if 'proparallel' in old_data and \
+                    old_data['proparallel'] is not None:
+                old_data['proparallel'] = \
+                    parallel_dict[old_data['proparallel']]
 
             # If any of the below argument is changed,
             # then CREATE OR REPLACE SQL statement should be called
@@ -1136,8 +1176,7 @@ class FunctionView(PGChildNodeView, DataTypeReader):
 
             data['change_func'] = False
             for arg in fun_change_args:
-                if arg == 'arguments' and arg in data and len(data[arg]) \
-                        > 0:
+                if arg == 'arguments' and arg in data and len(data[arg]) > 0:
                     data['change_func'] = True
                 elif arg in data:
                     data['change_func'] = True
@@ -1195,8 +1234,10 @@ class FunctionView(PGChildNodeView, DataTypeReader):
 
                 # Prepare final dict of new and old variables
                 for name, val in old_data['chngd_variables'].items():
-                    if name not in chngd_variables and name not in \
-                            del_variables:
+                    if (
+                        name not in chngd_variables and
+                        name not in del_variables
+                    ):
                         chngd_variables[name] = val
 
                 # Prepare dict in [{'name': var_name, 'value': var_val},..]
@@ -1234,15 +1275,19 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             elif 'args' in data and len(data['args']) > 0:
                 args_list = data['args']
             for a in args_list:
-                if (('argmode' in a and a['argmode'] != 'OUT' and
-                             a['argmode'] is not None
-                     ) or 'argmode' not in a):
+                if (
+                    (
+                        'argmode' in a and a['argmode'] != 'OUT' and
+                        a['argmode'] is not None
+                    ) or 'argmode' not in a
+                ):
                     if 'argmode' in a:
                         args += a['argmode'] + " "
-                    if 'argname' in a and a['argname'] != '' \
-                            and a['argname'] is not None:
-                        args += self.qtIdent(
-                            self.conn, a['argname']) + " "
+                    if (
+                        'argname' in a and a['argname'] != '' and
+                        a['argname'] is not None
+                    ):
+                        args += self.qtIdent(self.conn, a['argname']) + " "
                     if 'argtype' in a:
                         args += a['argtype']
                         args_without_name.append(a['argtype'])
@@ -1316,6 +1361,17 @@ class FunctionView(PGChildNodeView, DataTypeReader):
         resp_data.update(parse_variables_from_db([
             {"setconfig": resp_data['proconfig']}]))
 
+        # Reset Volatile, Cost and Parallel parameter for Procedures
+        # if language is not 'edbspl' and database server version is
+        # greater than 10.
+        if self.manager.sversion >= 110000 \
+            and ('prokind' in resp_data and resp_data['prokind'] == 'p') \
+                and ('lanname' in resp_data and
+                     resp_data['lanname'] != 'edbspl'):
+            resp_data['procost'] = None
+            resp_data['provolatile'] = None
+            resp_data['proparallel'] = None
+
         return resp_data
 
     def _get_schema(self, scid):
@@ -1344,7 +1400,7 @@ class FunctionView(PGChildNodeView, DataTypeReader):
         for p in privileges:
             if p['grantee'] == 'PUBLIC':
                 revoke_all = False
-                break;
+                break
 
         return revoke_all
 
@@ -1400,12 +1456,16 @@ class FunctionView(PGChildNodeView, DataTypeReader):
         """
         # Fetch the function definition.
         SQL = render_template("/".join([self.sql_template_path,
-                              'get_definition.sql']), fnid=fnid, scid=scid)
+                                        'get_definition.sql']), fnid=fnid,
+                              scid=scid)
         status, res = self.conn.execute_2darray(SQL)
         if not status:
             return internal_server_error(errormsg=res)
 
-        name = res['rows'][0]['name']
+        name = self.qtIdent(
+            self.conn, res['rows'][0]['nspname'],
+            res['rows'][0]['proname']
+        ) + '(' + res['rows'][0]['func_with_identity_arguments'] + ')'
 
         # Fetch only arguments
         argString = name[name.rfind('('):].strip('(').strip(')')
@@ -1460,7 +1520,10 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             name = name.replace(arg, formatted_arg)
 
         name = name.replace(')', '\n)')
-        sql = "EXEC {0}".format(name)
+        if self.manager.server_type == 'pg':
+            sql = "CALL {0}".format(name)
+        else:
+            sql = "EXEC {0}".format(name)
 
         return ajax_response(response=sql)
 
@@ -1487,7 +1550,8 @@ class FunctionView(PGChildNodeView, DataTypeReader):
             # Get schema name
             status, schema_name = self.conn.execute_scalar(
                 render_template(
-                    'schema/pg/#{0}#/sql/get_name.sql'.format(self.manager.version),
+                    'schema/pg/#{0}#/sql/get_name.sql'.format(
+                        self.manager.version),
                     scid=scid
                 )
             )
@@ -1549,10 +1613,11 @@ class ProcedureModule(SchemaChildModule):
         """
         super(ProcedureModule, self).__init__(*args, **kwargs)
 
-        self.min_ver = 90100
+        self.min_ver = 110000
         self.max_ver = None
+        self.min_ppasver = 90100
         self.min_gpdbver = 1000000000
-        self.server_type = ['ppas']
+        self.server_type = ['pg', 'ppas']
 
     def get_nodes(self, gid, sid, did, scid):
         """
