@@ -1,20 +1,23 @@
 /* Create and Register Function Collection and Node. */
 define('pgadmin.node.function', [
-  'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'pgadmin', 'pgadmin.browser', 'alertify',
-  'pgadmin.browser.collection', 'pgadmin.browser.server.privilege'
-], function(gettext, url_for, $, _, S, pgAdmin, pgBrowser, alertify) {
+  'sources/gettext', 'sources/url_for', 'jquery', 'underscore', 'backbone',
+  'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
+  'pgadmin.node.schema.dir/child', 'pgadmin.browser.collection',
+  'pgadmin.browser.server.privilege',
+], function(
+  gettext, url_for, $, _, Backbone, pgAdmin, pgBrowser, Backform, schemaChild
+) {
 
   if (!pgBrowser.Nodes['coll-function']) {
-    var functions = pgBrowser.Nodes['coll-function'] =
+    pgBrowser.Nodes['coll-function'] =
       pgBrowser.Collection.extend({
         node: 'function',
         label: gettext('Functions'),
         type: 'coll-function',
         columns: ['name', 'funcowner', 'description'],
-        hasStatistics: true
+        hasStatistics: true,
       });
-  };
+  }
 
   // Argument Model
   var ArgumentModel = pgBrowser.Node.Model.extend({
@@ -24,48 +27,48 @@ define('pgadmin.node.function', [
       argtype: undefined,
       argmode: undefined,
       argname: undefined,
-      argdefval: undefined
+      argdefval: undefined,
     },
     schema: [{
       id: 'argid', visible: false, type: 'text',
-      mode: ['properties', 'edit','create']
-      },{
-        id: 'argtype', label: gettext('Data Type'), cell:
+      mode: ['properties', 'edit','create'],
+    },{
+      id: 'argtype', label: gettext('Data Type'), cell:
         'node-ajax-options', cellHeaderClasses: 'width_percent_30',
-        control: 'node-ajax-options', type: 'text', url: 'get_types',
-        editable: function(m) {
-          node_info = this.get('node_info');
-          if(node_info && 'catalog' in node_info) {
-            return false;
-          }
-          return _.isUndefined(m.isNew) ? true : m.isNew();
-        }, first_empty: true
-      },{
-        id: 'argmode', label: gettext('Mode'), type: 'options',
-        control: 'node-ajax-options', cellHeaderClasses:'width_percent_20',
-        options:[
+      control: 'node-ajax-options', type: 'text', url: 'get_types',
+      editable: function(m) {
+        var node_info = this.get('node_info');
+        if(node_info && 'catalog' in node_info) {
+          return false;
+        }
+        return _.isUndefined(m.isNew) ? true : m.isNew();
+      }, first_empty: true,
+    },{
+      id: 'argmode', label: gettext('Mode'), type: 'options',
+      control: 'node-ajax-options', cellHeaderClasses:'width_percent_20',
+      options:[
           {'label': 'IN', 'value': 'IN'},
           {'label': 'OUT', 'value': 'OUT'},
           {'label': 'INOUT', 'value': 'INOUT'},
-          {'label': 'VARIADIC', 'value': 'VARIADIC'}
-        ], editable: function(m) {
-             node_info = this.get('node_info');
-             if(node_info && 'catalog' in node_info) {
-               return false;
-             }
-             return _.isUndefined(m.isNew) ? true : m.isNew();
-           }
-      },{
-        id: 'argname', label: gettext('Argument Name'), type: 'text',
-        cell: 'string', editable: 'isInCatalog', cellHeaderClasses:'width_percent_30'
-      },{
-        id: 'argdefval', label: gettext('Default Value'), type: 'text',
-        cell: 'string', editable: 'isInCatalog',  cellHeaderClasses:'width_percent_20'
-      }
+          {'label': 'VARIADIC', 'value': 'VARIADIC'},
+      ], editable: function(m) {
+        var node_info = this.get('node_info');
+        if(node_info && 'catalog' in node_info) {
+          return false;
+        }
+        return _.isUndefined(m.isNew) ? true : m.isNew();
+      },
+    },{
+      id: 'argname', label: gettext('Argument Name'), type: 'text',
+      cell: 'string', editable: 'isInCatalog', cellHeaderClasses:'width_percent_30',
+    },{
+      id: 'argdefval', label: gettext('Default Value'), type: 'text',
+      cell: 'string', editable: 'isInCatalog',  cellHeaderClasses:'width_percent_20',
+    },
     ],
     toJSON: Backbone.Model.prototype.toJSON,
     isInCatalog: function(m){
-      node_info = this.get('node_info');
+      var node_info = this.get('node_info');
       if(node_info && 'catalog' in node_info) {
         return false;
       }
@@ -79,11 +82,12 @@ define('pgadmin.node.function', [
       return true;
     },
     validate: function() {
-    }
+    },
   });
 
   if (!pgBrowser.Nodes['function']) {
-    pgBrowser.Nodes['function'] = pgBrowser.Node.extend({
+
+    pgBrowser.Nodes['function'] = schemaChild.SchemaChildNode.extend({
       type: 'function',
       sqlAlterHelp: 'sql-alterfunction.html',
       sqlCreateHelp: 'sql-createfunction.html',
@@ -92,13 +96,14 @@ define('pgadmin.node.function', [
       collection_type: 'coll-function',
       hasSQL: true,
       hasDepends: true,
-      hasStatistics: true,
+      hasStatistics: (treeInformation) => {
+        return treeInformation.server.server_type !== 'gpdb';
+      },
       hasScriptTypes: ['create', 'select'],
-      parent_type: ['schema', 'catalog'],
-      Init: function(args) {
+      Init: function() {
         /* Avoid mulitple registration of menus */
         if (this.initialized)
-            return;
+          return;
 
         this.initialized = true;
 
@@ -107,31 +112,30 @@ define('pgadmin.node.function', [
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Function...'),
           icon: 'wcTabIcon icon-function', data: {action: 'create', check: true},
-          enable: 'canCreate'
+          enable: 'canCreate',
         },{
           name: 'create_function', node: 'function', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Function...'),
           icon: 'wcTabIcon icon-function', data: {action: 'create', check: true},
-          enable: 'canCreate'
+          enable: 'canCreate',
         },{
           name: 'create_function', node: 'schema', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Function...'),
           icon: 'wcTabIcon icon-function', data: {action: 'create', check: false},
-          enable: 'canCreate'
-        }
+          enable: 'canCreate',
+        },
         ]);
 
       },
-      canDrop: pgBrowser.Nodes['schema'].canChildDrop,
-      canDropCascade: pgBrowser.Nodes['schema'].canChildDrop,
       model: pgBrowser.Node.Model.extend({
+        idAttribute: 'oid',
         initialize: function(attrs, args) {
           var isNew = (_.size(attrs) === 0);
           if (isNew) {
             // Set Selected Schema
-            schema_id = args.node_info.schema._id
+            var schema_id = args.node_info.schema._id;
             this.set({'pronamespace': schema_id}, {silent: true});
 
             // Set Current User
@@ -171,58 +175,58 @@ define('pgadmin.node.function', [
           seclabels: [],
           acl: [],
           sysfunc: undefined,
-          sysproc: undefined
+          sysproc: undefined,
         },
         schema: [{
           id: 'name', label: gettext('Name'), cell: 'string',
           type: 'text', mode: ['properties', 'create', 'edit'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'text' , mode: ['properties']
+          type: 'text' , mode: ['properties'],
         },{
           id: 'funcowner', label: gettext('Owner'), cell: 'string',
           control: Backform.NodeListByNameControl, node: 'role',  type:
-          'text', disabled: 'isDisabled'
+          'text', disabled: 'isDisabled',
         },{
           id: 'pronamespace', label: gettext('Schema'), cell: 'string',
           control: 'node-list-by-id', type: 'text', cache_level: 'database',
-          node: 'schema', disabled: 'isDisabled', mode: ['create', 'edit']
+          node: 'schema', disabled: 'isDisabled', mode: ['create', 'edit'],
         },{
           id: 'sysfunc', label: gettext('System function?'),
-           cell:'boolean', type: 'switch',
-           mode: ['properties'], visible: 'isVisible'
+          cell:'boolean', type: 'switch',
+          mode: ['properties'], visible: 'isVisible',
         },{
           id: 'sysproc', label: gettext('System procedure?'),
-           cell:'boolean', type: 'switch',
-           mode: ['properties'], visible: 'isVisible'
+          cell:'boolean', type: 'switch',
+          mode: ['properties'], visible: 'isVisible',
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
-          type: 'multiline', disabled: 'isDisabled'
+          type: 'multiline', disabled: 'isDisabled',
         },{
           id: 'pronargs', label: gettext('Argument count'), cell: 'string',
-          type: 'text', group: gettext('Definition'), mode: ['properties']
+          type: 'text', group: gettext('Definition'), mode: ['properties'],
         },{
           id: 'proargs', label: gettext('Arguments'), cell: 'string',
           type: 'text', group: gettext('Definition'), mode: ['properties', 'edit'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'proargtypenames', label: gettext('Signature arguments'), cell:
           'string', type: 'text', group: gettext('Definition'), mode: ['properties'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'prorettypename', label: gettext('Return type'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Definition'),
           url: 'get_types', disabled: 'isDisabled', first_empty: true,
-          mode: ['create'], visible: 'isVisible'
+          mode: ['create'], visible: 'isVisible',
         },{
           id: 'prorettypename', label: gettext('Return type'), cell: 'string',
-          type: 'text', group: gettext('Definition'), disabled: true,
-          mode: ['properties', 'edit'], disabled: 'isDisabled', visible: 'isVisible'
+          type: 'text', group: gettext('Definition'),
+          mode: ['properties', 'edit'], disabled: 'isDisabled', visible: 'isVisible',
         },  {
           id: 'lanname', label: gettext('Language'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Definition'),
-          url: 'get_languages', disabled: 'isDisabled'
+          url: 'get_languages', disabled: 'isDisabled',
         },{
           id: 'prosrc', label: gettext('Code'), cell: 'string',
           type: 'text', mode: ['properties', 'create', 'edit'],
@@ -234,72 +238,75 @@ define('pgadmin.node.function', [
               return false;
             }
             return true;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'probin', label: gettext('Object file'), cell: 'string',
           type: 'text', group: gettext('Definition'), deps: ['lanname'], visible:
           function(m) {
             if (m.get('lanname') == 'c') { return true; }
             return false;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'prosrc_c', label: gettext('Link symbol'), cell: 'string',
           type: 'text', group: gettext('Definition'),  deps: ['lanname'], visible:
           function(m) {
             if (m.get('lanname') == 'c') { return true; }
             return false;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'provolatile', label: gettext('Volatility'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Options'),
+          deps: ['lanname'],
           options:[
             {'label': 'VOLATILE', 'value': 'v'},
             {'label': 'STABLE', 'value': 's'},
             {'label': 'IMMUTABLE', 'value': 'i'},
-          ], disabled: 'isDisabled', select2: {allowClear: false}
+          ], disabled: 'isDisabled', select2: {allowClear: false},
         },{
           id: 'proretset', label: gettext('Returns a set?'), type: 'switch',
           disabled: 'isDisabled', group: gettext('Options'),
-          visible: 'isVisible'
+          visible: 'isVisible',
         },{
           id: 'proisstrict', label: gettext('Strict?'), type: 'switch',
           group: gettext('Options'), disabled: 'isDisabled',
+          deps: ['lanname'],
           options: {
-            'onText': 'Yes', 'offText': 'No',
+            'onText': gettext('Yes'), 'offText': gettext('No'),
             'onColor': 'success', 'offColor': 'primary',
-            'size': 'small'
-           }
+            'size': 'small',
+          },
         },{
           id: 'prosecdef', label: gettext('Security of definer?'),
-           group: gettext('Options'), type: 'switch',
-           disabled: 'isDisabled'
+          group: gettext('Options'), type: 'switch',
+          disabled: 'isDisabled',
         },{
           id: 'proiswindow', label: gettext('Window?'),
-           group: gettext('Options'), cell:'boolean', type: 'switch',
-            disabled: 'isDisabled', visible: 'isVisible'
+          group: gettext('Options'), cell:'boolean', type: 'switch',
+          disabled: 'isDisabled', visible: 'isVisible',
         },{
           id: 'proparallel', label: gettext('Parallel'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Options'),
+          deps: ['lanname'],
           options:[
             {'label': 'UNSAFE', 'value': 'u'},
             {'label': 'RESTRICTED', 'value': 'r'},
             {'label': 'SAFE', 'value': 's'},
           ], disabled: 'isDisabled', min_version: 90600,
-          select2: {allowClear: false}
+          select2: {allowClear: false},
         },{
           id: 'procost', label: gettext('Estimated cost'), group: gettext('Options'),
-          cell:'string', type: 'text', disabled: 'isDisabled'
+          cell:'string', type: 'text', disabled: 'isDisabled', deps: ['lanname'],
         },{
           id: 'prorows', label: gettext('Estimated rows'), type: 'text',
           deps: ['proretset'], visible: 'isVisible', disabled: 'isDisabled',
-          group: gettext('Options')
+          group: gettext('Options'),
         },{
           id: 'proleakproof', label: gettext('Leak proof?'),
           group: gettext('Options'), cell:'boolean', type: 'switch', min_version: 90200,
-          disabled: 'isDisabled'
+          disabled: 'isDisabled', deps: ['lanname'],
         },{
           id: 'proacl', label: gettext('Privileges'), type: 'text',
-          mode: ['properties'], group: gettext('Security')
+          mode: ['properties'], group: gettext('Security'),
         },{
           id: 'arguments', label: gettext('Arguments'), cell: 'string',
           group: gettext('Arguments'), type: 'collection', canAdd: function(m){
@@ -315,15 +322,15 @@ define('pgadmin.node.function', [
           group: gettext('Parameters'), control: 'variable-collection',
           model: pgBrowser.Node.VariableModel,
           mode: ['edit', 'create'], canAdd: 'canVarAdd', canEdit: false,
-          canDelete: true, disabled: 'isDisabled'
-        }, pgBrowser.SecurityGroupUnderSchema, {
+          canDelete: true, disabled: 'isDisabled',
+        }, pgBrowser.SecurityGroupSchema, {
           id: 'acl', label: gettext('Privileges'), editable: false,
           model: pgBrowser.Node.PrivilegeRoleModel.extend({
-            privileges: ['X']
+            privileges: ['X'],
           }), uniqueCol : ['grantee', 'grantor'], type: 'collection',
           group: 'security', mode: ['edit', 'create'], canAdd: true,
           canDelete: true, control: 'unique-col-collection',
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'seclabels', label: gettext('Security Labels'), canAdd: true,
           model: pgBrowser.SecLabelModel, type: 'collection',
@@ -331,15 +338,15 @@ define('pgadmin.node.function', [
           canEdit: false, canDelete: true, uniqueCol : ['provider'],
           disabled: 'isDisabled', control: 'unique-col-collection',
           visible: function() {
-            return this.node && this.node.type != "procedure";
-          }
-        }
+            return this.node && this.node.type != 'procedure';
+          },
+        },
         ],
         validate: function()
         {
           var err = {},
-              errmsg,
-              seclabels = this.get('seclabels');
+            errmsg,
+            seclabels = this.get('seclabels');
 
           if (_.isUndefined(this.get('name')) || String(this.get('name')).replace(/^\s+|\s+$/g, '') == '') {
             err['name'] = gettext('Name cannot be empty.');
@@ -405,73 +412,38 @@ define('pgadmin.node.function', [
 
           return null;
         },
-        isVisible: function(m){
+        isVisible: function() {
           if (this.name == 'sysproc') { return false; }
           return true;
         },
-        isDisabled: function(m){
-          if(this.node_info &&  'catalog' in this.node_info) {
+        isDisabled: function(m) {
+          if(this.node_info && 'catalog' in this.node_info) {
             return true;
           }
-          name = this.name;
-          switch(name){
-            case 'proargs':
-            case 'proargtypenames':
-            case 'prorettypename':
-            case 'proretset':
-            case 'proiswindow':
-              return !m.isNew();
-              break;
-            case 'prorows':
-              if(m.get('proretset') == true) {
-                return false;
-              }
-              else {
-                return true;
-              }
-              break;
-            default:
+          switch(this.name){
+          case 'proargs':
+          case 'proargtypenames':
+          case 'prorettypename':
+          case 'proretset':
+          case 'proiswindow':
+            return !m.isNew();
+          case 'prorows':
+            if(m.get('proretset') == true) {
               return false;
-              break;
+            }
+            return true;
+          default:
+            return false;
           }
-          return false;
         },
-        canVarAdd: function(m) {
+        canVarAdd: function() {
           if(this.node_info &&  'catalog' in this.node_info) {
             return false;
           }
-         return true;
-        }
-      }),
-      canCreate: function(itemData, item, data) {
-        //If check is false then , we will allow create menu
-        if (data && data.check == false)
           return true;
-
-        var t = pgBrowser.tree, i = item, d = itemData;
-        // To iterate over tree to check parent node
-        while (i) {
-          // If it is schema then allow user to create Function
-          if (_.indexOf(['schema'], d._type) > -1)
-            return true;
-
-          if ('coll-function' == d._type) {
-            //Check if we are not child of catalog
-            prev_i = t.hasParent(i) ? t.parent(i) : null;
-            prev_d = prev_i ? t.itemData(prev_i) : null;
-            if( prev_d._type == 'catalog') {
-              return false;
-            } else {
-              return true;
-            }
-          }
-          i = t.hasParent(i) ? t.parent(i) : null;
-          d = i ? t.itemData(i) : null;
-        }
-        // by default we do not want to allow create menu
-        return true;
-      }
-  });
+        },
+      }),
+    });
 
   }
 
