@@ -1,23 +1,26 @@
 /* Create and Register Function Collection and Node. */
 define('pgadmin.node.trigger_function', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser', 'alertify',
-  'pgadmin.browser.collection', 'pgadmin.browser.server.privilege'
-], function(gettext, url_for, $, _, S, pgAdmin, pgBrowser, alertify) {
+  'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
+  'pgadmin.node.schema.dir/child', 'pgadmin.browser.collection',
+  'pgadmin.browser.server.privilege',
+], function(
+  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, schemaChild
+) {
 
   if (!pgBrowser.Nodes['coll-trigger_function']) {
-    var trigger_functions = pgBrowser.Nodes['coll-trigger_function'] =
+    pgBrowser.Nodes['coll-trigger_function'] =
       pgBrowser.Collection.extend({
         node: 'trigger_function',
         label: gettext('Trigger functions'),
         type: 'coll-trigger_function',
         columns: ['name', 'funcowner', 'description'],
-        hasStatistics: true
+        hasStatistics: true,
       });
-  };
+  }
 
   if (!pgBrowser.Nodes['trigger_function']) {
-    pgBrowser.Nodes['trigger_function'] = pgBrowser.Node.extend({
+    pgBrowser.Nodes['trigger_function'] = schemaChild.SchemaChildNode.extend({
       type: 'trigger_function',
       sqlAlterHelp: 'plpgsql-trigger.html',
       sqlCreateHelp: 'plpgsql-trigger.html',
@@ -27,11 +30,10 @@ define('pgadmin.node.trigger_function', [
       hasSQL: true,
       hasDepends: true,
       hasStatistics: true,
-      parent_type: ['schema', 'catalog'],
-      Init: function(args) {
+      Init: function() {
         /* Avoid mulitple registration of menus */
         if (this.initialized)
-            return;
+          return;
 
         this.initialized = true;
 
@@ -40,31 +42,30 @@ define('pgadmin.node.trigger_function', [
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Trigger function...'),
           icon: 'wcTabIcon icon-trigger_function', data: {action: 'create', check: true},
-          enable: 'canCreate'
+          enable: 'canCreate',
         },{
           name: 'create_trigger_function', node: 'trigger_function', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Trigger function...'),
           icon: 'wcTabIcon icon-trigger_function', data: {action: 'create', check: true},
-          enable: 'canCreate'
+          enable: 'canCreate',
         },{
           name: 'create_trigger_function', node: 'schema', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 4, label: gettext('Trigger function...'),
           icon: 'wcTabIcon icon-trigger_function', data: {action: 'create', check: false},
-          enable: 'canCreate'
-        }
+          enable: 'canCreate',
+        },
         ]);
 
       },
-      canDrop: pgBrowser.Nodes['schema'].canChildDrop,
-      canDropCascade: pgBrowser.Nodes['schema'].canChildDrop,
       model: pgBrowser.Node.Model.extend({
+        idAttribute: 'oid',
         initialize: function(attrs, args) {
           var isNew = (_.size(attrs) === 0);
           if (isNew) {
             // Set Selected Schema
-            var schema_id = args.node_info.schema._id
+            var schema_id = args.node_info.schema._id;
             this.set({'pronamespace': schema_id}, {silent: true});
 
             // Set Current User
@@ -103,64 +104,67 @@ define('pgadmin.node.trigger_function', [
           seclabels: [],
           acl: [],
           sysfunc: undefined,
-          sysproc: undefined
+          sysproc: undefined,
         },
         schema: [{
           id: 'name', label: gettext('Name'), cell: 'string',
           type: 'text', mode: ['properties', 'create', 'edit'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'text' , mode: ['properties']
+          type: 'text' , mode: ['properties'],
         },{
           id: 'funcowner', label: gettext('Owner'), cell: 'string',
           control: Backform.NodeListByNameControl, node: 'role',  type:
-          'text', disabled: 'isDisabled'
+          'text', disabled: 'isDisabled',
         },{
           id: 'pronamespace', label: gettext('Schema'), cell: 'string',
           control: 'node-list-by-id', type: 'text', cache_level: 'database',
-          node: 'schema', disabled: 'isDisabled', mode: ['create', 'edit']
+          node: 'schema', disabled: 'isDisabled', mode: ['create', 'edit'],
         },{
           id: 'sysfunc', label: gettext('System function?'),
-           cell:'boolean', type: 'switch',
-           mode: ['properties'], visible: 'isVisible'
+          cell:'boolean', type: 'switch',
+          mode: ['properties'], visible: 'isVisible',
         },{
           id: 'sysproc', label: gettext('System procedure?'),
-           cell:'boolean', type: 'switch',
-           mode: ['properties'], visible: 'isVisible'
+          cell:'boolean', type: 'switch',
+          mode: ['properties'], visible: 'isVisible',
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
-          type: 'multiline', disabled: 'isDisabled'
+          type: 'multiline', disabled: 'isDisabled',
         },{
           id: 'pronargs', label: gettext('Argument count'), cell: 'string',
-          type: 'text', group: gettext('Definition'), mode: ['properties']
+          type: 'text', group: gettext('Definition'), mode: ['properties'],
         },{
           id: 'proargs', label: gettext('Arguments'), cell: 'string',
           type: 'text', group: gettext('Definition'), mode: ['properties', 'edit'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'proargtypenames', label: gettext('Signature arguments'), cell:
           'string', type: 'text', group: gettext('Definition'), mode: ['properties'],
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         },{
           id: 'prorettypename', label: gettext('Return type'), cell: 'string',
           control: 'select2', type: 'text', group: gettext('Definition'),
           disabled: 'isDisabled', first_empty: true,
-          select2: { width: "100%", allowClear: false },
+          select2: { width: '100%', allowClear: false },
           mode: ['create'], visible: 'isVisible', options: [
             {label: gettext('trigger'), value: 'trigger'},
-            {label: gettext('event_trigger'), value: 'event_trigger'}
-          ]
+            {label: gettext('event_trigger'), value: 'event_trigger'},
+          ],
         },{
           id: 'prorettypename', label: gettext('Return type'), cell: 'string',
           type: 'text', group: gettext('Definition'),
-          mode: ['properties', 'edit'], disabled: 'isDisabled', visible: 'isVisible'
+          mode: ['properties', 'edit'], disabled: 'isDisabled', visible: 'isVisible',
         },  {
           id: 'lanname', label: gettext('Language'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Definition'),
-          url: 'get_languages', disabled: 'isDisabled', transform: function(d, self) {
-             return _.reject(d, function(o){ return o.label == 'sql' || o.label == 'edbspl'; });
-          }, select2: { allowClear: false }
+          url: 'get_languages', disabled: 'isDisabled',
+          transform: function(d) {
+            return _.reject(d, function(o) {
+              return o.label == 'sql' || o.label == 'edbspl';
+            });
+          }, select2: { allowClear: false },
         },{
           id: 'prosrc', label: gettext('Code'), cell: 'string',
           type: 'text', mode: ['properties', 'create', 'edit'],
@@ -172,21 +176,21 @@ define('pgadmin.node.trigger_function', [
               return false;
             }
             return true;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'probin', label: gettext('Object file'), cell: 'string',
           type: 'text', group: gettext('Definition'), deps: ['lanname'], visible:
           function(m) {
             if (m.get('lanname') == 'c') { return true; }
             return false;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'prosrc_c', label: gettext('Link symbol'), cell: 'string',
           type: 'text', group: gettext('Definition'),  deps: ['lanname'], visible:
           function(m) {
             if (m.get('lanname') == 'c') { return true; }
             return false;
-          }, disabled: 'isDisabled'
+          }, disabled: 'isDisabled',
         },{
           id: 'provolatile', label: gettext('Volatility'), cell: 'string',
           control: 'node-ajax-options', type: 'text', group: gettext('Options'),
@@ -194,67 +198,67 @@ define('pgadmin.node.trigger_function', [
             {'label': 'VOLATILE', 'value': 'v'},
             {'label': 'STABLE', 'value': 's'},
             {'label': 'IMMUTABLE', 'value': 'i'},
-          ], disabled: 'isDisabled', select2: { allowClear: false }
+          ], disabled: 'isDisabled', select2: { allowClear: false },
         },{
           id: 'proretset', label: gettext('Returns a set?'), type: 'switch',
           group: gettext('Options'), disabled: 'isDisabled',
-          visible: 'isVisible'
+          visible: 'isVisible',
         },{
           id: 'proisstrict', label: gettext('Strict?'), type: 'switch',
           disabled: 'isDisabled', group: gettext('Options'),
           options: {
             'onText': gettext('Yes'), 'offText': gettext('No'),
             'onColor': 'success', 'offColor': 'primary',
-            'size': 'small'
-           }
+            'size': 'small',
+          },
         },{
           id: 'prosecdef', label: gettext('Security of definer?'),
-           group: gettext('Options'), cell:'boolean', type: 'switch',
-           disabled: 'isDisabled'
+          group: gettext('Options'), cell:'boolean', type: 'switch',
+          disabled: 'isDisabled',
         },{
           id: 'proiswindow', label: gettext('Window?'),
-           group: gettext('Options'), cell:'boolean', type: 'switch',
-            disabled: 'isDisabled', visible: 'isVisible'
+          group: gettext('Options'), cell:'boolean', type: 'switch',
+          disabled: 'isDisabled', visible: 'isVisible',
         },{
           id: 'procost', label: gettext('Estimated cost'), type: 'text',
-          group: gettext('Options'), disabled: 'isDisabled'
+          group: gettext('Options'), disabled: 'isDisabled',
         },{
           id: 'prorows', label: gettext('Estimated rows'), type: 'text',
           group: gettext('Options'),
           disabled: 'isDisabled',
-          deps: ['proretset'], visible: 'isVisible'
+          deps: ['proretset'], visible: 'isVisible',
         },{
           id: 'proleakproof', label: gettext('Leak proof?'),
           group: gettext('Options'), cell:'boolean', type: 'switch', min_version: 90200,
-          disabled: 'isDisabled'
+          disabled: 'isDisabled',
         }, pgBrowser.SecurityGroupSchema, {
           id: 'proacl', label: gettext('Privileges'), mode: ['properties'],
-           group: gettext('Security'), type: 'text'
+          group: gettext('Security'), type: 'text',
         },{
           id: 'variables', label: gettext('Parameters'), type: 'collection',
           group: gettext('Parameters'), control: 'variable-collection',
           model: pgBrowser.Node.VariableModel,
           mode: ['edit', 'create'], canAdd: 'canVarAdd', canEdit: false,
-          canDelete: true, disabled: 'isDisabled'
-         },{
+          canDelete: true, disabled: 'isDisabled',
+        },{
           id: 'acl', label: gettext('Privileges'), editable: false,
           type: 'collection', group: 'security', mode: ['edit', 'create'],
           model: pgBrowser.Node.PrivilegeRoleModel.extend({
-            privileges: ['X']
+            privileges: ['X'],
           }), uniqueCol : ['grantee', 'grantor'], disabled: 'isDisabled',
-          canAdd: true, canDelete: true, control: 'unique-col-collection'
+          canAdd: true, canDelete: true, control: 'unique-col-collection',
         },{
           id: 'seclabels', label: gettext('Security Labels'), canEdit: true,
           model: pgBrowser.SecLabelModel, type: 'collection',
           min_version: 90100, group: 'security', mode: ['edit', 'create'],
-           canDelete: true, control: 'unique-col-collection', canAdd: true,
-          uniqueCol : ['provider'], disabled: 'isDisabled'
+          canDelete: true, control: 'unique-col-collection', canAdd: true,
+          uniqueCol : ['provider'], disabled: 'isDisabled',
         }],
         validate: function(keys)
         {
           var err = {},
-              errmsg,
-              seclabels = this.get('seclabels');
+            errmsg,
+            seclabels = this.get('seclabels');
 
           // Nothing to validate
           if(keys && keys.length == 0) {
@@ -326,73 +330,35 @@ define('pgadmin.node.trigger_function', [
 
           return null;
         },
-        isVisible: function(m){
+        isVisible: function() {
           if (this.name == 'sysproc') { return false; }
           return true;
         },
-        isDisabled: function(m){
+        isDisabled: function(m) {
           if(this.node_info &&  'catalog' in this.node_info) {
             return true;
           }
-          name = this.name;
-          switch(name){
-            case 'proargs':
-            case 'proargtypenames':
-            case 'prorettypename':
-            case 'proretset':
-            case 'proiswindow':
-              return !m.isNew();
-              break;
-            case 'prorows':
-              if(m.get('proretset') == true) {
-                return false;
-              }
-              else {
-                return true;
-              }
-              break;
-            default:
+          switch(this.name){
+          case 'proargs':
+          case 'proargtypenames':
+          case 'prorettypename':
+          case 'proretset':
+          case 'proiswindow':
+            return !m.isNew();
+          case 'prorows':
+            if(m.get('proretset') == true) {
               return false;
-              break;
-          }
-          return false;
-        },
-        canVarAdd: function(m) {
-          if(this.node_info &&  'catalog' in this.node_info) {
+            }
+            return true;
+          default:
             return false;
           }
-         return true;
-        }
+        },
+        canVarAdd: function() {
+          return !(this.node_info &&  'catalog' in this.node_info);
+        },
       }),
-      canCreate: function(itemData, item, data) {
-        //If check is false then , we will allow create menu
-        if (data && data.check == false)
-          return true;
-
-        var t = pgBrowser.tree, i = item, d = itemData;
-        // To iterate over tree to check parent node
-        while (i) {
-          // If it is schema then allow user to create Function
-          if (_.indexOf(['schema'], d._type) > -1)
-            return true;
-
-          if ('coll-trigger_function' == d._type) {
-            //Check if we are not child of catalog
-            var prev_i = t.hasParent(i) ? t.parent(i) : null,
-              prev_d = prev_i ? t.itemData(prev_i) : null;
-            if( prev_d._type == 'catalog') {
-              return false;
-            } else {
-              return true;
-            }
-          }
-          i = t.hasParent(i) ? t.parent(i) : null;
-          d = i ? t.itemData(i) : null;
-        }
-        // by default we do not want to allow create menu
-        return true;
-      }
-  });
+    });
 
   }
 
