@@ -13,9 +13,12 @@ import copy
 import re
 from functools import wraps
 
-import pgadmin.browser.server_groups.servers.databases.schemas.packages as packages
 from flask import render_template, make_response
-from flask_babel import gettext
+from flask_babelex import gettext
+
+import pgadmin.browser.server_groups.servers.databases\
+    .schemas.packages as packages
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.server_groups.servers.databases.schemas.utils import \
     DataTypeReader
@@ -26,8 +29,6 @@ from pgadmin.utils.ajax import precondition_required
 from pgadmin.utils.driver import get_driver
 from pgadmin.utils.preferences import Preferences
 
-from config import PG_DEFAULT_DRIVER
-
 
 class EdbFuncModule(CollectionNodeModule):
     """
@@ -125,8 +126,8 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
     """
     class EdbFuncView(PGChildNodeView, DataTypeReader)
 
-    This class inherits PGChildNodeView and DataTypeReader to get the different routes for
-    the module.
+    This class inherits PGChildNodeView and DataTypeReader to get the different
+    routes for the module.
 
     The class is responsible to Create, Read, Update and Delete operations for
     the Functions.
@@ -390,32 +391,13 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
         proargmodenames = {'i': 'IN', 'o': 'OUT', 'b': 'INOUT',
                            'v': 'VARIADIC', 't': 'TABLE'}
 
-        # The proargtypes doesn't give OUT params, so we need to fetch
-        # those from database explicitly, below code is written for this
-        # purpose.
-        #
-        # proallargtypes gives all the Function's argument including OUT,
-        # but we have not used that column; as the data type of this
-        # column (i.e. oid[]) is not supported by oidvectortypes(oidvector)
-        # function which we have used to fetch the datatypes
-        # of the other parameters.
+        # EPAS explicitly converts OUT to INOUT, So we always have proargtypes
 
         proargmodes_fltrd = copy.deepcopy(proargmodes)
         proargnames_fltrd = []
         cnt = 0
         for m in proargmodes:
-            if m == 'o':  # Out Mode
-                SQL = render_template("/".join([self.sql_template_path,
-                                                'get_out_types.sql']),
-                                      out_arg_oid=proallargtypes[cnt])
-                status, out_arg_type = self.conn.execute_scalar(SQL)
-                if not status:
-                    return internal_server_error(errormsg=out_arg_type)
-
-                # Insert out parameter datatype
-                proargtypes.insert(cnt, out_arg_type)
-                proargdefaultvals.insert(cnt, '')
-            elif m == 'v':  # Variadic Mode
+            if m in ['v', 'o']:  # Out / Variadic Mode
                 proargdefaultvals.insert(cnt, '')
             elif m == 't':  # Table Mode
                 proargmodes_fltrd.remove(m)
@@ -461,7 +443,7 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
             proargnames[i] if len(proargnames) > i else '',
             proargdefaultvals[i] if len(proargdefaultvals) > i else ''
         )
-                   for i in range(len(proargtypes))]
+            for i in range(len(proargtypes))]
 
         proargs = {"proargs": ", ".join(proargs)}
 
@@ -529,9 +511,9 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
             scid: Schema Id
             fnid: Function Id
         """
-        SQL = render_template("/".join([self.sql_template_path, 'get_body.sql']),
-                              scid=scid,
-                              pkgid=pkgid)
+        SQL = render_template(
+            "/".join([self.sql_template_path, 'get_body.sql']),
+            edbfnid=edbfnid)
 
         status, res = self.conn.execute_dict(SQL)
         if not status:
@@ -541,7 +523,7 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
                 gettext("Could not find the function in the database.")
             )
 
-        body = self.get_inner(res['rows'][0]['pkgbodysrc'])
+        body = res['rows'][0]['funcdef']
 
         if body is None:
             body = ''
@@ -620,6 +602,7 @@ class EdbFuncView(PGChildNodeView, DataTypeReader):
 
         return sql[start:end].strip("\n")
 
+
 EdbFuncView.register_node_view(blueprint)
 
 
@@ -709,6 +692,7 @@ class EdbProcModule(CollectionNodeModule):
         """
         return False
 
+
 procedure_blueprint = EdbProcModule(__name__)
 
 
