@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,7 +14,7 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as database
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.server_groups.servers.databases.schemas.utils \
     import SchemaChildModule
 from pgadmin.browser.utils import PGChildNodeView
@@ -24,6 +24,7 @@ from pgadmin.utils.ajax import precondition_required
 from pgadmin.utils.driver import get_driver
 from config import PG_DEFAULT_DRIVER
 from pgadmin.utils import IS_PY2
+
 # If we are in Python3
 if not IS_PY2:
     unicode = str
@@ -164,9 +165,9 @@ class SynonymView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -198,12 +199,13 @@ class SynonymView(PGChildNodeView):
             if not self.conn.connected():
                 return precondition_required(
                     gettext(
-                        "Connection to the server has been lost!"
+                        "Connection to the server has been lost."
                     )
                 )
 
             # we will set template path for sql scripts
-            self.template_path = 'synonym/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'synonym/sql/#{0}#'.format(
+                self.manager.version)
 
             return f(*args, **kwargs)
 
@@ -212,7 +214,8 @@ class SynonymView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid, did, scid):
         """
-        This function is used to list all the synonym nodes within that collection.
+        This function is used to list all the synonym nodes within that
+        collection.
 
         Args:
             gid: Server group ID
@@ -238,7 +241,8 @@ class SynonymView(PGChildNodeView):
     @check_precondition
     def nodes(self, gid, sid, did, scid):
         """
-        This function will used to create all the child node within that collection.
+        This function will used to create all the child node within that
+        collection.
         Here it will create all the synonym node.
 
         Args:
@@ -335,19 +339,23 @@ class SynonymView(PGChildNodeView):
                 data[k] = v
 
         is_valid_request = True
-        if 'trgTyp' not in data or data['trgTyp'] is None or \
-            data['trgTyp'].strip() == '':
+        if (
+            'trgTyp' not in data or data['trgTyp'] is None or
+            data['trgTyp'].strip() == ''
+        ):
             is_valid_request = False
 
-        if 'trgSchema' not in data or data['trgSchema'] is None or \
-            data['trgSchema'].strip() == '':
+        if (
+            'trgSchema' not in data or data['trgSchema'] is None or
+            data['trgSchema'].strip() == ''
+        ):
             is_valid_request = False
 
         if is_valid_request:
             sql = render_template("/".join([self.template_path,
                                             'get_objects.sql']),
-                                trgTyp=data['trgTyp'],
-                                trgSchema=data['trgSchema'])
+                                  trgTyp=data['trgTyp'],
+                                  trgSchema=data['trgSchema'])
             status, rset = self.conn.execute_dict(sql)
 
             if not status:
@@ -460,7 +468,7 @@ class SynonymView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, syid):
+    def delete(self, gid, sid, did, scid, syid=None):
         """
         This function will delete existing the synonym object
 
@@ -471,42 +479,44 @@ class SynonymView(PGChildNodeView):
            scid: Schema ID
            syid: Synonym ID
         """
+        if syid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [syid]}
 
         # Below will decide if it's simple drop or drop with cascade call
 
         try:
-            SQL = render_template("/".join([self.template_path,
-                                            'properties.sql']),
-                                  scid=scid, syid=syid)
+            for syid in data['ids']:
+                SQL = render_template("/".join([self.template_path,
+                                                'properties.sql']),
+                                      scid=scid, syid=syid)
 
-            status, res = self.conn.execute_dict(SQL)
+                status, res = self.conn.execute_dict(SQL)
 
-            if not status:
-                return internal_server_error(errormsg=res)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            if len(res['rows']) > 0:
-                data = res['rows'][0]
-            else:
-                return gone(
-                    gettext('The specified synonym could not be found.')
-                )
+                if len(res['rows']) > 0:
+                    data = res['rows'][0]
+                else:
+                    return gone(
+                        gettext('The specified synonym could not be found.')
+                    )
 
-            SQL = render_template("/".join([self.template_path,
-                                            'delete.sql']),
-                                  data=data,
-                                  conn=self.conn)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      data=data,
+                                      conn=self.conn)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Synonym dropped"),
-                data={
-                    'id': syid,
-                    'scid': scid,
-                    'did': did
-                }
+                info=gettext("Synonym dropped")
             )
 
         except Exception as e:
@@ -642,7 +652,7 @@ class SynonymView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) > 0:
-           data = res['rows'][0]
+            data = res['rows'][0]
         else:
             return gone(
                 gettext('The specified synonym could not be found.')
