@@ -14,10 +14,10 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as database
 from flask import render_template, make_response, request, jsonify
-from flask_babel import gettext as _
+from flask_babelex import gettext as _
 from pgadmin.browser.collection import CollectionNodeModule
-from pgadmin.browser.server_groups.servers.databases.schemas.tables.constraints.type \
-    import ConstraintRegistry
+from pgadmin.browser.server_groups.servers.databases.schemas.tables.\
+    constraints.type import ConstraintRegistry
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
@@ -75,7 +75,6 @@ class CheckConstraintModule(CollectionNodeModule):
         """
         return database.DatabaseModule.NODE_TYPE
 
-
     @property
     def module_use_template_javascript(self):
         """
@@ -84,7 +83,6 @@ class CheckConstraintModule(CollectionNodeModule):
         """
         return False
 
-
     @property
     def csssnippets(self):
         """
@@ -175,9 +173,9 @@ class CheckConstraintView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -217,7 +215,8 @@ class CheckConstraintView(PGChildNodeView):
             self.qtIdent = driver.qtIdent
 
             # Set the template path for the SQL scripts
-            self.template_path = 'check_constraint/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'check_constraint/sql/#{0}#'.format(
+                self.manager.version)
 
             SQL = render_template("/".join([self.template_path,
                                             'get_parent.sql']),
@@ -286,7 +285,8 @@ class CheckConstraintView(PGChildNodeView):
         self.qtIdent = driver.qtIdent
 
         # Set the template path for the SQL scripts
-        self.template_path = 'check_constraint/sql/#{0}#'.format(self.manager.version)
+        self.template_path = 'check_constraint/sql/#{0}#'.format(
+            self.manager.version)
 
         SQL = render_template("/".join([self.template_path,
                                         'get_parent.sql']),
@@ -325,19 +325,20 @@ class CheckConstraintView(PGChildNodeView):
         if len(rset['rows']) == 0:
             return gone(_("""Could not find the check constraint."""))
 
-        if "convalidated" in rset['rows'][0] and rset['rows'][0]["convalidated"]:
+        if "convalidated" in rset['rows'][0] and \
+                rset['rows'][0]["convalidated"]:
             icon = "icon-check_constraint_bad"
             valid = False
         else:
             icon = "icon-check_constraint"
             valid = True
         res = self.blueprint.generate_browser_node(
-                rset['rows'][0]['oid'],
-                tid,
-                rset['rows'][0]['name'],
-                icon=icon,
-                valid=valid
-            )
+            rset['rows'][0]['oid'],
+            tid,
+            rset['rows'][0]['name'],
+            icon=icon,
+            valid=valid
+        )
         return make_json_response(
             data=res,
             status=200
@@ -403,7 +404,8 @@ class CheckConstraintView(PGChildNodeView):
         self.qtIdent = driver.qtIdent
 
         # Set the template path for the SQL scripts
-        self.template_path = 'check_constraint/sql/#{0}#'.format(self.manager.version)
+        self.template_path = 'check_constraint/sql/#{0}#'.format(
+            self.manager.version)
 
         SQL = render_template("/".join([self.template_path,
                                         'get_parent.sql']),
@@ -546,15 +548,18 @@ class CheckConstraintView(PGChildNodeView):
                 data['name'] = res['rows'][0]['name']
 
             else:
-                sql = render_template("/".join([self.template_path, 'get_oid.sql']),
-                                      tid=tid,
-                                      name=data['name'])
+                sql = render_template(
+                    "/".join([self.template_path, 'get_oid.sql']),
+                    tid=tid,
+                    name=data['name']
+                )
                 status, res = self.conn.execute_dict(sql)
                 if not status:
                     self.end_transaction()
                     return internal_server_error(errormsg=res)
 
-            if "convalidated" in res['rows'][0] and res['rows'][0]["convalidated"]:
+            if "convalidated" in res['rows'][0] and \
+                    res['rows'][0]["convalidated"]:
                 icon = "icon-check_constraint_bad"
                 valid = False
             else:
@@ -580,7 +585,7 @@ class CheckConstraintView(PGChildNodeView):
             )
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, tid, cid):
+    def delete(self, gid, sid, did, scid, tid, cid=None):
         """
         Drops the Check Constraint object.
 
@@ -592,45 +597,47 @@ class CheckConstraintView(PGChildNodeView):
             tid: Check Id
             cid: Check Constraint Id
         """
+        if cid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [cid]}
+
         try:
-            SQL = render_template("/".join([self.template_path,
-                                            'properties.sql']),
-                                  tid=tid, cid=cid)
-            status, res = self.conn.execute_dict(SQL)
+            for cid in data['ids']:
+                SQL = render_template("/".join([self.template_path,
+                                                'properties.sql']),
+                                      tid=tid, cid=cid)
+                status, res = self.conn.execute_dict(SQL)
 
-            if not status:
-                return internal_server_error(errormsg=res)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            if not res['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=_(
-                        'Error: Object not found.'
-                    ),
-                    info=_(
-                        'The specified check constraint could not be found.\n'
+                if not res['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=_(
+                            'Error: Object not found.'
+                        ),
+                        info=_(
+                            'The specified check constraint '
+                            'could not be found.\n'
+                        )
                     )
-                )
 
-            data = res['rows'][0]
+                data = res['rows'][0]
 
-            SQL = render_template("/".join([self.template_path,
-                                            'delete.sql']),
-                                  data=data)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      data=data)
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=_("Check constraint dropped."),
-                data={
-                    'id': tid,
-                    'scid': scid,
-                    'sid': sid,
-                    'gid': gid,
-                    'did': did
-                }
+                info=_("Check constraint dropped.")
             )
 
         except Exception as e:
@@ -666,13 +673,14 @@ class CheckConstraintView(PGChildNodeView):
             if not status:
                 return internal_server_error(errormsg=res)
 
-            sql = render_template("/".join([self.template_path, 'get_name.sql']),
-                                  cid=cid)
+            sql = render_template(
+                "/".join([self.template_path, 'get_name.sql']), cid=cid)
             status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
 
-            if "convalidated" in res['rows'][0] and res['rows'][0]["convalidated"]:
+            if "convalidated" in res['rows'][0] and \
+                    res['rows'][0]["convalidated"]:
                 icon = 'icon-check_constraint_bad'
                 valid = False
             else:
@@ -884,13 +892,15 @@ class CheckConstraintView(PGChildNodeView):
         try:
             data['schema'] = self.schema
             data['table'] = self.table
-            sql = render_template("/".join([self.template_path, 'get_name.sql']), cid=cid)
+            sql = render_template(
+                "/".join([self.template_path, 'get_name.sql']), cid=cid)
             status, res = self.conn.execute_scalar(sql)
             if not status:
                 return internal_server_error(errormsg=res)
 
             data['name'] = res
-            sql = render_template("/".join([self.template_path, 'validate.sql']), data=data)
+            sql = render_template(
+                "/".join([self.template_path, 'validate.sql']), data=data)
             status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
