@@ -14,11 +14,14 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as database
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
+from pgadmin.browser.server_groups.servers.databases.schemas.tables.\
+    partitions import backend_supported
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
+from pgadmin.utils.compile_template_name import compile_template_path
 from pgadmin.utils.driver import get_driver
 from config import PG_DEFAULT_DRIVER
 from pgadmin.utils import IS_PY2
@@ -72,19 +75,10 @@ class IndexesModule(CollectionNodeModule):
         if super(IndexesModule, self).BackendSupported(manager, **kwargs):
             conn = manager.connection(did=kwargs['did'])
 
-            # In case of partitioned table return false.
-            if 'tid' in kwargs and manager.version >= 100000:
-                partition_path = 'partition/sql/#{0}#'.format(manager.version)
-                SQL = render_template(
-                    "/".join([partition_path, 'backend_support.sql']),
-                    tid=kwargs['tid']
-                )
-                status, res = conn.execute_scalar(SQL)
-
-                # check if any errors
-                if not status:
-                    return internal_server_error(errormsg=res)
-                return not res
+            # If PG version > 100000 and < 110000 then index is
+            # not supported for partitioned table.
+            if 'tid' in kwargs and 100000 <= manager.version < 110000:
+                return not backend_supported(self, manager, **kwargs)
 
             if 'vid' not in kwargs:
                 return True
@@ -217,9 +211,9 @@ class IndexesView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -258,7 +252,11 @@ class IndexesView(PGChildNodeView):
                 kwargs['did'] in self.manager.db_info else 0
 
             # we will set template path for sql scripts
-            self.template_path = 'index/sql/#{0}#'.format(self.manager.version)
+            self.template_path = compile_template_path(
+                'index/sql/',
+                self.manager.server_type,
+                self.manager.version
+            )
 
             # We need parent's name eg table name and schema name
             # when we create new index in update we can fetch it using
@@ -377,7 +375,8 @@ class IndexesView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid, did, scid, tid):
         """
-        This function is used to list all the schema nodes within that collection.
+        This function is used to list all the schema nodes within that
+        collection.
 
         Args:
             gid: Server group ID
@@ -405,8 +404,8 @@ class IndexesView(PGChildNodeView):
     @check_precondition
     def node(self, gid, sid, did, scid, tid, idx):
         """
-        This function will used to create all the child node within that collection.
-        Here it will create all the schema node.
+        This function will used to create all the child node within that
+        collection. Here it will create all the schema node.
 
         Args:
             gid: Server Group ID
@@ -431,11 +430,11 @@ class IndexesView(PGChildNodeView):
             return gone(gettext("""Could not find the index in the table."""))
 
         res = self.blueprint.generate_browser_node(
-                rset['rows'][0]['oid'],
-                tid,
-                rset['rows'][0]['name'],
-                icon="icon-index"
-            )
+            rset['rows'][0]['oid'],
+            tid,
+            rset['rows'][0]['name'],
+            icon="icon-index"
+        )
 
         return make_json_response(
             data=res,
@@ -445,8 +444,8 @@ class IndexesView(PGChildNodeView):
     @check_precondition
     def nodes(self, gid, sid, did, scid, tid):
         """
-        This function will used to create all the child node within that collection.
-        Here it will create all the schema node.
+        This function will used to create all the child node within that
+        collection. Here it will create all the schema node.
 
         Args:
             gid: Server Group ID
@@ -529,10 +528,35 @@ class IndexesView(PGChildNodeView):
         # Push as collection
         data['columns'] = columns
         # Push as string
-        data['cols'] = ', '.join(cols)
+        data['columns_csv'] = ', '.join(cols)
 
         return data
 
+    def _include_details(self, idx, data, mode='properties'):
+        """
+        This functional will fetch list of include details for index
+        supported with Postgres 11+
+
+        Args:
+            idx: Index OID
+            data: Properties data
+
+        Returns:
+            Updated properties data with include details
+        """
+
+        SQL = render_template(
+            "/".join([self.template_path, 'include_details.sql']), idx=idx
+        )
+        status, rset = self.conn.execute_2darray(SQL)
+
+        if not status:
+            return internal_server_error(errormsg=rset)
+
+        # Push as collection
+        data['include'] = [col['colname'] for col in rset['rows']]
+        return data
+
     @check_precondition
     def properties(self, gid, sid, did, scid, tid, idx):
         """
@@ -570,6 +594,10 @@ class IndexesView(PGChildNodeView):
         # Add column details for current index
         data = self._column_details(idx, data)
 
+        # Add Include details of the index
+        if self.manager.version >= 110000:
+            data = self._include_details(idx, data)
+
         return ajax_response(
             response=data,
             status=200
@@ -676,7 +704,7 @@ class IndexesView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, tid, idx):
+    def delete(self, gid, sid, did, scid, tid, idx=None):
         """
         This function will updates existing the schema object
 
@@ -688,6 +716,13 @@ class IndexesView(PGChildNodeView):
            tid: Table ID
            idx: Index ID
         """
+        if idx is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [idx]}
+
         # Below will decide if it's simple drop or drop with cascade call
         if self.cmd == 'delete':
             # This is a cascade operation
@@ -696,45 +731,42 @@ class IndexesView(PGChildNodeView):
             cascade = False
 
         try:
-            # We will first fetch the index name for current request
-            # so that we create template for dropping index
-            SQL = render_template(
-                "/".join([self.template_path, 'properties.sql']),
-                did=did, tid=tid, idx=idx, datlastsysoid=self.datlastsysoid
-            )
-
-            status, res = self.conn.execute_dict(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
-
-            if not res['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified index could not be found.\n'
-                    )
+            for idx in data['ids']:
+                # We will first fetch the index name for current request
+                # so that we create template for dropping index
+                SQL = render_template(
+                    "/".join([self.template_path, 'properties.sql']),
+                    did=did, tid=tid, idx=idx, datlastsysoid=self.datlastsysoid
                 )
 
-            data = dict(res['rows'][0])
+                status, res = self.conn.execute_dict(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            SQL = render_template(
-                "/".join([self.template_path, 'delete.sql']),
-                data=data, conn=self.conn, cascade=cascade
-            )
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                if not res['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified index could not be found.\n'
+                        )
+                    )
+
+                data = dict(res['rows'][0])
+
+                SQL = render_template(
+                    "/".join([self.template_path, 'delete.sql']),
+                    data=data, conn=self.conn, cascade=cascade
+                )
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Index is dropped"),
-                data={
-                    'id': idx,
-                    'tid': tid
-                }
+                info=gettext("Index is dropped")
             )
 
         except Exception as e:
@@ -830,7 +862,9 @@ class IndexesView(PGChildNodeView):
             if not status:
                 return internal_server_error(errormsg=res)
             if len(res['rows']) == 0:
-                return gone(gettext("""Could not find the index in the table."""))
+                return gone(
+                    gettext("""Could not find the index in the table.""")
+                )
 
             old_data = dict(res['rows'][0])
 
@@ -905,6 +939,10 @@ class IndexesView(PGChildNodeView):
         # Add column details for current index
         data = self._column_details(idx, data, 'create')
 
+        # Add Include details of the index
+        if self.manager.version >= 110000:
+            data = self._include_details(idx, data, 'create')
+
         SQL, name = self.get_sql(did, scid, tid, None, data)
         if not isinstance(SQL, (str, unicode)):
             return SQL
