@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,7 +14,7 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as database
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
@@ -62,6 +62,7 @@ class TriggerModule(CollectionNodeModule):
         """
         self.min_ver = None
         self.max_ver = None
+        self.min_gpdbver = 1000000000
         super(TriggerModule, self).__init__(*args, **kwargs)
 
     def BackendSupported(self, manager, **kwargs):
@@ -69,6 +70,8 @@ class TriggerModule(CollectionNodeModule):
         Load this module if vid is view, we will not load it under
         material view
         """
+        if manager.server_type == 'gpdb':
+            return False
         if super(TriggerModule, self).BackendSupported(manager, **kwargs):
             conn = manager.connection(did=kwargs['did'])
 
@@ -114,6 +117,14 @@ class TriggerModule(CollectionNodeModule):
         """
         return False
 
+    @property
+    def module_use_template_javascript(self):
+        """
+        Returns whether Jinja2 template is used for generating the javascript
+        module.
+        """
+        return False
+
     @property
     def csssnippets(self):
         """
@@ -227,9 +238,9 @@ class TriggerView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{'delete': 'delete'}],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'children': [{'get': 'children'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -258,14 +269,16 @@ class TriggerView(PGChildNodeView):
                 kwargs['sid']
             )
             self.conn = self.manager.connection(did=kwargs['did'])
-            # We need datlastsysoid to check if current trigger is system trigger
+            # We need datlastsysoid to check if current trigger is system
+            # trigger
             self.datlastsysoid = self.manager.db_info[
                 kwargs['did']
             ]['datlastsysoid'] if self.manager.db_info is not None and \
                 kwargs['did'] in self.manager.db_info else 0
 
             # we will set template path for sql scripts
-            self.template_path = 'trigger/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'trigger/sql/#{0}#'.format(
+                self.manager.version)
             # Store server type
             self.server_type = self.manager.server_type
             # We need parent's name eg table name and schema name
@@ -316,10 +329,10 @@ class TriggerView(PGChildNodeView):
                 'value': 'Inline EDB-SPL'
             })
         try:
-            SQL = render_template("/".join([self.template_path,
-                                            'get_triggerfunctions.sql']),
-                                  show_system_objects=self.blueprint.show_system_objects
-                                  )
+            SQL = render_template(
+                "/".join([self.template_path, 'get_triggerfunctions.sql']),
+                show_system_objects=self.blueprint.show_system_objects
+            )
             status, rset = self.conn.execute_2darray(SQL)
             if not status:
                 return internal_server_error(errormsg=res)
@@ -340,7 +353,8 @@ class TriggerView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid, did, scid, tid):
         """
-        This function is used to list all the trigger nodes within that collection.
+        This function is used to list all the trigger nodes within that
+        collection.
 
         Args:
             gid: Server group ID
@@ -367,7 +381,8 @@ class TriggerView(PGChildNodeView):
     @check_precondition
     def node(self, gid, sid, did, scid, tid, trid):
         """
-        This function will used to create the child node within that collection.
+        This function will used to create the child node within that
+        collection.
         Here it will create specific the trigger node.
 
         Args:
@@ -391,15 +406,17 @@ class TriggerView(PGChildNodeView):
             return internal_server_error(errormsg=rset)
 
         if len(rset['rows']) == 0:
-            return gone(gettext("""Could not find the trigger in the table."""))
+            return gone(
+                gettext("""Could not find the trigger in the table.""")
+            )
 
         res = self.blueprint.generate_browser_node(
-                rset['rows'][0]['oid'],
-                tid,
-                rset['rows'][0]['name'],
-                icon="icon-trigger" if rset['rows'][0]['is_enable_trigger']
-                else "icon-trigger-bad"
-            )
+            rset['rows'][0]['oid'],
+            tid,
+            rset['rows'][0]['name'],
+            icon="icon-trigger" if
+            rset['rows'][0]['is_enable_trigger'] else "icon-trigger-bad"
+        )
 
         return make_json_response(
             data=res,
@@ -409,7 +426,8 @@ class TriggerView(PGChildNodeView):
     @check_precondition
     def nodes(self, gid, sid, did, scid, tid):
         """
-        This function will used to create all the child node within that collection.
+        This function will used to create all the child node within that
+        collection.
         Here it will create all the trigger node.
 
         Args:
@@ -531,7 +549,6 @@ class TriggerView(PGChildNodeView):
         formatted_args = ["'{0}'".format(arg) for arg in args]
         return ', '.join(formatted_args)
 
-
     @check_precondition
     def properties(self, gid, sid, did, scid, tid, trid):
         """
@@ -561,7 +578,8 @@ class TriggerView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the trigger in the table."""))
+            return gone(
+                gettext("""Could not find the trigger in the table."""))
 
         # Making copy of output for future use
         data = dict(res['rows'][0])
@@ -615,8 +633,10 @@ class TriggerView(PGChildNodeView):
                 return make_json_response(
                     status=410,
                     success=0,
-                    errormsg=gettext("Could not find the required parameter (%s)." % \
-                                     required_args[arg])
+                    errormsg=gettext(
+                        "Could not find the required parameter (%s)." %
+                        required_args[arg]
+                    )
                 )
 
         # Adding parent into data dict, will be using it while creating sql
@@ -651,7 +671,7 @@ class TriggerView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, tid, trid):
+    def delete(self, gid, sid, did, scid, tid, trid=None):
         """
         This function will updates existing the trigger object
 
@@ -663,6 +683,13 @@ class TriggerView(PGChildNodeView):
            tid: Table ID
            trid: Trigger ID
         """
+        if trid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [trid]}
+
         # Below will decide if it's simple drop or drop with cascade call
         if self.cmd == 'delete':
             # This is a cascade operation
@@ -671,44 +698,44 @@ class TriggerView(PGChildNodeView):
             cascade = False
 
         try:
-            # We will first fetch the trigger name for current request
-            # so that we create template for dropping trigger
-            SQL = render_template("/".join([self.template_path,
-                                            'properties.sql']),
-                                  tid=tid, trid=trid,
-                                  datlastsysoid=self.datlastsysoid)
+            for trid in data['ids']:
+                # We will first fetch the trigger name for current request
+                # so that we create template for dropping trigger
+                SQL = render_template("/".join([self.template_path,
+                                                'properties.sql']),
+                                      tid=tid, trid=trid,
+                                      datlastsysoid=self.datlastsysoid)
 
-            status, res = self.conn.execute_dict(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                status, res = self.conn.execute_dict(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
-            if not res['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified trigger could not be found.\n'
+                if not res['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified trigger could not be found.\n'
+                        )
                     )
-                )
 
-            data = dict(res['rows'][0])
+                data = dict(res['rows'][0])
 
-            SQL = render_template("/".join([self.template_path,
-                                            'delete.sql']),
-                                  data=data, conn=self.conn, cascade=cascade)
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                SQL = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      data=data,
+                                      conn=self.conn,
+                                      cascade=cascade
+                                      )
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Trigger is dropped"),
-                data={
-                    'id': trid,
-                    'tid': tid
-                }
+                info=gettext("Trigger is dropped")
             )
 
         except Exception as e:
@@ -746,8 +773,10 @@ class TriggerView(PGChildNodeView):
             # We need oid to add object in browser tree and if user
             # update the trigger then new OID is getting generated
             # so we need to return new OID of trigger.
-            SQL = render_template("/".join([self.template_path, 'get_oid.sql']),
-                                  tid=tid, data=data)
+            SQL = render_template(
+                "/".join([self.template_path, 'get_oid.sql']),
+                tid=tid, data=data
+            )
             status, new_trid = self.conn.execute_scalar(SQL)
             if not status:
                 return internal_server_error(errormsg=new_trid)
@@ -825,23 +854,26 @@ class TriggerView(PGChildNodeView):
         """
         This function will return trigger function with schema name
         """
-        # If language is 'edbspl' then trigger function should be 'Inline EDB-SPL'
-        # else we will find the trigger function with schema name.
+        # If language is 'edbspl' then trigger function should be
+        # 'Inline EDB-SPL' else we will find the trigger function
+        # with schema name.
         if data['lanname'] == 'edbspl':
             data['tfunction'] = 'Inline EDB-SPL'
         else:
-            SQL = render_template("/".join([self.template_path,
-                                            'get_triggerfunctions.sql']),
-                                  tgfoid=data['tgfoid'],
-                                  show_system_objects=self.blueprint.show_system_objects)
+            SQL = render_template(
+                "/".join([self.template_path, 'get_triggerfunctions.sql']),
+                tgfoid=data['tgfoid'],
+                show_system_objects=self.blueprint.show_system_objects
+            )
 
             status, result = self.conn.execute_dict(SQL)
             if not status:
                 return internal_server_error(errormsg=res)
 
-            # Update the trigger function which we have fetched with schema name
+            # Update the trigger function which we have fetched with schema
+            # name
             if 'rows' in result and len(result['rows']) > 0 and \
-                            'tfunctions' in result['rows'][0]:
+                    'tfunctions' in result['rows'][0]:
                 data['tfunction'] = result['rows'][0]['tfunctions']
         return data
 
@@ -877,8 +909,10 @@ class TriggerView(PGChildNodeView):
             old_data = self.get_trigger_function_schema(old_data)
 
             if len(old_data['custom_tgargs']) > 1:
-                # We know that trigger has more than 1 argument, let's join them
-                old_data['tgargs'] = self._format_args(old_data['custom_tgargs'])
+                # We know that trigger has more than 1 argument, let's join
+                # them
+                old_data['tgargs'] = \
+                    self._format_args(old_data['custom_tgargs'])
 
             if len(old_data['tgattr']) > 1:
                 columns = ', '.join(old_data['tgattr'].split(' '))
@@ -928,7 +962,8 @@ class TriggerView(PGChildNodeView):
         if not status:
             return internal_server_error(errormsg=res)
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the trigger in the table."""))
+            return gone(
+                gettext("""Could not find the trigger in the table."""))
 
         data = dict(res['rows'][0])
         # Adding parent into data dict, will be using it while creating sql
@@ -941,7 +976,7 @@ class TriggerView(PGChildNodeView):
             # We know that trigger has more than 1 argument, let's join them
             data['tgargs'] = self._format_args(data['custom_tgargs'])
 
-        if len(data['tgattr']) > 1:
+        if len(data['tgattr']) >= 1:
             columns = ', '.join(data['tgattr'].split(' '))
             data['columns'] = self._column_details(tid, columns)
 
