@@ -25,7 +25,7 @@ define('pgadmin.node.trigger', [
       '  <span class="<%=Backform.helpMessageClassName%>"><%=helpMessage%></span>',
       '<% } %>',
     ].join('\n')),
-    className: 'pgadmin-control-group form-group col-xs-6',
+    className: 'pgadmin-control-group form-group col-6',
   });
 
   if (!pgBrowser.Nodes['coll-trigger']) {
@@ -181,6 +181,7 @@ define('pgadmin.node.trigger', [
       canDrop: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
       canDropCascade: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
       model: pgAdmin.Browser.Node.Model.extend({
+        idAttribute: 'oid',
         defaults: {
           name: undefined,
           is_row_trigger: true,
