@@ -26,11 +26,13 @@ from . import utils as triggers_utils
 
 class TriggersUpdateTestCase(BaseTestGenerator):
     """This class will update trigger under table node."""
+    skip_on_database = ['gpdb']
     scenarios = [
         ('Put trigger Node URL', dict(url='/browser/trigger/obj/'))
     ]
 
     def setUp(self):
+        super(TriggersUpdateTestCase, self).setUp()
         self.db_name = parent_node_dict["database"][-1]["db_name"]
         schema_info = parent_node_dict["schema"][-1]
         self.server_id = schema_info["server_id"]
@@ -54,7 +56,7 @@ class TriggersUpdateTestCase(BaseTestGenerator):
         self.func_name = "trigger_func_add_%s" % str(uuid.uuid4())[1:8]
         self.function_info = \
             trigger_funcs_utils.create_trigger_function_with_trigger(
-            self.server, self.db_name, self.schema_name, self.func_name)
+                self.server, self.db_name, self.schema_name, self.func_name)
         self.trigger_name = "test_trigger_delete_%s" % (str(uuid.uuid4())[1:8])
         self.trigger_id = triggers_utils.create_trigger(self.server,
                                                         self.db_name,
