@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -12,7 +12,7 @@ from functools import wraps
 import json
 
 from flask import render_template, request, jsonify
-from flask_babel import gettext as _
+from flask_babelex import gettext as _
 
 from config import PG_DEFAULT_DRIVER
 
@@ -22,6 +22,7 @@ from pgadmin.browser.server_groups import servers
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone, success_return
 from pgadmin.utils.driver import get_driver
+from pgadmin.utils.preferences import Preferences
 
 
 class JobModule(CollectionNodeModule):
@@ -50,9 +51,14 @@ class JobModule(CollectionNodeModule):
 
         conn = manager.connection()
 
+        if manager.server_type == 'gpdb':
+            return False
+
         status, res = conn.execute_scalar("""
 SELECT
-    has_table_privilege('pgagent.pga_job', 'INSERT, SELECT, UPDATE') has_priviledge
+    has_table_privilege(
+      'pgagent.pga_job', 'INSERT, SELECT, UPDATE'
+    ) has_priviledge
 WHERE EXISTS(
     SELECT has_schema_privilege('pgagent', 'USAGE')
     WHERE EXISTS(
@@ -124,7 +130,7 @@ class JobView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'properties', 'post': 'create'}
+            {'get': 'properties', 'post': 'create', 'delete': 'delete'}
         ],
         'nodes': [{'get': 'nodes'}, {'get': 'nodes'}],
         'sql': [{'get': 'sql'}],
@@ -145,7 +151,11 @@ class JobView(PGChildNodeView):
         @wraps(f)
         def wrap(self, *args, **kwargs):
 
-            self.manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(kwargs['sid'])
+            self.manager = get_driver(
+                PG_DEFAULT_DRIVER
+            ).connection_manager(
+                kwargs['sid']
+            )
             self.conn = self.manager.connection()
 
             # Set the template path for the sql scripts.
@@ -179,9 +189,8 @@ SELECT EXISTS(
         if jid is not None:
             if len(rset['rows']) != 1:
                 return gone(
-                    errormsg=_(
-                        "Could not find the pgAgent job on the server."
-                        ))
+                    errormsg=_("Could not find the pgAgent job on the server.")
+                )
             return make_json_response(
                 data=self.blueprint.generate_browser_node(
                     rset['rows'][0]['jobid'],
@@ -353,23 +362,31 @@ SELECT EXISTS(
                 jid,
                 sid,
                 row['jobname'],
-                icon="icon-pga_job" if row['jobenabled'] else
-                    "icon-pga_job-disabled"
+                icon="icon-pga_job" if row['jobenabled']
+                else "icon-pga_job-disabled"
             )
         )
 
     @check_precondition
-    def delete(self, gid, sid, jid):
+    def delete(self, gid, sid, jid=None):
         """Delete the pgAgent Job."""
 
-        status, res = self.conn.execute_void(
-            render_template(
-                "/".join([self.template_path, 'delete.sql']),
-                jid=jid, conn=self.conn
+        if jid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
             )
-        )
-        if not status:
-            return internal_server_error(errormsg=res)
+        else:
+            data = {'ids': [jid]}
+
+        for jid in data['ids']:
+            status, res = self.conn.execute_void(
+                render_template(
+                    "/".join([self.template_path, 'delete.sql']),
+                    jid=jid, conn=self.conn
+                )
+            )
+            if not status:
+                return internal_server_error(errormsg=res)
 
         return make_json_response(success=1)
 
@@ -407,10 +424,16 @@ SELECT EXISTS(
         otherwise it will return statistics for all the databases in that
         server.
         """
+        pref = Preferences.module('browser')
+        rows_threshold = pref.preference(
+            'pgagent_row_threshold'
+        )
+
         status, res = self.conn.execute_dict(
             render_template(
                 "/".join([self.template_path, 'stats.sql']),
-                jid=jid, conn=self.conn
+                jid=jid, conn=self.conn,
+                rows_threshold=rows_threshold.get()
             )
         )
 
@@ -435,7 +458,6 @@ SELECT EXISTS(
         if not status:
             return internal_server_error(errormsg=res)
 
-
         if len(res['rows']) == 0:
             return gone(
                 _("Could not find the object on the server.")
@@ -443,7 +465,7 @@ SELECT EXISTS(
 
         row = res['rows'][0]
 
-        status, res= self.conn.execute_dict(
+        status, res = self.conn.execute_dict(
             render_template(
                 "/".join([self.template_path, 'steps.sql']),
                 jid=jid, conn=self.conn,
@@ -475,7 +497,7 @@ SELECT EXISTS(
                         'jexdate': schedule['jexdate'][idx],
                         'jextime': schedule['jextime'][idx]
                     })
-                    idx+=1
+                    idx += 1
             del schedule['jexid']
             del schedule['jexdate']
             del schedule['jextime']
@@ -524,4 +546,5 @@ SELECT EXISTS(
             status=200
         )
 
+
 JobView.register_node_view(blueprint)
