@@ -12,7 +12,7 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers as servers
 from flask import render_template, make_response, request, jsonify, current_app
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.server_groups.servers.utils import parse_priv_from_db, \
     parse_priv_to_db
@@ -32,6 +32,10 @@ class TablespaceModule(CollectionNodeModule):
     NODE_TYPE = 'tablespace'
     COLLECTION_LABEL = gettext("Tablespaces")
 
+    def __init__(self, import_name, **kwargs):
+        super(TablespaceModule, self).__init__(import_name, **kwargs)
+        self.min_gpdbver = 90000
+
     def get_nodes(self, gid, sid):
         """
         Generate the collection node
@@ -76,7 +80,7 @@ class TablespaceView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'children': [{'get': 'children'}],
@@ -115,7 +119,11 @@ class TablespaceView(PGChildNodeView):
         def wrap(*args, **kwargs):
             # Here args[0] will hold self & kwargs will hold gid,sid,tsid
             self = args[0]
-            self.manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(kwargs['sid'])
+            self.manager = get_driver(
+                PG_DEFAULT_DRIVER
+            ).connection_manager(
+                kwargs['sid']
+            )
             self.conn = self.manager.connection()
 
             # If DB not connected then return error to browser
@@ -129,7 +137,9 @@ class TablespaceView(PGChildNodeView):
                     )
                 )
 
-            self.template_path = 'tablespaces/sql/#{0}#'.format(self.manager.version)
+            self.template_path = 'tablespaces/sql/#{0}#'.format(
+                self.manager.version
+            )
             current_app.logger.debug(
                 "Using the template path: %s", self.template_path
             )
@@ -169,11 +179,11 @@ class TablespaceView(PGChildNodeView):
             return gone(gettext("""Could not find the tablespace."""))
 
         res = self.blueprint.generate_browser_node(
-                rset['rows'][0]['oid'],
-                sid,
-                rset['rows'][0]['name'],
-                icon="icon-tablespace"
-            )
+            rset['rows'][0]['oid'],
+            sid,
+            rset['rows'][0]['name'],
+            icon="icon-tablespace"
+        )
 
         return make_json_response(
             data=res,
@@ -265,7 +275,9 @@ class TablespaceView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the tablespace information."""))
+            return gone(
+                gettext("""Could not find the tablespace information.""")
+            )
 
         # Making copy of output for future use
         copy_data = dict(res['rows'][0])
@@ -383,51 +395,54 @@ class TablespaceView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, tsid):
+    def delete(self, gid, sid, tsid=None):
         """
         This function will drop the tablespace object
         """
-        try:
-            # Get name for tablespace from tsid
-            status, rset = self.conn.execute_dict(
-                render_template(
-                    "/".join([self.template_path, 'nodes.sql']),
-                    tsid=tsid, conn=self.conn
-                )
+        if tsid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
             )
+        else:
+            data = {'ids': [tsid]}
 
-            if not status:
-                return internal_server_error(errormsg=rset)
-
-            if not rset['rows']:
-                return make_json_response(
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified tablespace could not be found.\n'
+        try:
+            for tsid in data['ids']:
+                # Get name for tablespace from tsid
+                status, rset = self.conn.execute_dict(
+                    render_template(
+                        "/".join([self.template_path, 'nodes.sql']),
+                        tsid=tsid, conn=self.conn
                     )
                 )
 
-            # drop tablespace
-            SQL = render_template(
-                "/".join([self.template_path, 'delete.sql']),
-                tsname=(rset['rows'][0])['name'], conn=self.conn
-            )
+                if not status:
+                    return internal_server_error(errormsg=rset)
 
-            status, res = self.conn.execute_scalar(SQL)
-            if not status:
-                return internal_server_error(errormsg=res)
+                if not rset['rows']:
+                    return make_json_response(
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified tablespace could not be found.\n'
+                        )
+                    )
+
+                # drop tablespace
+                SQL = render_template(
+                    "/".join([self.template_path, 'delete.sql']),
+                    tsname=(rset['rows'][0])['name'], conn=self.conn
+                )
+
+                status, res = self.conn.execute_scalar(SQL)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Tablespace dropped"),
-                data={
-                    'id': tsid,
-                    'sid': sid,
-                    'gid': gid,
-                }
+                info=gettext("Tablespace dropped")
             )
 
         except Exception as e:
@@ -490,11 +505,17 @@ class TablespaceView(PGChildNodeView):
             for key in ['spcacl']:
                 if key in data and data[key] is not None:
                     if 'added' in data[key]:
-                        data[key]['added'] = parse_priv_to_db(data[key]['added'], self.acl)
+                        data[key]['added'] = parse_priv_to_db(
+                            data[key]['added'], self.acl
+                        )
                     if 'changed' in data[key]:
-                        data[key]['changed'] = parse_priv_to_db(data[key]['changed'], self.acl)
+                        data[key]['changed'] = parse_priv_to_db(
+                            data[key]['changed'], self.acl
+                        )
                     if 'deleted' in data[key]:
-                        data[key]['deleted'] = parse_priv_to_db(data[key]['deleted'], self.acl)
+                        data[key]['deleted'] = parse_priv_to_db(
+                            data[key]['deleted'], self.acl
+                        )
 
             # If name is not present with in update data then copy it
             # from old data
@@ -685,7 +706,10 @@ class TablespaceView(PGChildNodeView):
         }
 
         # Fetching databases with CONNECT privileges status.
-        query = render_template("/".join([self.template_path, 'dependents.sql']), fetch_database=True)
+        query = render_template(
+            "/".join([self.template_path, 'dependents.sql']),
+            fetch_database=True
+        )
         status, db_result = self.conn.execute_dict(query)
         if not status:
             current_app.logger.error(db_result)
@@ -722,8 +746,10 @@ class TablespaceView(PGChildNodeView):
                 current_app.logger.exception(e)
 
             if temp_conn.connected():
-                query = render_template("/".join([self.template_path, 'dependents.sql']),
-                                        fetch_dependents=True, tsid=tsid)
+                query = render_template(
+                    "/".join([self.template_path, 'dependents.sql']),
+                    fetch_dependents=True, tsid=tsid
+                )
                 status, result = temp_conn.execute_dict(query)
                 if not status:
                     current_app.logger.error(result)
@@ -759,7 +785,13 @@ class TablespaceView(PGChildNodeView):
                     else:
                         continue
 
-                    dependents.append({'type': type_name, 'name': rel_name, 'field': db_row['datname']})
+                    dependents.append(
+                        {
+                            'type': type_name,
+                            'name': rel_name,
+                            'field': db_row['datname']
+                        }
+                    )
 
                 # Release only those connections which we have created above.
                 if not is_connected:
