@@ -9,7 +9,7 @@ define('pgadmin.browser', [
   'pgadmin.browser.error', 'pgadmin.browser.frame',
   'pgadmin.browser.node', 'pgadmin.browser.collection',
   'sources/codemirror/addon/fold/pgadmin-sqlfoldcode',
-  'pgadmin.browser.keyboard',
+  'pgadmin.browser.keyboard', 'sources/tree/pgadmin_tree_save_state',
 ], function(
   tree,
   gettext, url_for, require, $, _, S, Bootstrap, pgAdmin, Alertify,
@@ -132,7 +132,7 @@ define('pgadmin.browser', [
         isCloseable: false,
         isPrivate: true,
         elContainer: true,
-        content: '<div class="obj_properties"><div class="alert alert-info pg-panel-message">' + select_object_msg + '</div></div>',
+        content: '<div class="obj_properties container-fluid"><div class="alert alert-info pg-panel-message">' + select_object_msg + '</div></div>',
         events: panelEvents,
         onCreate: function(myPanel, $container) {
           $container.addClass('pg-no-overflow');
@@ -146,7 +146,7 @@ define('pgadmin.browser', [
         width: 500,
         isCloseable: false,
         isPrivate: true,
-        content: '<div><div class="alert alert-info pg-panel-message pg-panel-statistics-message">' + select_object_msg + '</div><div class="pg-panel-statistics-container hidden"></div></div>',
+        content: '<div><div class="alert alert-info pg-panel-message pg-panel-statistics-message">' + select_object_msg + '</div><div class="pg-panel-statistics-container d-none"></div></div>',
         events: panelEvents,
       }),
       // Reversed engineered SQL for the object
@@ -167,7 +167,7 @@ define('pgadmin.browser', [
         width: 500,
         isCloseable: false,
         isPrivate: true,
-        content: '<div><div class="alert alert-info pg-panel-message pg-panel-depends-message">' + select_object_msg + '</div><div class="pg-panel-depends-container hidden"></div></div>',
+        content: '<div><div class="alert alert-info pg-panel-message pg-panel-depends-message">' + select_object_msg + '</div><div class="pg-panel-depends-container d-none"></div></div>',
         events: panelEvents,
       }),
       // Dependents of the object
@@ -178,7 +178,7 @@ define('pgadmin.browser', [
         width: 500,
         isCloseable: false,
         isPrivate: true,
-        content: '<div><div class="alert alert-info pg-panel-message pg-panel-depends-message">' + select_object_msg + '</div><div class="pg-panel-depends-container hidden"></div></div>',
+        content: '<div><div class="alert alert-info pg-panel-message pg-panel-depends-message">' + select_object_msg + '</div><div class="pg-panel-depends-container d-none"></div></div>',
         events: panelEvents,
       }),
     },
@@ -279,7 +279,7 @@ define('pgadmin.browser', [
         // menu navigation bar
         navbar = $('#navbar-menu > ul').first(),
         // Drop down menu for objects
-        $obj_mnu = navbar.find('li#mnu_obj > ul.dropdown-menu').first(),
+        $obj_mnu = navbar.find('li#mnu_obj .dropdown-menu').first(),
         // data for current selected object
         d = obj.tree.itemData(item),
         update_menuitem = function(m) {
@@ -320,7 +320,7 @@ define('pgadmin.browser', [
         // Create a dummy 'no object seleted' menu
         var create_submenu = pgAdmin.Browser.MenuGroup(
           obj.menu_categories['create'], [{
-            $el: $('<li class="menu-item disabled"><a href="#">' + gettext('No object selected') + '</a></li>'),
+            $el: $('<li><a class="dropdown-item text-white disabled" href="#">' + gettext('No object selected') + '</a></li>'),
             priority: 1,
             category: 'create',
             update: function() {},
@@ -513,9 +513,11 @@ define('pgadmin.browser', [
         .done(function() {})
         .fail(function() {});
       }, 300000);
+
       obj.Events.on('pgadmin:browser:tree:add', obj.onAddTreeNode, obj);
       obj.Events.on('pgadmin:browser:tree:update', obj.onUpdateTreeNode, obj);
       obj.Events.on('pgadmin:browser:tree:refresh', obj.onRefreshTreeNode, obj);
+
     },
 
     add_menu_category: function(
@@ -609,11 +611,11 @@ define('pgadmin.browser', [
           if (pgAdmin.Browser.MenuCreator(
             $dropdown, obj.menus[o.menu], obj.menu_categories
           )) {
-            $mnu.removeClass('hide');
+            $mnu.removeClass('d-none');
           }
         });
 
-      navbar.children('#mnu_obj').removeClass('hide');
+      navbar.children('#mnu_obj').removeClass('d-none');
       obj.enable_disable_menus();
     },
     // General function to handle callbacks for object or dialog help.
@@ -1942,19 +1944,10 @@ define('pgadmin.browser', [
     pgAdmin.Browser.editor_shortcut_keys.Tab = 'insertSoftTab';
   }
 
-  window.onbeforeunload = function() {
-    var e = window.event,
-      msg = S(gettext('Are you sure you wish to close the %s browser?')).sprintf(pgBrowser.utils.app_name).value();
-
-    // For IE and Firefox prior to version 4
-    if (e) {
-      e.returnValue = msg;
-    }
-
-    // For Safari
-    return msg;
-  };
-
+  $(window).on('beforeunload', function() {
+    if (pgBrowser.get_preference('browser', 'browser_tree_state_save_interval').value !== -1)
+      pgAdmin.Browser.browserTreeState.save_state();
+  });
 
   return pgAdmin.Browser;
 });
