@@ -1,11 +1,11 @@
 define('pgadmin.browser.node', [
-  'sources/tree/pgadmin_tree_node',
+  'sources/tree/pgadmin_tree_node', 'sources/url_for',
   'sources/gettext', 'jquery', 'underscore', 'underscore.string', 'sources/pgadmin',
   'pgadmin.browser.menu', 'backbone', 'pgadmin.alertifyjs', 'pgadmin.browser.datamodel',
   'backform', 'sources/browser/generate_url', 'sources/utils', 'pgadmin.browser.utils',
   'pgadmin.backform',
 ], function(
-  pgadminTreeNode,
+  pgadminTreeNode, url_for,
   gettext, $, _, S, pgAdmin, Menu, Backbone, Alertify, pgBrowser, Backform, generateUrl, commonUtils
 ) {
 
@@ -395,7 +395,7 @@ define('pgadmin.browser.node', [
               success: function() {
                 // Clear timeout and remove message
                 clearTimeout(timer);
-                $msgDiv.addClass('hidden');
+                $msgDiv.addClass('d-none');
 
                 // We got the latest attributes of the object. Render the view
                 // now.
@@ -479,7 +479,7 @@ define('pgadmin.browser.node', [
         isCloseable: true,
         isPrivate: true,
         elContainer: true,
-        content: '<div class="obj_properties"><div class="alert alert-info pg-panel-message">' + gettext('Please wait while we fetch information about the node from the server...') + '</div></div>',
+        content: '<div class="obj_properties container-fluid"><div class="alert alert-info pg-panel-message">' + gettext('Please wait while we fetch information about the node from the server...') + '</div></div>',
         onCreate: function(myPanel, $container) {
           $container.addClass('pg-no-overflow');
         },
@@ -849,7 +849,6 @@ define('pgadmin.browser.node', [
           }
         }
       },
-
       added: function(item, data, browser) {
         var b = browser || pgBrowser,
           t = b.tree,
@@ -876,6 +875,8 @@ define('pgadmin.browser.node', [
           );
         }
 
+        pgBrowser.Events.trigger('pgadmin:browser:tree:expand-from-previous-tree-state',
+         item);
         pgBrowser.Node.callbacks.change_server_background(item, data);
       },
       // Callback called - when a node is selected in browser tree.
@@ -905,6 +906,8 @@ define('pgadmin.browser.node', [
           }
         }
 
+        pgBrowser.Events.trigger('pgadmin:browser:tree:update-tree-state',
+         item);
         return true;
       },
       removed: function(item) {
@@ -958,6 +961,14 @@ define('pgadmin.browser.node', [
             },
           });
       },
+      opened: function(item) {
+        pgBrowser.Events.trigger('pgadmin:browser:tree:update-tree-state',
+         item);
+      },
+      closed: function(item) {
+        pgBrowser.Events.trigger('pgadmin:browser:tree:remove-from-tree-state',
+         item);
+      },
     },
     /**********************************************************************
      * A hook (not a callback) to show object properties in given HTML
@@ -972,7 +983,7 @@ define('pgadmin.browser.node', [
         j = panel.$container.find('.obj_properties').first(),
         view = j.data('obj-view'),
         content = $('<div tabindex="1"></div>')
-        .addClass('pg-prop-content col-xs-12');
+        .addClass('pg-prop-content col-12');
 
       // Handle key press events for Cancel, save and help button
       var handleKeyDown = function(event, context) {
