@@ -28,6 +28,10 @@ class LogoutTest(BaseTestGenerator):
     def setUpClass(cls):
         pass
 
+    # No need to call base class setup function
+    def setUp(self):
+        pass
+
     def runTest(self):
         """This function checks the logout functionality."""
 
