@@ -2,18 +2,15 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 """A blueprint module implementing the dashboard frame."""
-MODULE_NAME = 'dashboard'
-
 from functools import wraps
-
 from flask import render_template, url_for, Response, g
-from flask_babel import gettext
+from flask_babelex import gettext
 from flask_security import login_required
 from pgadmin.utils import PgAdminModule
 from pgadmin.utils.ajax import make_response as ajax_response,\
@@ -25,6 +22,8 @@ from pgadmin.utils.preferences import Preferences
 
 from config import PG_DEFAULT_DRIVER
 
+MODULE_NAME = 'dashboard'
+
 
 class DashboardModule(PgAdminModule):
     def __init__(self, *args, **kwargs):
@@ -45,9 +44,7 @@ class DashboardModule(PgAdminModule):
         Returns:
             list: the stylesheets used by this module.
         """
-        stylesheets = [
-            url_for('dashboard.static', filename='css/dashboard.css')
-        ]
+        stylesheets = []
         return stylesheets
 
     def get_panels(self):
@@ -62,7 +59,8 @@ class DashboardModule(PgAdminModule):
                 isPrivate=False,
                 limit=1,
                 isIframe=False,
-                canHide=True)
+                canHide=True
+            ).__dict__
         ]
 
     def register_preferences(self):
@@ -71,8 +69,9 @@ class DashboardModule(PgAdminModule):
         Register preferences for this module.
         """
         # Register options for the PG and PPAS help paths
-        self.dashboard_preference = Preferences('dashboards',
-                                                gettext('Dashboards'))
+        self.dashboard_preference = Preferences(
+            'dashboards', gettext('Dashboards')
+        )
 
         self.session_stats_refresh = self.dashboard_preference.register(
             'dashboards', 'session_stats_refresh',
@@ -82,7 +81,7 @@ class DashboardModule(PgAdminModule):
             help_str=gettext('The number of seconds between graph samples.')
         )
 
-        self.session_stats_refresh = self.dashboard_preference.register(
+        self.tps_stats_refresh = self.dashboard_preference.register(
             'dashboards', 'tps_stats_refresh',
             gettext("Transaction throughput refresh rate"), 'integer',
             1, min_val=1, max_val=999999,
@@ -90,7 +89,7 @@ class DashboardModule(PgAdminModule):
             help_str=gettext('The number of seconds between graph samples.')
         )
 
-        self.session_stats_refresh = self.dashboard_preference.register(
+        self.ti_stats_refresh = self.dashboard_preference.register(
             'dashboards', 'ti_stats_refresh',
             gettext("Tuples in refresh rate"), 'integer',
             1, min_val=1, max_val=999999,
@@ -98,7 +97,7 @@ class DashboardModule(PgAdminModule):
             help_str=gettext('The number of seconds between graph samples.')
         )
 
-        self.session_stats_refresh = self.dashboard_preference.register(
+        self.to_stats_refresh = self.dashboard_preference.register(
             'dashboards', 'to_stats_refresh',
             gettext("Tuples out refresh rate"), 'integer',
             1, min_val=1, max_val=999999,
@@ -106,7 +105,7 @@ class DashboardModule(PgAdminModule):
             help_str=gettext('The number of seconds between graph samples.')
         )
 
-        self.session_stats_refresh = self.dashboard_preference.register(
+        self.bio_stats_refresh = self.dashboard_preference.register(
             'dashboards', 'bio_stats_refresh',
             gettext("Block I/O statistics refresh rate"), 'integer',
             1, min_val=1, max_val=999999,
@@ -114,6 +113,75 @@ class DashboardModule(PgAdminModule):
             help_str=gettext('The number of seconds between graph samples.')
         )
 
+        self.display_graphs = self.dashboard_preference.register(
+            'display', 'show_graphs',
+            gettext("Show graphs?"), 'boolean', True,
+            category_label=gettext('Display'),
+            help_str=gettext('If set to True, graphs '
+                             'will be displayed on dashboards.')
+        )
+
+        self.display_server_activity = self.dashboard_preference.register(
+            'display', 'show_activity',
+            gettext("Show activity?"), 'boolean', True,
+            category_label=gettext('Display'),
+            help_str=gettext('If set to True, activity tables '
+                             'will be displayed on dashboards.')
+        )
+
+        self.graph_data_points = self.dashboard_preference.register(
+            'display', 'graph_data_points',
+            gettext("Show graph data points?"), 'boolean', False,
+            category_label=gettext('Display'),
+            help_str=gettext('If set to True, data points will be '
+                             'visible on graph lines.')
+        )
+
+        self.graph_mouse_track = self.dashboard_preference.register(
+            'display', 'graph_mouse_track',
+            gettext("Show mouse hover tooltip?"), 'boolean', True,
+            category_label=gettext('Display'),
+            help_str=gettext('If set to True, tooltip will appear on mouse '
+                             'hover on the graph lines giving the data point '
+                             'details')
+        )
+
+    def get_exposed_url_endpoints(self):
+        """
+        Returns:
+            list: a list of url endpoints exposed to the client.
+        """
+        return [
+            'dashboard.index', 'dashboard.get_by_sever_id',
+            'dashboard.get_by_database_id',
+            'dashboard.session_stats',
+            'dashboard.get_session_stats_by_sever_id',
+            'dashboard.get_session_stats_by_database_id',
+            'dashboard.tps_stats',
+            'dashboard.tps_stats_by_server_id',
+            'dashboard.tps_stats_by_database_id',
+            'dashboard.ti_stats',
+            'dashboard.ti_stats_by_server_id',
+            'dashboard.ti_stats_by_database_id',
+            'dashboard.to_stats',
+            'dashboard.to_stats_by_server_id',
+            'dashboard.to_stats_by_database_id',
+            'dashboard.bio_stats',
+            'dashboard.bio_stats_by_server_id',
+            'dashboard.bio_stats_by_database_id',
+            'dashboard.activity',
+            'dashboard.get_activity_by_server_id',
+            'dashboard.get_activity_by_database_id',
+            'dashboard.locks',
+            'dashboard.get_locks_by_server_id',
+            'dashboard.get_locks_by_database_id',
+            'dashboard.prepared',
+            'dashboard.get_prepared_by_server_id',
+            'dashboard.get_prepared_by_database_id',
+            'dashboard.config',
+            'dashboard.get_config_by_server_id',
+        ]
+
 
 blueprint = DashboardModule(MODULE_NAME, __name__)
 
@@ -134,11 +202,13 @@ def check_precondition(f):
             kwargs['sid']
         )
 
+        stats_type = ('activity', 'prepared', 'locks', 'config')
+
         # Below check handle the case where existing server is deleted
         # by user and python server will raise exception if this check
         # is not introduce.
         if g.manager is None:
-            if f.__name__ in ['activity', 'prepared', 'locks', 'config']:
+            if f.__name__ in stats_type:
                 return precondition_required(
                     gettext("Please connect to the selected server"
                             " to view the table.")
@@ -153,7 +223,7 @@ def check_precondition(f):
 
         # If DB not connected then return error to browser
         if not g.conn.connected():
-            if f.__name__ in ['activity', 'prepared', 'locks', 'config']:
+            if f.__name__ in stats_type:
                 return precondition_required(
                     gettext("Please connect to the selected server"
                             " to view the table.")
@@ -168,7 +238,7 @@ def check_precondition(f):
             db_conn = g.manager.connection(did=kwargs['did'])
             # If the selected DB not connected then return error to browser
             if not db_conn.connected():
-                if f.__name__ in ['activity', 'prepared', 'locks', 'config']:
+                if f.__name__ in stats_type:
                     return precondition_required(
                         gettext("Please connect to the selected database"
                                 " to view the table.")
@@ -183,7 +253,11 @@ def check_precondition(f):
         g.server_type = g.manager.server_type
         g.version = g.manager.version
 
-        g.template_path = 'dashboard/sql/#{0}#'.format(g.version)
+        # Include server_type in template_path when server_type is gpdb
+        g.template_path = 'dashboard/sql/' + (
+            '#{0}#{1}#'.format(g.server_type, g.version)
+            if g.server_type == 'gpdb' else '#{0}#'.format(g.version)
+        )
 
         return f(*args, **kwargs)
 
@@ -194,15 +268,19 @@ def check_precondition(f):
 @login_required
 def script():
     """render the required javascript"""
-    return Response(response=render_template("dashboard/js/dashboard.js",
-                                             _=gettext),
-                    status=200,
-                    mimetype="application/javascript")
+    return Response(
+        response=render_template(
+            "dashboard/js/dashboard.js",
+            _=gettext
+        ),
+        status=200,
+        mimetype="application/javascript"
+    )
 
 
-@blueprint.route('/')
-@blueprint.route('/<int:sid>')
-@blueprint.route('/<int:sid>/<int:did>')
+@blueprint.route('/', endpoint='index')
+@blueprint.route('/<int:sid>', endpoint='get_by_sever_id')
+@blueprint.route('/<int:sid>/<int:did>', endpoint='get_by_database_id')
 @login_required
 def index(sid=None, did=None):
     """
@@ -215,8 +293,7 @@ def index(sid=None, did=None):
 
     """
     rates = {}
-
-    prefs = Preferences.module('dashboards')
+    settings = {}
 
     # Get the server version
     if sid is not None:
@@ -229,31 +306,24 @@ def index(sid=None, did=None):
         if not g.conn.connected():
             g.version = 0
 
-    session_stats_refresh_pref = prefs.preference('session_stats_refresh')
-    rates['session_stats_refresh'] = session_stats_refresh_pref.get()
-    tps_stats_refresh_pref = prefs.preference('tps_stats_refresh')
-    rates['tps_stats_refresh'] = tps_stats_refresh_pref.get()
-    ti_stats_refresh_pref = prefs.preference('ti_stats_refresh')
-    rates['ti_stats_refresh'] = ti_stats_refresh_pref.get()
-    to_stats_refresh_pref = prefs.preference('to_stats_refresh')
-    rates['to_stats_refresh'] = to_stats_refresh_pref.get()
-    bio_stats_refresh_pref = prefs.preference('bio_stats_refresh')
-    rates['bio_stats_refresh'] = bio_stats_refresh_pref.get()
-
     # Show the appropriate dashboard based on the identifiers passed to us
     if sid is None and did is None:
         return render_template('/dashboard/welcome_dashboard.html')
     if did is None:
-        return render_template('/dashboard/server_dashboard.html',
-                               sid=sid,
-                               rates=rates,
-                               version=g.version)
+        return render_template(
+            '/dashboard/server_dashboard.html',
+            sid=sid,
+            rates=rates,
+            version=g.version
+        )
     else:
-        return render_template('/dashboard/database_dashboard.html',
-                               sid=sid,
-                               did=did,
-                               rates=rates,
-                               version=g.version)
+        return render_template(
+            '/dashboard/database_dashboard.html',
+            sid=sid,
+            did=did,
+            rates=rates,
+            version=g.version
+        )
 
 
 def get_data(sid, did, template):
@@ -286,9 +356,14 @@ def get_data(sid, did, template):
     )
 
 
-@blueprint.route('/session_stats/')
-@blueprint.route('/session_stats/<int:sid>')
-@blueprint.route('/session_stats/<int:sid>/<int:did>')
+@blueprint.route('/session_stats/', endpoint='session_stats')
+@blueprint.route(
+    '/session_stats/<int:sid>', endpoint='get_session_stats_by_sever_id'
+)
+@blueprint.route(
+    '/session_stats/<int:sid>/<int:did>',
+    endpoint='get_session_stats_by_database_id'
+)
 @login_required
 @check_precondition
 def session_stats(sid=None, did=None):
@@ -300,9 +375,11 @@ def session_stats(sid=None, did=None):
     return get_data(sid, did, 'session_stats.sql')
 
 
-@blueprint.route('/tps_stats/')
-@blueprint.route('/tps_stats/<int:sid>')
-@blueprint.route('/tps_stats/<int:sid>/<int:did>')
+@blueprint.route('/tps_stats/', endpoint='tps_stats')
+@blueprint.route('/tps_stats/<int:sid>', endpoint='tps_stats_by_server_id')
+@blueprint.route(
+    '/tps_stats/<int:sid>/<int:did>', endpoint='tps_stats_by_database_id'
+)
 @login_required
 @check_precondition
 def tps_stats(sid=None, did=None):
@@ -314,9 +391,11 @@ def tps_stats(sid=None, did=None):
     return get_data(sid, did, 'tps_stats.sql')
 
 
-@blueprint.route('/ti_stats/')
-@blueprint.route('/ti_stats/<int:sid>')
-@blueprint.route('/ti_stats/<int:sid>/<int:did>')
+@blueprint.route('/ti_stats/', endpoint='ti_stats')
+@blueprint.route('/ti_stats/<int:sid>', endpoint='ti_stats_by_server_id')
+@blueprint.route(
+    '/ti_stats/<int:sid>/<int:did>', endpoint='ti_stats_by_database_id'
+)
 @login_required
 @check_precondition
 def ti_stats(sid=None, did=None):
@@ -328,9 +407,11 @@ def ti_stats(sid=None, did=None):
     return get_data(sid, did, 'ti_stats.sql')
 
 
-@blueprint.route('/to_stats/')
-@blueprint.route('/to_stats/<int:sid>')
-@blueprint.route('/to_stats/<int:sid>/<int:did>')
+@blueprint.route('/to_stats/', endpoint='to_stats')
+@blueprint.route('/to_stats/<int:sid>', endpoint='to_stats_by_server_id')
+@blueprint.route(
+    '/to_stats/<int:sid>/<int:did>', endpoint='to_stats_by_database_id'
+)
 @login_required
 @check_precondition
 def to_stats(sid=None, did=None):
@@ -342,9 +423,11 @@ def to_stats(sid=None, did=None):
     return get_data(sid, did, 'to_stats.sql')
 
 
-@blueprint.route('/bio_stats/')
-@blueprint.route('/bio_stats/<int:sid>')
-@blueprint.route('/bio_stats/<int:sid>/<int:did>')
+@blueprint.route('/bio_stats/', endpoint='bio_stats')
+@blueprint.route('/bio_stats/<int:sid>', endpoint='bio_stats_by_server_id')
+@blueprint.route(
+    '/bio_stats/<int:sid>/<int:did>', endpoint='bio_stats_by_database_id'
+)
 @login_required
 @check_precondition
 def bio_stats(sid=None, did=None):
@@ -356,9 +439,11 @@ def bio_stats(sid=None, did=None):
     return get_data(sid, did, 'bio_stats.sql')
 
 
-@blueprint.route('/activity/')
-@blueprint.route('/activity/<int:sid>')
-@blueprint.route('/activity/<int:sid>/<int:did>')
+@blueprint.route('/activity/', endpoint='activity')
+@blueprint.route('/activity/<int:sid>', endpoint='get_activity_by_server_id')
+@blueprint.route(
+    '/activity/<int:sid>/<int:did>', endpoint='get_activity_by_database_id'
+)
 @login_required
 @check_precondition
 def activity(sid=None, did=None):
@@ -370,9 +455,11 @@ def activity(sid=None, did=None):
     return get_data(sid, did, 'activity.sql')
 
 
-@blueprint.route('/locks/')
-@blueprint.route('/locks/<int:sid>')
-@blueprint.route('/locks/<int:sid>/<int:did>')
+@blueprint.route('/locks/', endpoint='locks')
+@blueprint.route('/locks/<int:sid>', endpoint='get_locks_by_server_id')
+@blueprint.route(
+    '/locks/<int:sid>/<int:did>', endpoint='get_locks_by_database_id'
+)
 @login_required
 @check_precondition
 def locks(sid=None, did=None):
@@ -384,9 +471,11 @@ def locks(sid=None, did=None):
     return get_data(sid, did, 'locks.sql')
 
 
-@blueprint.route('/prepared/')
-@blueprint.route('/prepared/<int:sid>')
-@blueprint.route('/prepared/<int:sid>/<int:did>')
+@blueprint.route('/prepared/', endpoint='prepared')
+@blueprint.route('/prepared/<int:sid>', endpoint='get_prepared_by_server_id')
+@blueprint.route(
+    '/prepared/<int:sid>/<int:did>', endpoint='get_prepared_by_database_id'
+)
 @login_required
 @check_precondition
 def prepared(sid=None, did=None):
@@ -398,8 +487,8 @@ def prepared(sid=None, did=None):
     return get_data(sid, did, 'prepared.sql')
 
 
-@blueprint.route('/config/')
-@blueprint.route('/config/<int:sid>')
+@blueprint.route('/config/', endpoint='config')
+@blueprint.route('/config/<int:sid>', endpoint='get_config_by_server_id')
 @login_required
 @check_precondition
 def config(sid=None):
@@ -409,3 +498,57 @@ def config(sid=None):
     :return:
     """
     return get_data(sid, None, 'config.sql')
+
+
+@blueprint.route(
+    '/cancel_query/<int:sid>/<int:pid>', methods=['DELETE']
+)
+@blueprint.route(
+    '/cancel_query/<int:sid>/<int:did>/<int:pid>', methods=['DELETE']
+)
+@login_required
+@check_precondition
+def cancel_query(sid=None, did=None, pid=None):
+    """
+    This function cancel the specific session
+    :param sid: server id
+    :param did: database id
+    :param pid: session/process id
+    :return: Response
+    """
+    sql = "SELECT pg_cancel_backend({0});".format(pid)
+    status, res = g.conn.execute_scalar(sql)
+    if not status:
+        return internal_server_error(errormsg=res)
+
+    return ajax_response(
+        response=gettext("Success") if res else gettext("Failed"),
+        status=200
+    )
+
+
+@blueprint.route(
+    '/terminate_session/<int:sid>/<int:pid>', methods=['DELETE']
+)
+@blueprint.route(
+    '/terminate_session/<int:sid>/<int:did>/<int:pid>', methods=['DELETE']
+)
+@login_required
+@check_precondition
+def terminate_session(sid=None, did=None, pid=None):
+    """
+    This function terminate the specific session
+    :param sid: server id
+    :param did: database id
+    :param pid: session/process id
+    :return: Response
+    """
+    sql = "SELECT pg_terminate_backend({0});".format(pid)
+    status, res = g.conn.execute_scalar(sql)
+    if not status:
+        return internal_server_error(errormsg=res)
+
+    return ajax_response(
+        response=gettext("Success") if res else gettext("Failed"),
+        status=200
+    )
