@@ -8,6 +8,7 @@
 ##########################################################################
 
 import pyperclip
+import random
 
 from selenium.webdriver import ActionChains
 from selenium.webdriver.common.keys import Keys
@@ -25,23 +26,18 @@ class QueryToolJourneyTest(BaseFeatureTest):
         ("Tests the path through the query tool", dict())
     ]
 
+    test_table_name = ""
+
     def before(self):
-        connection = test_utils.get_db_connection(self.server['db'],
-                                                  self.server['username'],
-                                                  self.server['db_password'],
-                                                  self.server['host'],
-                                                  self.server['port'])
-        test_utils.drop_database(connection, "acceptance_test_db")
-        test_utils.create_database(self.server, "acceptance_test_db")
-        test_utils.create_table(self.server, "acceptance_test_db", "test_table")
+        self.test_table_name = "test_table" + str(random.randint(1000, 3000))
+        test_utils.create_table(
+            self.server, self.test_db, self.test_table_name)
         self.page.add_server(self.server)
 
     def runTest(self):
         self._navigate_to_query_tool()
-
         self._execute_query(
-            "SELECT * FROM test_table ORDER BY value"
-        )
+            "SELECT * FROM %s ORDER BY value " % self.test_table_name)
 
         self._test_copies_rows()
         self._test_copies_columns()
@@ -50,8 +46,10 @@ class QueryToolJourneyTest(BaseFeatureTest):
     def _test_copies_rows(self):
         pyperclip.copy("old clipboard contents")
         self.page.driver.switch_to.default_content()
-        self.page.driver.switch_to_frame(self.page.driver.find_element_by_tag_name("iframe"))
-        self.page.find_by_xpath("//*[contains(@class, 'slick-row')]/*[1]").click()
+        self.page.driver.switch_to_frame(
+            self.page.driver.find_element_by_tag_name("iframe"))
+        self.page.find_by_xpath(
+            "//*[contains(@class, 'slick-row')]/*[1]").click()
         self.page.find_by_xpath("//*[@id='btn-copy-row']").click()
 
         self.assertEqual('"Some-Name"\t"6"\t"some info"',
@@ -61,8 +59,12 @@ class QueryToolJourneyTest(BaseFeatureTest):
         pyperclip.copy("old clipboard contents")
 
         self.page.driver.switch_to.default_content()
-        self.page.driver.switch_to_frame(self.page.driver.find_element_by_tag_name("iframe"))
-        self.page.find_by_xpath("//*[@data-test='output-column-header' and contains(., 'some_column')]").click()
+        self.page.driver.switch_to_frame(
+            self.page.driver.find_element_by_tag_name("iframe"))
+        self.page.find_by_xpath(
+            "//*[@data-test='output-column-header' and "
+            "contains(., 'some_column')]"
+        ).click()
         self.page.find_by_xpath("//*[@id='btn-copy-row']").click()
 
         self.assertTrue('"Some-Name"' in pyperclip.paste())
@@ -76,22 +78,39 @@ class QueryToolJourneyTest(BaseFeatureTest):
         self._execute_query("SELECT * FROM table_that_doesnt_exist")
 
         self.page.click_tab("Query History")
-        selected_history_entry = self.page.find_by_css_selector("#query_list .selected")
-        self.assertIn("SELECT * FROM table_that_doesnt_exist", selected_history_entry.text)
+        selected_history_entry = self.page.find_by_css_selector(
+            "#query_list .selected")
+        self.assertIn("SELECT * FROM table_that_doesnt_exist",
+                      selected_history_entry.text)
         failed_history_detail_pane = self.page.find_by_id("query_detail")
 
-        self.assertIn("Error Message relation \"table_that_doesnt_exist\" does not exist", failed_history_detail_pane.text)
+        self.assertIn(
+            "Error Message relation \"table_that_doesnt_exist\" "
+            "does not exist", failed_history_detail_pane.text
+        )
         ActionChains(self.page.driver) \
             .send_keys(Keys.ARROW_DOWN) \
             .perform()
-        selected_history_entry = self.page.find_by_css_selector("#query_list .selected")
-        self.assertIn("SELECT * FROM test_table ORDER BY value", selected_history_entry.text)
-        selected_history_detail_pane = self.page.find_by_id("query_detail")
-        self.assertIn("SELECT * FROM test_table ORDER BY value", selected_history_detail_pane.text)
-        newly_selected_history_entry = self.page.find_by_xpath("//*[@id='query_list']/ul/li[2]")
+        selected_history_entry = self.page.find_by_css_selector(
+            "#query_list .selected")
+        self.assertIn(("SELECT * FROM %s ORDER BY value" %
+                       self.test_table_name),
+                      selected_history_entry.text)
+
+        query_element = self.page.driver.\
+            find_element_by_xpath(
+                "//div[@id='history_grid']//div[@class='entry selected']"
+                "/div[@class='query']")
+
+        self.assertIn(("SELECT * FROM %s ORDER BY value"
+                       % self.test_table_name), query_element.text)
+
+        newly_selected_history_entry = self.page.find_by_xpath(
+            "//*[@id='query_list']/ul/li[2]")
         self.page.click_element(newly_selected_history_entry)
         selected_history_detail_pane = self.page.find_by_id("query_detail")
-        self.assertIn("SELECT * FROM table_that_doesnt_exist", selected_history_detail_pane.text)
+        self.assertIn("SELECT * FROM table_that_doesnt_exist",
+                      selected_history_detail_pane.text)
 
         self.__clear_query_tool()
 
@@ -104,7 +123,8 @@ class QueryToolJourneyTest(BaseFeatureTest):
 
         self.page.click_tab("Query History")
 
-        query_we_need_to_scroll_to = self.page.find_by_xpath("//*[@id='query_list']/ul/li[17]")
+        query_we_need_to_scroll_to = self.page.find_by_xpath(
+            "//*[@id='query_list']/ul/li[17]")
 
         self.page.click_element(query_we_need_to_scroll_to)
 
@@ -147,7 +167,7 @@ class QueryToolJourneyTest(BaseFeatureTest):
     def _navigate_to_query_tool(self):
         self.page.toggle_open_tree_item(self.server['name'])
         self.page.toggle_open_tree_item('Databases')
-        self.page.toggle_open_tree_item('acceptance_test_db')
+        self.page.toggle_open_tree_item(self.test_db)
         self.page.open_query_tool()
 
     def _execute_query(self, query):
@@ -160,10 +180,3 @@ class QueryToolJourneyTest(BaseFeatureTest):
     def after(self):
         self.page.close_query_tool()
         self.page.remove_server(self.server)
-
-        connection = test_utils.get_db_connection(self.server['db'],
-                                                  self.server['username'],
-                                                  self.server['db_password'],
-                                                  self.server['host'],
-                                                  self.server['port'])
-        test_utils.drop_database(connection, "acceptance_test_db")
