@@ -14,25 +14,32 @@ Introduce a function to run the process executor in detached mode.
 import csv
 import os
 import sys
+import psutil
 from abc import ABCMeta, abstractproperty, abstractmethod
 from datetime import datetime
 from pickle import dumps, loads
 from subprocess import Popen
 
-from pgadmin.utils import IS_PY2, u, file_quote, fs_encoding
+from pgadmin.utils import IS_PY2, u, file_quote, fs_encoding, \
+    get_complete_file_path
 
-if IS_PY2:
-    from StringIO import StringIO
-else:
-    from io import StringIO
 import pytz
 from dateutil import parser
 from flask import current_app
-from flask_babel import gettext as _
+from flask_babelex import gettext as _
 from flask_security import current_user
 
 import config
 from pgadmin.model import Process, db
+if IS_PY2:
+    from StringIO import StringIO
+else:
+    from io import StringIO
+
+PROCESS_NOT_STARTED = 0
+PROCESS_STARTED = 1
+PROCESS_FINISHED = 2
+PROCESS_TERMINATED = 3
 
 
 def get_current_time(format='%Y-%m-%d %H:%M:%S.%f %z'):
@@ -62,11 +69,14 @@ class BatchProcess(object):
         self.id = self.desc = self.cmd = self.args = self.log_dir = \
             self.stdout = self.stderr = self.stime = self.etime = \
             self.ecode = None
+        self.env = dict()
 
         if 'id' in kwargs:
             self._retrieve_process(kwargs['id'])
         else:
-            self._create_process(kwargs['desc'], kwargs['cmd'], kwargs['args'])
+            self._create_process(
+                kwargs['desc'], kwargs['cmd'], kwargs['args']
+            )
 
     def _retrieve_process(self, _id):
         p = Process.query.filter_by(pid=_id, user_id=current_user.id).first()
@@ -76,10 +86,20 @@ class BatchProcess(object):
                 _("Could not find a process with the specified ID.")
             )
 
+        try:
+            tmp_desc = loads(p.desc.encode('latin-1')) if \
+                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+        except UnicodeDecodeError:
+            tmp_desc = loads(p.desc.encode('utf-8')) if \
+                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+        except Exception as e:
+            tmp_desc = loads(p.desc.encode('utf-8', 'ignore')) if \
+                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+
         # ID
         self.id = _id
         # Description
-        self.desc = loads(p.desc)
+        self.desc = tmp_desc
         # Status Acknowledged time
         self.atime = p.acknowledge
         # Command
@@ -98,6 +118,8 @@ class BatchProcess(object):
         self.etime = p.end_time
         # Exit code
         self.ecode = p.exit_code
+        # Process State
+        self.process_state = p.process_state
 
     def _create_process(self, _desc, _cmd, _args):
         ctime = get_current_time(format='%y%m%d%H%M%S%f')
@@ -151,6 +173,8 @@ class BatchProcess(object):
         self.etime = None
         # Exit code
         self.ecode = None
+        # Process State
+        self.process_state = PROCESS_NOT_STARTED
 
         # Arguments
         self.args = _args
@@ -158,18 +182,36 @@ class BatchProcess(object):
         csv_writer = csv.writer(
             args_csv_io, delimiter=str(','), quoting=csv.QUOTE_MINIMAL
         )
-        if sys.version_info.major == 2:
-            csv_writer.writerow([a.encode('utf-8') if isinstance(a, unicode) else a for a in _args])
+        if sys.version_info[0] == 2:
+            csv_writer.writerow(
+                [
+                    a.encode('utf-8')
+                    if isinstance(a, unicode) else a for a in _args
+                ]
+            )
         else:
             csv_writer.writerow(_args)
 
         args_val = args_csv_io.getvalue().strip(str('\r\n'))
+        tmp_desc = dumps(self.desc)
+        try:
+            tmp_desc = tmp_desc.decode('utf-8') if \
+                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
+        except UnicodeDecodeError:
+            tmp_desc = tmp_desc.decode('latin-1') if \
+                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
+        except Exception:
+            tmp_desc = tmp_desc.decode('utf-8', 'ignore') if \
+                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
 
         j = Process(
-            pid=int(id), command=_cmd,
-            arguments=args_val.decode('utf-8', 'replace') if IS_PY2 and hasattr(args_val, 'decode') \
-                else args_val,
-            logdir=log_dir, desc=dumps(self.desc), user_id=current_user.id
+            pid=int(id),
+            command=_cmd,
+            arguments=args_val.decode('utf-8', 'replace')
+            if IS_PY2 and hasattr(args_val, 'decode') else args_val,
+            logdir=log_dir,
+            desc=tmp_desc,
+            user_id=current_user.id
         )
         db.session.add(j)
         db.session.commit()
@@ -278,7 +320,9 @@ class BatchProcess(object):
         if os.name == 'nt' and IS_PY2:
             command = []
             for c in cmd:
-                command.append(c.encode('utf-8') if isinstance(c, unicode) else str(c))
+                command.append(
+                    c.encode('utf-8') if isinstance(c, unicode) else str(c)
+                )
 
             current_app.logger.info(
                 u"Executing the process executor with the arguments: %s",
@@ -288,7 +332,8 @@ class BatchProcess(object):
             cmd = command
         else:
             current_app.logger.info(
-                u"Executing the process executor with the arguments: %s", str(cmd)
+                u"Executing the process executor with the arguments: %s",
+                str(cmd)
             )
 
         # Make a copy of environment, and add new variables to support
@@ -297,6 +342,9 @@ class BatchProcess(object):
         env['OUTDIR'] = self.log_dir
         env['PGA_BGP_FOREGROUND'] = "1"
 
+        if self.env:
+            env.update(self.env)
+
         if cb is not None:
             cb(env)
 
@@ -318,8 +366,12 @@ class BatchProcess(object):
             stderr = open(stderr, "a")
 
             p = Popen(
-                cmd, close_fds=False, env=env, stdout=stdout.fileno(),
-                stderr=stderr.fileno(), stdin=stdin.fileno(),
+                cmd,
+                close_fds=False,
+                env=env,
+                stdout=stdout.fileno(),
+                stderr=stderr.fileno(),
+                stdin=stdin.fileno(),
                 creationflags=(CREATE_NEW_PROCESS_GROUP | DETACHED_PROCESS)
             )
         else:
@@ -350,6 +402,14 @@ class BatchProcess(object):
             p.start_time = p.end_time = get_current_time()
             if not p.exit_code:
                 p.exit_code = self.ecode
+            p.process_state = PROCESS_FINISHED
+            db.session.commit()
+        else:
+            # Update the process state to "Started"
+            p = Process.query.filter_by(
+                pid=self.id, user_id=current_user.id
+            ).first()
+            p.process_state = PROCESS_STARTED
             db.session.commit()
 
     def status(self, out=0, err=0):
@@ -421,13 +481,15 @@ class BatchProcess(object):
                 stime = parser.parse(self.stime)
                 etime = parser.parse(self.etime or get_current_time())
 
-                execution_time = (etime - stime).total_seconds()
+                execution_time = BatchProcess.total_seconds(etime - stime)
 
             if process_output:
-                out, out_completed = read_log(self.stdout, stdout, out, ctime,
-                                              self.ecode)
-                err, err_completed = read_log(self.stderr, stderr, err, ctime,
-                                              self.ecode)
+                out, out_completed = read_log(
+                    self.stdout, stdout, out, ctime, self.ecode
+                )
+                err, err_completed = read_log(
+                    self.stderr, stderr, err, ctime, self.ecode
+                )
         else:
             out_completed = err_completed = False
 
@@ -435,15 +497,25 @@ class BatchProcess(object):
             return {
                 'start_time': self.stime,
                 'exit_code': self.ecode,
-                'execution_time': execution_time
+                'execution_time': execution_time,
+                'process_state': self.process_state
             }
 
         return {
-            'out': {'pos': out, 'lines': stdout, 'done': out_completed},
-            'err': {'pos': err, 'lines': stderr, 'done': err_completed},
+            'out': {
+                'pos': out,
+                'lines': stdout,
+                'done': out_completed
+            },
+            'err': {
+                'pos': err,
+                'lines': stderr,
+                'done': err_completed
+            },
             'start_time': self.stime,
             'exit_code': self.ecode,
-            'execution_time': execution_time
+            'execution_time': execution_time,
+            'process_state': self.process_state
         }
 
     @staticmethod
@@ -471,13 +543,16 @@ class BatchProcess(object):
                             if 'end_time' in data and data['end_time']:
                                 p.end_time = data['end_time']
 
+                    # get the pid of the utility.
+                    if 'pid' in data:
+                        p.utility_pid = data['pid']
+
                     return True, True
 
                 except ValueError as e:
                     current_app.logger.warning(
-                        _("Status for the background process '{0}' could not be loaded.").format(
-                            p.pid
-                        )
+                        _("Status for the background process '{0}' could "
+                          "not be loaded.").format(p.pid)
                     )
                     current_app.logger.exception(e)
                     return False, False
@@ -507,15 +582,25 @@ class BatchProcess(object):
             stime = parser.parse(p.start_time)
             etime = parser.parse(p.end_time or get_current_time())
 
-            execution_time = (etime - stime).total_seconds()
-            desc = loads(p.desc)
+            execution_time = BatchProcess.total_seconds(etime - stime)
+            desc = ""
+            try:
+                desc = loads(p.desc.encode('latin-1')) if \
+                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+            except UnicodeDecodeError:
+                desc = loads(p.desc.encode('utf-8')) if \
+                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+            except Exception:
+                desc = loads(p.desc.encode('utf-8', 'ignore')) if \
+                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+
             details = desc
 
             if isinstance(desc, IProcessDesc):
                 args = []
                 args_csv = StringIO(
-                  p.arguments.encode('utf-8') \
-                      if hasattr(p.arguments, 'decode') else p.arguments
+                    p.arguments.encode('utf-8')
+                    if hasattr(p.arguments, 'decode') else p.arguments
                 )
                 args_reader = csv.reader(args_csv, delimiter=str(','))
                 for arg in args_reader:
@@ -531,7 +616,8 @@ class BatchProcess(object):
                 'etime': p.end_time,
                 'exit_code': p.exit_code,
                 'acknowledge': p.acknowledge,
-                'execution_time': execution_time
+                'execution_time': execution_time,
+                'process_state': p.process_state
             })
 
         if changed:
@@ -539,6 +625,16 @@ class BatchProcess(object):
 
         return res
 
+    @staticmethod
+    def total_seconds(dt):
+        # Keep backward compatibility with Python 2.6 which doesn't have
+        # this method
+        if hasattr(dt, 'total_seconds'):
+            return dt.total_seconds()
+        else:
+            return (dt.microseconds + (dt.seconds + dt.days * 24 * 3600) *
+                    10**6) / 10**6
+
     @staticmethod
     def acknowledge(_pid):
         """
@@ -566,3 +662,49 @@ class BatchProcess(object):
             p.acknowledge = get_current_time()
 
         db.session.commit()
+
+    def set_env_variables(self, server, **kwargs):
+        """Set environment variables"""
+        if server:
+            # Set SSL related ENV variables
+            if server.sslcert and server.sslkey and server.sslrootcert:
+                # SSL environment variables
+                self.env['PGSSLMODE'] = server.ssl_mode
+                self.env['PGSSLCERT'] = get_complete_file_path(server.sslcert)
+                self.env['PGSSLKEY'] = get_complete_file_path(server.sslkey)
+                self.env['PGSSLROOTCERT'] = get_complete_file_path(
+                    server.sslrootcert
+                )
+
+            # Set service name related ENV variable
+            if server.service:
+                self.env['PGSERVICE'] = server.service
+
+        if 'env' in kwargs:
+            self.env.update(kwargs['env'])
+
+    @staticmethod
+    def stop_process(_pid):
+        """
+        """
+        p = Process.query.filter_by(
+            user_id=current_user.id, pid=_pid
+        ).first()
+
+        if p is None:
+            raise LookupError(
+                _("Could not find a process with the specified ID.")
+            )
+
+        try:
+            process = psutil.Process(p.utility_pid)
+            process.terminate()
+            # Update the process state to "Terminated"
+            p.process_state = PROCESS_TERMINATED
+            db.session.commit()
+        except psutil.Error as e:
+            current_app.logger.warning(
+                _("Unable to kill the background process '{0}'").format(
+                    p.utility_pid)
+            )
+            current_app.logger.exception(e)
