@@ -1,7 +1,9 @@
-define([
-  'pgadmin', 'underscore', 'underscore.string', 'jquery', 'pgadmin.browser',
-  'alertify', 'pgadmin.browser.messages'
-], function(pgAdmin, _, S, $, pgBrowser, alertify, pgMessages) {
+define('misc.bgprocess', [
+  'sources/pgadmin', 'sources/gettext', 'sources/url_for', 'underscore',
+  'underscore.string', 'jquery', 'pgadmin.browser', 'alertify',
+], function(
+  pgAdmin, gettext, url_for, _, S, $, pgBrowser, Alertify
+) {
 
   pgBrowser.BackgroundProcessObsorver = pgBrowser.BackgroundProcessObsorver || {};
 
@@ -9,6 +11,8 @@ define([
     return pgBrowser.BackgroundProcessObsorver;
   }
 
+  var wcDocker = window.wcDocker;
+
   var BGProcess = function(info, notify) {
     var self = this;
     setTimeout(
@@ -25,7 +29,7 @@ define([
           details: false,
           notify: (_.isUndefined(notify) || notify),
           curr_status: null,
-          state: 0, // 0: NOT Started, 1: Started, 2: Finished
+          state: 0, // 0: NOT Started, 1: Started, 2: Finished, 3: Terminated
           completed: false,
 
           id: info['id'],
@@ -42,7 +46,9 @@ define([
           notifier: null,
           container: null,
           panel: null,
-          logs: $('<ol></ol>', {class: 'pg-bg-process-logs'})
+          logs: $('<ol></ol>', {
+            class: 'pg-bg-process-logs',
+          }),
         });
 
         if (this.notify) {
@@ -59,7 +65,7 @@ define([
               if (!process.notifier)
                 process.show.apply(process);
             }
-          )
+          );
         }
         var self = this;
 
@@ -70,27 +76,38 @@ define([
         );
       },
 
-      url: function(type) {
-        var url = S('%s%s').sprintf(pgMessages['bgprocess.index'], this.id).value();
-
+      bgprocess_url: function(type) {
         switch (type) {
-          case 'status':
-            if (this.details && this.out != -1 && this.err != -1) {
-              url = S('%s/%s/%s/').sprintf(
-                url, this.out, this.err
-              ).value();
-            }
-            break;
+        case 'status':
+          if (this.details && this.out != -1 && this.err != -1) {
+            return url_for(
+                'bgprocess.detailed_status', {
+                  'pid': this.id,
+                  'out': this.out,
+                  'err': this.err,
+                }
+              );
+          }
+          return url_for('bgprocess.status', {
+            'pid': this.id,
+          });
+        case 'acknowledge':
+          return url_for('bgprocess.acknowledge', {
+            'pid': this.id,
+          });
+        case 'stop_process':
+          return url_for('bgprocess.stop_process', {
+            'pid': this.id,
+          });
+        default:
+          return url_for('bgprocess.list');
         }
-
-        return url;
       },
 
       update: function(data) {
         var self = this,
-            out = [],
-            err = [],
-            idx = 0;
+          out = [],
+          err = [];
 
         if ('stime' in data)
           self.stime = new Date(data.stime);
@@ -107,6 +124,9 @@ define([
         if ('exit_code' in data)
           self.exit_code = data.exit_code;
 
+        if ('process_state' in data)
+          self.state = data.process_state;
+
         if ('out' in data) {
           self.out = data.out && data.out.pos;
 
@@ -124,19 +144,19 @@ define([
         }
         self.completed = self.completed || (
           'err' in data && 'out' in data && data.err.done && data.out.done
-        ) || (
-          !self.details && !_.isNull(self.exit_code)
-        );
+        ) || (!self.details && !_.isNull(self.exit_code));
 
-        var io = ie = 0, res = [],
-            escapeEl = document.createElement('textarea'),
-            escapeHTML = function(html) {
-              escapeEl.textContent = html;
-              return escapeEl.innerHTML;
-            };
+        var io = 0,
+          ie = 0,
+          res = [],
+          escapeEl = document.createElement('textarea'),
+          escapeHTML = function(html) {
+            escapeEl.textContent = html;
+            return escapeEl.innerHTML;
+          };
 
         while (io < out.length && ie < err.length) {
-          if (pgAdmin.natural_sort(out[io][0], err[ie][0]) <= 0){
+          if (pgAdmin.natural_sort(out[io][0], err[ie][0]) <= 0) {
             res.push('<li class="pg-bg-res-out">' + escapeHTML(out[io++][1]) + '</li>');
           } else {
             res.push('<li class="pg-bg-res-err">' + escapeHTML(err[ie++][1]) + '</li>');
@@ -150,23 +170,29 @@ define([
         while (ie < err.length) {
           res.push('<li class="pg-bg-res-err">' + escapeHTML(err[ie++][1]) + '</li>');
         }
+
         if (res.length) {
           self.logs.append(res.join(''));
+          setTimeout(function() {
+            self.logs[0].scrollTop = self.logs[0].scrollHeight;
+          });
         }
 
         if (self.stime) {
-          self.curr_status = pgMessages['started'];
+          self.curr_status = gettext('Started');
 
           if (self.execution_time >= 2) {
-            self.curr_status = pgMessages['running'];
+            self.curr_status = gettext['Running...'];
           }
 
           if (!_.isNull(self.exit_code)) {
-            if (self.exit_code == 0) {
-              self.curr_status = pgMessages['successfully_finished'];
+            if (self.state === 3) {
+              self.curr_status = gettext('Terminated by user.');
+            } else if (self.exit_code == 0) {
+              self.curr_status = gettext('Successfully completed.');
             } else {
               self.curr_status = S(
-                pgMessages['failed_with_exit_code']
+                gettext('Failed (exit code: %s).')
               ).sprintf(String(self.exit_code)).value();
             }
           }
@@ -185,7 +211,9 @@ define([
             );
           }
 
-          setTimeout(function() {self.show.apply(self)}, 10);
+          setTimeout(function() {
+            self.show.apply(self);
+          }, 10);
         }
 
         if (!self.completed) {
@@ -203,18 +231,22 @@ define([
         $.ajax({
           typs: 'GET',
           timeout: 30000,
-          url: self.url('status'),
+          url: self.bgprocess_url('status'),
           cache: false,
           async: true,
-          contentType: "application/json",
-          success: function(res) {
-            setTimeout(function() { self.update(res); }, 10);
-          },
-          error: function(res) {
-            // Try after some time only if job id present
-            if (res.status != 410)
-              setTimeout(function() { self.update(res); }, 10000);
-          }
+          contentType: 'application/json',
+        })
+        .done(function(res) {
+          setTimeout(function() {
+            self.update(res);
+          }, 10);
+        })
+        .fail(function(res) {
+          // Try after some time only if job id present
+          if (res.status != 410)
+            setTimeout(function() {
+              self.update(res);
+            }, 10000);
         });
       },
 
@@ -224,35 +256,57 @@ define([
         if (self.notify && !self.details) {
           if (!self.notifier) {
             var header = $('<div></div>', {
-                  class: "h5 pg-bg-notify-header"
-                }).append($('<span></span>').text(self.desc)),
-                content = $('<div class="pg-bg-bgprocess row"></div>').append(
-                  header
-                ).append(
-                  $('<div></div>', {class: 'pg-bg-notify-body h6' }).append(
-                    $('<div></div>', {class: 'pg-bg-start col-xs-12' }).append(
-                      $('<div></div>').text(self.stime.toString())
-                    ).append(
-                      $('<div class="pg-bg-etime"></div>')
-                    )
-                  )
-                ),
-                for_details = $('<div></div>', {
-                  class: "col-xs-12 text-center pg-bg-click h6"
+                class: 'pg-bg-notify-header',
+              }).append($('<span></span>').text(_.unescape(self.desc))),
+              content = $('<div class="pg-bg-bgprocess"></div>').append(
+                header
+              ).append(
+                $('<div></div>', {
+                  class: 'pg-bg-notify-body',
                 }).append(
-                  $('<span></span>').text(pgMessages.CLICK_FOR_DETAILED_MSG)
-                ).appendTo(content),
-                status = $('<div></div>', {
-                  class: "pg-bg-status col-xs-12 h5 " + ((self.exit_code === 0) ?
-                      'bg-success': (self.exit_code == 1) ?
-                      'bg-failed' : '')
-                }).appendTo(content),
-                close_me = $(
-                  '<div class="bg-close"><i class="fa fa-close"></i></div>'
-                ).appendTo(header);
+                  $('<div></div>', {
+                    class: 'pg-bg-start',
+                  }).append(
+                    $('<div></div>', {
+                      class: 'row align-items-center',
+                    }).append(
+                      $('<div></div>', {
+                        class: 'col-9',
+                      }).text(self.stime.toString())
+                    ).append(
+                      $('<div></div>', {
+                        class: 'col-3',
+                      }).append(
+                        $('<button></button>', {
+                          type: 'button',
+                          class: 'btn btn-danger btn-sm float-right bg-process-stop',
+                        }).text('Stop Process')
+                      )
+                    )
+                  ).append(
+                    $('<div class="pg-bg-etime"></div>')
+                  )
+                )
+              ),
+              for_details = $('<div></div>', {
+                class: 'text-center pg-bg-click',
+              }).append(
+                $('<span></span>').text(gettext('Click here for details.'))
+              ).appendTo(content),
+              close_me = $(
+                '<div class="bg-close"><i class="fa fa-close"></i></div>'
+              ).appendTo(header);
+
+            $('<div></div>', {
+              class: 'pg-bg-status ' + ((self.exit_code === 0) ?
+                'bg-success' : (self.exit_code == 1) ?
+                'bg-failed' : ''),
+            }).appendTo(content);
+
+            $('<div></div>').appendTo(content);
 
             self.container = content;
-            self.notifier = alertify.notify(
+            self.notifier = Alertify.notify(
               content.get(0), 'bg-bgprocess', 0, null
             );
 
@@ -268,7 +322,7 @@ define([
               this.show_detailed_view.apply(this);
             }.bind(self));
 
-            close_me.on('click', function(ev) {
+            close_me.on('click', function() {
               this.notifier.dismiss();
               this.notifier = null;
               this.acknowledge_server.apply(this);
@@ -283,6 +337,9 @@ define([
 
               return;
             });
+
+            // On Click event to stop the process.
+            content.find('.bg-process-stop').off('click').on('click', self.stop_process.bind(this));
           }
           // TODO:: Formatted execution time
           self.container.find('.pg-bg-etime').empty().append(
@@ -290,46 +347,76 @@ define([
               String(self.execution_time)
             )
           ).append(
-            $('<span></span>').text(' ' + pgMessages['seconds'])
+            $('<span></span>').text(' ' + gettext('seconds'))
           );
-          self.container.find('.pg-bg-status').empty().append(
+          var $status_bar = $(self.container.find('.pg-bg-status'));
+          $status_bar.empty().append(
             self.curr_status
           );
+
+          // Enable/Disable stop process button
+          var $btn_stop_process = $(self.container.find('.bg-process-stop'));
+          if (isNaN(parseInt(self.exit_code))) {
+            $btn_stop_process.attr('disabled', false);
+          } else {
+            $btn_stop_process.attr('disabled', true);
+            if (self.exit_code === 0 && self.state != 3) {
+              $status_bar.addClass('bg-success');
+              $status_bar.removeClass('bg-failed');
+            } else {
+              $status_bar.addClass('bg-failed');
+              $status_bar.removeClass('bg-success');
+            }
+          }
         } else {
-          self.show_detailed_view.apply(self)
+          self.show_detailed_view.apply(self);
         }
       },
 
       show_detailed_view: function() {
         var self = this,
-            panel = this.panel,
-            is_new = false;
+          panel = this.panel,
+          is_new = false;
 
         if (!self.panel) {
           is_new = true;
           panel = this.panel =
-              pgBrowser.BackgroundProcessObsorver.create_panel();
+            pgBrowser.BackgroundProcessObsorver.create_panel();
 
-          panel.title('Process Watcher - ' + self.desc);
+          panel.title('Process Watcher - ' + _.escape(self.desc));
           panel.focus();
         }
 
         var container = panel.$container,
-            status_class = (
-              (self.exit_code === 0) ?
-                'bg-bgprocess-success': (self.exit_code == 1) ?
-                  'bg-bgprocess-failed' : ''
-            ),
-            $logs = container.find('.bg-process-watcher'),
-            $header = container.find('.bg-process-details'),
-            $footer = container.find('.bg-process-footer');
+          status_class = '',
+          $logs = container.find('.bg-process-watcher'),
+          $header = container.find('.bg-process-details'),
+          $footer = container.find('.bg-process-footer'),
+          $btn_stop_process = container.find('.bg-process-stop');
+
+        // Enable/Disable stop process button
+        if (isNaN(parseInt(self.exit_code))) {
+          $btn_stop_process.attr('disabled', false);
+        } else {
+          $btn_stop_process.attr('disabled', true);
+          if (self.exit_code === 0 && self.state != 3) {
+            status_class = 'bg-bgprocess-success';
+          } else {
+            status_class = 'bg-bgprocess-failed';
+          }
+        }
+
+        // On Click event to stop the process.
+        $btn_stop_process.off('click').on('click', self.stop_process.bind(this));
 
         if (is_new) {
           // set logs
           $logs.html(self.logs);
-
+          setTimeout(function() {
+            self.logs[0].scrollTop = self.logs[0].scrollHeight;
+          });
           // set bgprocess detailed description
-          $header.find('.bg-detailed-desc').html(self.detailed_desc);
+          $header.find('.bg-detailed-desc').html(_.unescape(self.detailed_desc));
         }
 
         // set bgprocess start time
@@ -348,7 +435,7 @@ define([
             String(self.execution_time)
           )
         ).append(
-          $('<span></span>').text(' ' + pgMessages['seconds'])
+          $('<span></span>').text(' ' + gettext('seconds'))
         );
 
         if (is_new) {
@@ -388,17 +475,39 @@ define([
         $.ajax({
           type: 'PUT',
           timeout: 30000,
-          url: self.url('acknowledge'),
+          url: self.bgprocess_url('acknowledge'),
           cache: false,
           async: true,
-          contentType: "application/json",
-          success: function(res) {
-            return;
-          },
-          error: function(res) {
-          }
+          contentType: 'application/json',
+        })
+        .done(function() {
+          return;
+        })
+        .fail(function() {
+          console.warn(arguments);
         });
-      }
+      },
+
+      stop_process: function() {
+        var self = this;
+        // Set the state to terminated.
+        self.state = 3;
+        $.ajax({
+          type: 'PUT',
+          timeout: 30000,
+          url: self.bgprocess_url('stop_process'),
+          cache: false,
+          async: true,
+          contentType: 'application/json',
+        })
+        .done(function() {
+          return;
+        })
+        .fail(function() {
+          console.warn(arguments);
+        });
+      },
+
     });
 
   _.extend(
@@ -427,7 +536,7 @@ define([
               }, 1000
             );
           }
-        )
+        );
       },
 
       update_process_list: function(recheck) {
@@ -436,35 +545,35 @@ define([
         $.ajax({
           typs: 'GET',
           timeout: 30000,
-          url: pgMessages['bgprocess.index'],
+          url: url_for('bgprocess.list'),
           cache: false,
           async: true,
-          contentType: "application/json",
-          success: function(res) {
-            var cnt = 0;
-            if (!res || !_.isArray(res)) {
-              return;
-            }
-            for (idx in res) {
-              var process = res[idx];
-              if ('id' in process) {
-                if (!(process.id in observer.bgprocesses)) {
-                  observer.bgprocesses[process.id] = new BGProcess(process);
-                }
+          contentType: 'application/json',
+        })
+        .done(function(res) {
+          if (!res || !_.isArray(res)) {
+            return;
+          }
+          for (var idx in res) {
+            var process = res[idx];
+            if ('id' in process) {
+              if (!(process.id in observer.bgprocesses)) {
+                observer.bgprocesses[process.id] = new BGProcess(process);
               }
             }
-            if (recheck && res.length == 0) {
-              // Recheck after some more time
-              setTimeout(
-                function() {
-                  observer.update_process_list(false);
-                }, 3000
-              );
-            }
-          },
-          error: function(res) {
-            // FIXME:: What to do now?
           }
+          if (recheck && res.length == 0) {
+            // Recheck after some more time
+            setTimeout(
+              function() {
+                observer.update_process_list(false);
+              }, 3000
+            );
+          }
+        })
+        .fail(function() {
+          // FIXME:: What to do now?
+          console.warn(arguments);
         });
       },
 
@@ -476,11 +585,11 @@ define([
           wcDocker.DOCK.FLOAT,
           null, {
             w: (screen.width < 700 ?
-                screen.width * 0.95 : screen.width * 0.5),
-                h: (screen.height < 500 ?
-                    screen.height * 0.95 : screen.height * 0.5),
-                    x: (screen.width < 700 ? '2%' : '25%'),
-                    y: (screen.height < 500 ? '2%' : '25%')
+              screen.width * 0.95 : screen.width * 0.5),
+            h: (screen.height < 500 ?
+              screen.height * 0.95 : screen.height * 0.5),
+            x: (screen.width < 700 ? '2%' : '25%'),
+            y: (screen.height < 500 ? '2%' : '25%'),
           });
       },
 
@@ -488,39 +597,47 @@ define([
         var w = pgBrowser.docker,
           panels = w.findPanels('bg_process_watcher');
 
-          if (panels && panels.length >= 1)
-            return;
+        if (panels && panels.length >= 1)
+          return;
 
-          var p = new pgBrowser.Panel({
-                name: 'bg_process_watcher',
-                showTitle: true,
-                isCloseable: true,
-                isPrivate: true,
-                content: '<div class="bg-process-details col-xs-12">'+
-                  '<p class="bg-detailed-desc"></p>'+
-                  '<div class="bg-process-stats">'+
-                  '<span><b>' + pgMessages['START_TIME'] + ': </b>'+
-                  '<span class="bgprocess-start-time"></span>'+
-                  '</span></div>'+
-                  '</div>'+
-                  '<div class="bg-process-watcher col-xs-12">'+
-                  '</div>'+
-                  '<div class="bg-process-footer col-xs-12">'+
-                  '<div class="bg-process-status col-xs-6">'+
-                  '<span><b>' + pgMessages['STATUS'] + ':</b></span><p></p>'+
-                  '</div>'+
-                  '<div class="bg-process-exec-time col-xs-6">'+
-                  '<div class="exec-div pull-right">'+
-                  '<span><b>' + pgMessages['EXECUTION_TIME'] + ':</b></span><p></p>'+
-                  '</div>'+
-                  '</div>'+
-                  '</div>',
-                onCreate: function(myPanel, $container) {
-                  $container.addClass('pg-no-overflow');
-                }
-              });
-          p.load(pgBrowser.docker);
-      }
+        var p = new pgBrowser.Panel({
+          name: 'bg_process_watcher',
+          showTitle: true,
+          isCloseable: true,
+          isPrivate: true,
+          content: '<div class="bg-process-details">' +
+            '<p class="bg-detailed-desc"></p>' +
+            '<div class="bg-process-stats" style="padding-bottom: 5px">' +
+              '<div class="row align-items-center">' +
+                '<div class="col-9">' +
+                  '<span><b>' + gettext('Start time') + ': </b>' +
+                    '<span class="bgprocess-start-time"></span>' +
+                  '</span>' +
+                '</div>' +
+                '<div class="col-3">' +
+                  '<button type="button" class="btn btn-danger btn-sm float-right bg-process-stop">' +
+                    gettext('Stop Process') + '</button>' +
+                '</div>' +
+              '</div>' +
+            '</div>' +
+            '<div class="bg-process-watcher">' +
+            '</div>' +
+            '<div class="bg-process-footer row">' +
+              '<div class="bg-process-status col-6">' +
+              '<span><b>' + gettext('Status') + ':</b></span><p></p>' +
+              '</div>' +
+              '<div class="bg-process-exec-time col-6">' +
+              '<div class="exec-div pull-right">' +
+              '<span><b>' + gettext('Execution time') + ':</b></span><p></p>' +
+              '</div>' +
+            '</div>' +
+            '</div>',
+          onCreate: function(myPanel, $container) {
+            $container.addClass('pg-no-overflow');
+          },
+        });
+        p.load(pgBrowser.docker);
+      },
     });
 
   return pgBrowser.BackgroundProcessObsorver;
