@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -20,12 +20,14 @@ import config
 import codecs
 
 import simplejson as json
-from flask import render_template, Response, session, request as req, url_for
-from flask_babel import gettext
+from flask import render_template, Response, session, request as req, \
+    url_for, current_app
+from flask_babelex import gettext
 from flask_security import login_required
 from pgadmin.utils import PgAdminModule
 from pgadmin.utils import get_storage_directory
 from pgadmin.utils.ajax import make_json_response
+from pgadmin.utils.preferences import Preferences
 
 # Checks if platform is Windows
 if _platform == "win32":
@@ -135,8 +137,7 @@ class FileManagerModule(PgAdminModule):
 
     def get_own_stylesheets(self):
         return [
-            url_for('static', filename='vendor/jquery.dropzone/dropzone.css'),
-            url_for('file_manager.static', filename='css/file_manager.css')
+            url_for('static', filename='vendor/jquery.dropzone/dropzone.css')
         ]
 
     def get_own_menuitems(self):
@@ -144,6 +145,21 @@ class FileManagerModule(PgAdminModule):
             'file_items': []
         }
 
+    def get_exposed_url_endpoints(self):
+        """
+        Returns:
+            list: a list of url endpoints exposed to the client.
+        """
+        return [
+            'file_manager.filemanager',
+            'file_manager.index',
+            'file_manager.get_trans_id',
+            'file_manager.delete_trans_id',
+            'file_manager.save_last_dir',
+            'file_manager.save_file_dialog_view',
+            'file_manager.save_show_hidden_file_option'
+        ]
+
     def get_file_size_preference(self):
         return self.file_upload_size
 
@@ -159,13 +175,25 @@ class FileManagerModule(PgAdminModule):
             gettext("Last directory visited"), 'text', '/',
             category_label=gettext('Options')
         )
+        self.file_dialog_view = self.preference.register(
+            'options', 'file_dialog_view',
+            gettext("File dialog view"), 'options', 'list',
+            category_label=gettext('Options'),
+            options=[{'label': gettext('List'), 'value': 'list'},
+                     {'label': gettext('Grid'), 'value': 'grid'}]
+        )
+        self.show_hidden_files = self.preference.register(
+            'options', 'show_hidden_files',
+            gettext("Show hidden files and folders?"), 'boolean', False,
+            category_label=gettext('Options')
+        )
 
 
 # Initialise the module
 blueprint = FileManagerModule(MODULE_NAME, __name__)
 
 
-@blueprint.route("/")
+@blueprint.route("/", endpoint='index')
 @login_required
 def index():
     """Render the preferences dialog."""
@@ -219,14 +247,25 @@ def file_manager_config(trans_id):
     """render the required json"""
     # trans_id = Filemanager.create_new_transaction()
     data = Filemanager.get_trasaction_selection(trans_id)
+    pref = Preferences.module('file_manager')
+    file_dialog_view = pref.preference('file_dialog_view').get()
+    show_hidden_files = pref.preference('show_hidden_files').get()
+
     return Response(response=render_template(
-        "file_manager/js/file_manager_config.json", _=gettext,
-        data=data),
+        "file_manager/js/file_manager_config.json",
+        _=gettext,
+        data=data,
+        file_dialog_view=file_dialog_view,
+        show_hidden_files=show_hidden_files
+    ),
         status=200,
-        mimetype="application/json")
+        mimetype="application/json"
+    )
 
 
-@blueprint.route("/get_trans_id", methods=["GET", "POST"])
+@blueprint.route(
+    "/get_trans_id", methods=["GET", "POST"], endpoint='get_trans_id'
+)
 @login_required
 def get_trans_id():
     if len(req.data) != 0:
@@ -239,7 +278,10 @@ def get_trans_id():
     )
 
 
-@blueprint.route("/del_trans_id/<int:trans_id>", methods=["GET", "POST"])
+@blueprint.route(
+    "/del_trans_id/<int:trans_id>",
+    methods=["GET", "POST"], endpoint='delete_trans_id'
+)
 @login_required
 def delete_trans_id(trans_id):
     Filemanager.release_transaction(trans_id)
@@ -248,7 +290,9 @@ def delete_trans_id(trans_id):
     )
 
 
-@blueprint.route("/save_last_dir/<int:trans_id>", methods=["POST"])
+@blueprint.route(
+    "/save_last_dir/<int:trans_id>", methods=["POST"], endpoint='save_last_dir'
+)
 @login_required
 def save_last_directory_visited(trans_id):
     blueprint.last_directory_visited.set(req.json['path'])
@@ -257,6 +301,30 @@ def save_last_directory_visited(trans_id):
     )
 
 
+@blueprint.route(
+    "/save_file_dialog_view/<int:trans_id>", methods=["POST"],
+    endpoint='save_file_dialog_view'
+)
+@login_required
+def save_file_dialog_view(trans_id):
+    blueprint.file_dialog_view.set(req.json['view'])
+    return make_json_response(
+        data={'status': True}
+    )
+
+
+@blueprint.route(
+    "/save_show_hidden_file_option/<int:trans_id>", methods=["PUT"],
+    endpoint='save_show_hidden_file_option'
+)
+@login_required
+def save_show_hidden_file_option(trans_id):
+    blueprint.show_hidden_files.set(req.json['show_hidden'])
+    return make_json_response(
+        data={'status': True}
+    )
+
+
 class Filemanager(object):
     """FileManager Class."""
 
@@ -264,7 +332,9 @@ class Filemanager(object):
         self.trans_id = trans_id
         self.patherror = encode_json(
             {
-                'Error': gettext('No permission to operate on specified path.'),
+                'Error': gettext(
+                    'No permission to operate on specified path.'
+                ),
                 'Code': 0
             }
         )
@@ -340,7 +410,8 @@ class Filemanager(object):
                 last_dir = last_dir[:-1]
             while last_dir:
                 if os.path.exists(
-                        (storage_dir if storage_dir is not None else '') + last_dir):
+                        storage_dir
+                        if storage_dir is not None else '' + last_dir):
                     break
                 if _platform == 'win32':
                     index = max(last_dir.rfind('\\'), last_dir.rfind('/'))
@@ -359,7 +430,8 @@ class Filemanager(object):
 
         # create configs using above configs
         configs = {
-            "fileroot": last_dir.replace('\\', '\\\\'),  # for JS json compatibility
+            # for JS json compatibility
+            "fileroot": last_dir.replace('\\', '\\\\'),
             "dialog_type": fm_type,
             "title": title,
             "upload": {
@@ -449,7 +521,7 @@ class Filemanager(object):
                         drives.append(letter)
                     bitmask >>= 1
                 if (drive_name != '' and drive_name is not None and
-                            drive_name in drives):
+                        drive_name in drives):
                     return u"{0}{1}".format(drive_name, ':')
                 else:
                     return drives  # return drives if no argument is passed
@@ -477,12 +549,14 @@ class Filemanager(object):
             kernel32.SetThreadErrorMode(oldmode, ctypes.byref(oldmode))
 
     @staticmethod
-    def list_filesystem(dir, path, trans_data, file_type):
+    def list_filesystem(dir, path, trans_data, file_type, show_hidden):
         """
         It lists all file and folders within the given
         directory.
         """
         Filemanager.suspend_windows_warning()
+        is_show_hidden_files = show_hidden
+
         path = unquote(path)
         if hasattr(str, 'decode'):
             path = unquote(path).encode('utf-8').decode('utf-8')
@@ -508,7 +582,7 @@ class Filemanager(object):
                 try:
                     drive_size = getDriveSize(path)
                     drive_size_in_units = sizeof_fmt(drive_size)
-                except:
+                except Exception:
                     drive_size = 0
                 protected = 1 if drive_size == 0 else 0
                 files[file_name] = {
@@ -537,10 +611,10 @@ class Filemanager(object):
             }
 
         user_dir = path
-        folders_only = trans_data['folders_only'] if 'folders_only' in \
-                                                     trans_data else ''
-        files_only = trans_data['files_only'] if 'files_only' in \
-                                                 trans_data else ''
+        folders_only = trans_data['folders_only'] \
+            if 'folders_only' in trans_data else ''
+        files_only = trans_data['files_only'] \
+            if 'files_only' in trans_data else ''
         supported_types = trans_data['supported_types'] \
             if 'supported_types' in trans_data else []
 
@@ -551,8 +625,9 @@ class Filemanager(object):
                 protected = 0
                 system_path = os.path.join(os.path.join(orig_path, f))
 
-                # continue if file/folder is hidden
-                if is_folder_hidden(system_path) or f.startswith('.'):
+                # continue if file/folder is hidden (based on user preference)
+                if not is_show_hidden_files and \
+                        (is_folder_hidden(system_path) or f.startswith('.')):
                     continue
 
                 user_path = os.path.join(os.path.join(user_dir, f))
@@ -575,8 +650,8 @@ class Filemanager(object):
                     # filter files based on file_type
                     if file_type is not None and file_type != "*":
                         if folders_only or len(supported_types) > 0 and \
-                                        file_extension not in supported_types or \
-                                        file_type != file_extension:
+                                file_extension not in supported_types or \
+                                file_type != file_extension:
                             continue
 
                 # create a list of files and folders
@@ -720,8 +795,9 @@ class Filemanager(object):
         }
 
         if not path_exists(orig_path):
-            thefile['Error'] = gettext(u"'{0}' file does not exist.".format(
-                path))
+            thefile['Error'] = gettext(
+                u"'{0}' file does not exist.".format(path)
+            )
             thefile['Code'] = -1
             return thefile
 
@@ -740,7 +816,8 @@ class Filemanager(object):
 
         return thefile
 
-    def getfolder(self, path=None, file_type="", name=None, req=None):
+    def getfolder(self, path=None, file_type="", name=None, req=None,
+                  show_hidden=False):
         """
         Returns files and folders in give path
         """
@@ -751,7 +828,8 @@ class Filemanager(object):
             if not dir.endswith('/'):
                 dir += u'/'
 
-        filelist = self.list_filesystem(dir, path, trans_data, file_type)
+        filelist = self.list_filesystem(
+            dir, path, trans_data, file_type, show_hidden)
         return filelist
 
     def rename(self, old=None, new=None, req=None):
@@ -830,7 +908,8 @@ class Filemanager(object):
             }
 
         dir = self.dir if self.dir is not None else ''
-        path = path.encode('utf-8').decode('utf-8') if hasattr(str, 'decode') else path
+        path = path.encode(
+            'utf-8').decode('utf-8') if hasattr(str, 'decode') else path
         orig_path = u"{0}{1}".format(dir, path)
 
         try:
@@ -880,16 +959,23 @@ class Filemanager(object):
             file_obj = req.files['newfile']
             file_name = file_obj.filename
             if hasattr(str, 'decode'):
-                path = req.form.get('currentpath').encode('utf-8').decode('utf-8')
+                path = req.form.get('currentpath').encode(
+                    'utf-8').decode('utf-8')
                 file_name = file_obj.filename.encode('utf-8').decode('utf-8')
             orig_path = u"{0}{1}".format(dir, path)
             newName = u"{0}{1}".format(orig_path, file_name)
 
             with open(newName, 'wb') as f:
-                f.write(file_obj.read())
+                while True:
+                    # 4MB chunk (4 * 1024 * 1024 Bytes)
+                    data = file_obj.read(4194304)
+                    if not data:
+                        break
+                    f.write(data)
         except Exception as e:
             code = 0
-            err_msg = u"Error: {0}".format(e.strerror)
+            err_msg = u"Error: {0}".format(
+                e.strerror if hasattr(e, 'strerror') else u'Unknown')
 
         try:
             Filemanager.check_access_permission(dir, path)
@@ -923,7 +1009,8 @@ class Filemanager(object):
             path = path.encode('utf-8').decode('utf-8')
         try:
             orig_path = u"{0}{1}".format(dir, path)
-            Filemanager.check_access_permission(dir, u"{}{}".format(path, name))
+            Filemanager.check_access_permission(
+                dir, u"{}{}".format(path, name))
 
             newName = u"{0}{1}".format(orig_path, name)
             if not os.path.exists(newName):
@@ -980,17 +1067,18 @@ class Filemanager(object):
         status = True
         err_msg = None
         is_startswith_bom = False
+        is_binary = False
 
         # check if file type is text or binary
         text_chars = bytearray([7, 8, 9, 10, 12, 13, 27]) \
-                    + bytearray(range(0x20, 0x7f)) \
-                    + bytearray(range(0x80, 0x100))
+            + bytearray(range(0x20, 0x7f)) \
+            + bytearray(range(0x80, 0x100))
 
         def is_binary_string(bytes_data):
             """Checks if string data is binary"""
             return bool(
-                    bytes_data.translate(None, text_chars)
-                )
+                bytes_data.translate(None, text_chars)
+            )
 
         # read the file
         try:
@@ -1023,6 +1111,13 @@ class Filemanager(object):
             status = False
             err_msg = u"Error: {0}".format(str(ex))
 
+        # Remove root storage path from error message
+        # when running in Server mode
+        if not status and not current_app.PGADMIN_RUNTIME:
+            storage_directory = get_storage_directory()
+            if storage_directory:
+                err_msg = err_msg.replace(storage_directory, '')
+
         return status, err_msg, is_binary, is_startswith_bom, enc
 
     def addfolder(self, path, name):
@@ -1097,11 +1192,13 @@ class Filemanager(object):
             orig_path = u"{0}{1}".format(dir, path)
 
         try:
-            Filemanager.check_access_permission(dir, u"{}{}".format(
-                path, path))
+            Filemanager.check_access_permission(
+                dir, u"{}{}".format(path, path)
+            )
         except Exception as e:
             resp = Response(gettext(u"Error: {0}".format(e)))
-            resp.headers['Content-Disposition'] = 'attachment; filename=' + name
+            resp.headers['Content-Disposition'] = \
+                'attachment; filename=' + name
             return resp
 
         name = path.split('/')[-1]
@@ -1122,7 +1219,10 @@ class Filemanager(object):
         return res
 
 
-@blueprint.route("/filemanager/<int:trans_id>/", methods=["GET", "POST"])
+@blueprint.route(
+    "/filemanager/<int:trans_id>/",
+    methods=["GET", "POST"], endpoint='filemanager'
+)
 @login_required
 def file_manager(trans_id):
     """
