@@ -19,7 +19,7 @@ from flask_babelex import gettext as _
 from flask_security import login_required, current_user
 from pgadmin.misc.bgprocess.processes import BatchProcess, IProcessDesc
 from pgadmin.utils import PgAdminModule, get_storage_directory, html, \
-    fs_short_path, document_dir
+    fs_short_path, document_dir, is_utility_exists
 from pgadmin.utils.ajax import make_json_response, bad_request
 
 from config import PG_DEFAULT_DRIVER
@@ -63,7 +63,8 @@ class BackupModule(PgAdminModule):
         Returns:
             list: URL endpoints for backup module
         """
-        return ['backup.create_server_job', 'backup.create_object_job']
+        return ['backup.create_server_job', 'backup.create_object_job',
+                'backup.utility_exists']
 
 
 # Create blueprint for BackupModule class
@@ -158,7 +159,7 @@ class BackupMessage(IProcessDesc):
     def details(self, cmd, args):
         name, host, port = self.get_server_details()
 
-        res = '<div class="h5">'
+        res = '<div>'
 
         if self.backup_type == BACKUP.OBJECT:
             res += _(
@@ -193,7 +194,7 @@ class BackupMessage(IProcessDesc):
             # It should never reach here.
             res += "Backup"
 
-        res += '</div><div class="h5">'
+        res += '</div><div>'
         res += _("Running command:")
         res += '</b><br><span class="pg-bg-cmd enable-selection">'
         res += html.safe_str(cmd + self.cmd)
@@ -320,6 +321,13 @@ def create_backup_objects_job(sid):
     utility = manager.utility('backup') if backup_obj_type == 'objects' \
         else manager.utility('backup_server')
 
+    ret_val = is_utility_exists(utility)
+    if ret_val:
+        return make_json_response(
+            success=0,
+            errormsg=ret_val
+        )
+
     args = [
         '--file',
         backup_file,
@@ -461,3 +469,44 @@ def create_backup_objects_job(sid):
     return make_json_response(
         data={'job_id': jid, 'Success': 1}
     )
+
+
+@blueprint.route(
+    '/utility_exists/<int:sid>/<backup_obj_type>', endpoint='utility_exists'
+)
+@login_required
+def check_utility_exists(sid, backup_obj_type):
+    """
+    This function checks the utility file exist on the given path.
+
+    Args:
+        sid: Server ID
+        backup_obj_type: Type of the object
+    Returns:
+        None
+    """
+    server = Server.query.filter_by(
+        id=sid, user_id=current_user.id
+    ).first()
+
+    if server is None:
+        return make_json_response(
+            success=0,
+            errormsg=_("Could not find the specified server.")
+        )
+
+    from pgadmin.utils.driver import get_driver
+    driver = get_driver(PG_DEFAULT_DRIVER)
+    manager = driver.connection_manager(server.id)
+
+    utility = manager.utility('backup') if backup_obj_type == 'objects' \
+        else manager.utility('backup_server')
+
+    ret_val = is_utility_exists(utility)
+    if ret_val:
+        return make_json_response(
+            success=0,
+            errormsg=ret_val
+        )
+
+    return make_json_response(success=1)
