@@ -10,10 +10,11 @@
 
 import sys
 import simplejson as json
+import os
 
 from pgadmin.utils.route import BaseTestGenerator
 from regression import parent_node_dict
-from pgadmin.utils import server_utils as server_utils
+from pgadmin.utils import server_utils as server_utils, is_utility_exists
 from pgadmin.browser.server_groups.servers.databases.tests import utils as \
     database_utils
 
@@ -265,7 +266,7 @@ class BackupCreateJobTest(BaseTestGenerator):
              not_expected_cmd_opts=[],
              expected_exit_code=[0, None],
              server_min_version=110000,
-             message='Backup object with --no-comments are not supported '
+             message='Backup object with --no-comments is not supported '
                      'by EPAS/PG server less than 11.0'
          )),
         ('When backup the object with option - all queries',
@@ -532,7 +533,7 @@ class BackupCreateJobTest(BaseTestGenerator):
              not_expected_cmd_opts=[],
              expected_exit_code=[0, None],
              server_min_version=110000,
-             message='Backup server with --no-comments are not supported '
+             message='Backup server with --no-comments is not supported '
                      'by EPAS/PG server less than 11.0'
          )),
         ('When backup the server with option - all queries',
@@ -611,7 +612,7 @@ class BackupCreateJobTest(BaseTestGenerator):
              not_expected_cmd_opts=[],
              expected_exit_code=[0, None],
              server_min_version=110000,
-             message='Backup server with encoding are not supported '
+             message='Backup server with encoding is not supported '
                      'by EPAS/PG server less than 11.0'
          )),
         ('When backup globals',
@@ -639,13 +640,23 @@ class BackupCreateJobTest(BaseTestGenerator):
     ]
 
     def setUp(self):
-        if self.server['default_binary_paths'] is None:
+        if 'default_binary_paths' not in self.server or \
+            self.server['default_binary_paths'] is None or \
+            self.server['type'] not in self.server['default_binary_paths'] or \
+                self.server['default_binary_paths'][self.server['type']] == '':
             self.skipTest(
                 "default_binary_paths is not set for the server {0}".format(
                     self.server['name']
                 )
             )
 
+        binary_path = os.path.join(
+            self.server['default_binary_paths'][self.server['type']],
+            'pg_dump')
+        retVal = is_utility_exists(binary_path)
+        if retVal is not None:
+            self.skipTest(retVal)
+
     @patch('pgadmin.tools.backup.Server')
     @patch('pgadmin.tools.backup.BackupMessage')
     @patch('pgadmin.tools.backup.filename_with_file_manager_path')
