@@ -129,6 +129,7 @@ define([
       },
 
       setActiveLine: function(lineNo) {
+        var self = this;
         let editor = pgTools.DirectDebug.editor;
 
         /* If lineNo sent, remove active line */
@@ -390,12 +391,15 @@ define([
                   }
 
                   // Enable all the buttons as we got the results
-                  self.enable('stop', true);
-                  self.enable('step_over', true);
-                  self.enable('step_into', true);
-                  self.enable('continue', true);
-                  self.enable('toggle_breakpoint', true);
-                  self.enable('clear_all_breakpoints', true);
+                  // TODO: Fix this properly so a timeout isn't required.
+                  setTimeout(function() {
+                    self.enable('stop', true);
+                    self.enable('step_over', true);
+                    self.enable('step_into', true);
+                    self.enable('continue', true);
+                    self.enable('toggle_breakpoint', true);
+                    self.enable('clear_all_breakpoints', true);
+                  }, 500);
                 }
               } else if (res.data.status === 'Busy') {
                 pgTools.DirectDebug.polling_timeout_idle = true;
@@ -518,12 +522,15 @@ define([
                   // Execution completed so disable the buttons other than
                   // "Continue/Start" button because user can still
                   // start the same execution again.
-                  self.enable('stop', false);
-                  self.enable('step_over', false);
-                  self.enable('step_into', false);
-                  self.enable('toggle_breakpoint', false);
-                  self.enable('clear_all_breakpoints', false);
-                  self.enable('continue', true);
+                  setTimeout(function() {
+                    self.enable('stop', false);
+                    self.enable('step_over', false);
+                    self.enable('step_into', false);
+                    self.enable('toggle_breakpoint', false);
+                    self.enable('clear_all_breakpoints', false);
+                    self.enable('continue', true);
+                  }, 500);
+
                   // Stop further polling
                   pgTools.DirectDebug.is_polling_required = false;
                 } else {
@@ -549,12 +556,14 @@ define([
                     // Execution completed so disable the buttons other than
                     // "Continue/Start" button because user can still
                     // start the same execution again.
-                    self.enable('stop', false);
-                    self.enable('step_over', false);
-                    self.enable('step_into', false);
-                    self.enable('toggle_breakpoint', false);
-                    self.enable('clear_all_breakpoints', false);
-                    self.enable('continue', true);
+                    setTimeout(function() {
+                      self.enable('stop', false);
+                      self.enable('step_over', false);
+                      self.enable('step_into', false);
+                      self.enable('toggle_breakpoint', false);
+                      self.enable('clear_all_breakpoints', false);
+                      self.enable('continue', true);
+                    }, 500);
 
                     // Stop further pooling
                     pgTools.DirectDebug.is_polling_required = false;
@@ -1565,11 +1574,18 @@ define([
             controller.poll_result(trans_id);
           }
         })
-        .fail(function() {
-          Alertify.alert(
-            gettext('Debugger Error'),
-            gettext('Error while starting debugging listener.')
-          );
+        .fail(function(xhr) {
+          try {
+            var err = JSON.parse(xhr.responseText);
+            if (err.success == 0) {
+              Alertify.alert(gettext('Debugger Error'), err.errormsg);
+            }
+          } catch (e) {
+            Alertify.alert(
+              gettext('Debugger Error'),
+              gettext('Error while starting debugging listener.')
+            );
+          }
         });
       } else if (trans_id != undefined && debug_type) {
         // Make ajax call to execute the and start the target for execution
@@ -1586,11 +1602,18 @@ define([
             self.messages(trans_id);
           }
         })
-        .fail(function() {
-          Alertify.alert(
-            gettext('Debugger Error'),
-            gettext('Error while starting debugging listener.')
-          );
+        .fail(function(xhr) {
+          try {
+            var err = JSON.parse(xhr.responseText);
+            if (err.success == 0) {
+              Alertify.alert(gettext('Debugger Error'), err.errormsg);
+            }
+          } catch (e) {
+            Alertify.alert(
+              gettext('Debugger Error'),
+              gettext('Error while starting debugging listener.')
+            );
+          }
         });
       } else
         this.intializePanels();
