@@ -13,9 +13,9 @@ import simplejson as json
 
 from flask import url_for, Response, render_template, request, current_app
 from flask_babelex import gettext as _
-from flask_security import login_required
+from flask_security import login_required, current_user
 from pgadmin.misc.bgprocess.processes import BatchProcess, IProcessDesc
-from pgadmin.utils import PgAdminModule, html
+from pgadmin.utils import PgAdminModule, html, is_utility_exists
 from pgadmin.utils.ajax import bad_request, make_json_response
 from pgadmin.utils.driver import get_driver
 
@@ -68,7 +68,7 @@ class MaintenanceModule(PgAdminModule):
         Returns:
             list: URL endpoints for backup module
         """
-        return ['maintenance.create_job']
+        return ['maintenance.create_job', 'maintenance.utility_exists']
 
 
 blueprint = MaintenanceModule(MODULE_NAME, __name__)
@@ -128,9 +128,9 @@ class Message(IProcessDesc):
         if self.data['op'] == "CLUSTER":
             res = _('CLUSTER')
 
-        res = '<div class="h5">' + html.safe_str(res)
+        res = '<div>' + html.safe_str(res)
 
-        res += '</div><div class="h5">'
+        res += '</div><div>'
         res += html.safe_str(
             _("Running Query:")
         )
@@ -214,6 +214,12 @@ def create_maintenance_job(sid, did):
         )
 
     utility = manager.utility('sql')
+    ret_val = is_utility_exists(utility)
+    if ret_val:
+        return make_json_response(
+            success=0,
+            errormsg=ret_val
+        )
 
     # Create the command for the vacuum operation
     query = render_template(
@@ -262,3 +268,41 @@ def create_maintenance_job(sid, did):
         data={'job_id': jid, 'status': True,
               'info': 'Maintenance job created.'}
     )
+
+
+@blueprint.route(
+    '/utility_exists/<int:sid>', endpoint='utility_exists'
+)
+@login_required
+def check_utility_exists(sid):
+    """
+    This function checks the utility file exist on the given path.
+
+    Args:
+        sid: Server ID
+    Returns:
+        None
+    """
+    server = Server.query.filter_by(
+        id=sid, user_id=current_user.id
+    ).first()
+
+    if server is None:
+        return make_json_response(
+            success=0,
+            errormsg=_("Could not find the specified server.")
+        )
+
+    from pgadmin.utils.driver import get_driver
+    driver = get_driver(PG_DEFAULT_DRIVER)
+    manager = driver.connection_manager(server.id)
+
+    utility = manager.utility('sql')
+    ret_val = is_utility_exists(utility)
+    if ret_val:
+        return make_json_response(
+            success=0,
+            errormsg=ret_val
+        )
+
+    return make_json_response(success=1)
