@@ -34,7 +34,7 @@ define([
       '  <span class="<%=Backform.helpMessageClassName%>"><%=helpMessage%></span>',
       '<% } %>',
     ].join('\n')),
-    className: 'pgadmin-control-group form-group col-xs-6',
+    className: 'pgadmin-control-group form-group col-6',
   });
 
   // Main model for Maintenance functionality
@@ -80,8 +80,8 @@ define([
       ],
       control: Backform.RadioControl.extend({
         template: _.template([
-          '<label class="control-label col-sm-4 col-xs-12"><%=label%></label>',
-          '<div class="pgadmin-controls col-xs-12 col-sm-8 btn-group pg-maintenance-op" data-toggle="buttons">',
+          '<label class="control-label col-sm-4 col-12"><%=label%></label>',
+          '<div class="pgadmin-controls col-12 col-sm-8 btn-group pg-maintenance-op" data-toggle="buttons">',
           ' <% for (var i=0; i < options.length; i++) { %>',
           ' <% var option = options[i]; %>',
           ' <label class="btn btn-primary<% if (i == 0) { %> active<%}%>">',
@@ -283,8 +283,8 @@ define([
             setup: function() {
               return {
                 buttons: [{
-                  text: '',
-                  className: 'btn btn-default pull-left fa fa-lg fa-info',
+                  text: '<span class="fa fa-lg fa-info"></span>',
+                  className: 'btn btn-default pull-left',
                   attrs: {
                     name: 'object_help',
                     type: 'button',
@@ -292,9 +292,9 @@ define([
                     label: gettext('Maintenance'),
                   },
                 }, {
-                  text: '',
+                  text: '<span class="fa fa-lg fa-question"></span>',
                   key: 112,
-                  className: 'btn btn-default pull-left fa fa-lg fa-question',
+                  className: 'btn btn-default pull-left',
                   attrs: {
                     name: 'dialog_help',
                     type: 'button',
@@ -392,7 +392,10 @@ define([
                     Alertify.success(res.data.info);
                     pgBrowser.Events.trigger('pgadmin-bgprocess:created', self);
                   } else {
-                    Alertify.error(res.data.errmsg);
+                    Alertify.alert(
+                      gettext('Maintenance job creation failed.'),
+                      res.errormsg
+                    );
                   }
                 })
                 .fail(function() {
@@ -467,8 +470,33 @@ define([
         });
       }
 
-      // Open the Alertify dialog
-      Alertify.MaintenanceDialog('Maintenance...').set('resizable', true).resizeTo('60%', '80%');
+      const baseUrl = url_for('maintenance.utility_exists', {
+        'sid': server_data._id,
+      });
+
+      // Check psql utility exists or not.
+      $.ajax({
+        url: baseUrl,
+        type:'GET',
+      })
+      .done(function(res) {
+        if (!res.success) {
+          Alertify.alert(
+            gettext('Utility not found'),
+            res.errormsg
+          );
+          return;
+        }
+        // Open the Alertify dialog
+        Alertify.MaintenanceDialog('Maintenance...').set('resizable', true).resizeTo('60%', '80%');
+      })
+      .fail(function() {
+        Alertify.alert(
+          gettext('Utility not found'),
+          gettext('Failed to fetch Utility information')
+        );
+        return;
+      });
     },
   };
 
