@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -19,14 +19,25 @@ import signal
 import sys
 import traceback
 import json
+import random
 
 from selenium import webdriver
+from selenium.webdriver.chrome.options import Options
+from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
 
 if sys.version_info < (2, 7):
     import unittest2 as unit_test
 else:
     import unittest as unit_test
 
+if sys.version_info[0] >= 3:
+    import builtins
+else:
+    import __builtin__ as builtins
+
+# Ensure the global server mode is set.
+builtins.SERVER_MODE = None
+
 logger = logging.getLogger(__name__)
 file_name = os.path.basename(__file__)
 
@@ -43,6 +54,12 @@ if sys.path[0] != root:
 
 from pgadmin import create_app
 import config
+
+if config.SERVER_MODE is True:
+    config.SECURITY_RECOVERABLE = True
+    config.SECURITY_CHANGEABLE = True
+    config.SECURITY_POST_CHANGE_VIEW = 'browser.change_password'
+
 from regression import test_setup
 from regression.feature_utils.app_starter import AppStarter
 
@@ -73,7 +90,7 @@ if pgadmin_credentials:
                 'login_password'])
 
 # Execute the setup file
-exec (open("setup.py").read())
+exec(open("setup.py").read())
 
 # Get the config database schema version. We store this in pgadmin.model
 # as it turns out that putting it in the config files isn't a great idea
@@ -93,10 +110,14 @@ config.CONSOLE_LOG_LEVEL = WARNING
 app = create_app()
 app.config['WTF_CSRF_ENABLED'] = False
 app.PGADMIN_KEY = ''
+app.config.update({'SESSION_COOKIE_DOMAIN': None})
 test_client = app.test_client()
 driver = None
 app_starter = None
 handle_cleanup = None
+app.PGADMIN_RUNTIME = True
+if config.SERVER_MODE is True:
+    app.PGADMIN_RUNTIME = False
 
 setattr(unit_test.result.TestResult, "passed", [])
 
@@ -106,10 +127,12 @@ unit_test.runner.TextTestResult.addSuccess = test_utils.add_success
 scenarios.apply_scenario = test_utils.apply_scenario
 
 
-def get_suite(module_list, test_server, test_app_client):
+def get_suite(module_list, test_server, test_app_client, server_information,
+              test_db_name):
     """
      This function add the tests to test suite and return modified test suite
       variable.
+    :param server_information:
     :param module_list: test module list
     :type module_list: list
     :param test_server: server details
@@ -134,6 +157,8 @@ def get_suite(module_list, test_server, test_app_client):
         obj.setTestClient(test_app_client)
         obj.setTestServer(test_server)
         obj.setDriver(driver)
+        obj.setServerInformation(server_information)
+        obj.setTestDatabaseName(test_db_name)
         scenario = scenarios.generate_scenarios(obj)
         pgadmin_suite.addTests(scenario)
 
@@ -163,7 +188,40 @@ def get_test_modules(arguments):
         exclude_pkgs += arguments['exclude'].split(',')
 
     if 'feature_tests' not in exclude_pkgs:
-        driver = webdriver.Chrome()
+        default_browser = 'chrome'
+
+        # Check default browser provided through command line. If provided
+        # then use that browser as default browser else check for the setting
+        # provided in test_config.json file.
+        if (
+            'default_browser' in arguments and
+            arguments['default_browser'] is not None
+        ):
+            default_browser = arguments['default_browser'].lower()
+        elif (
+            test_setup.config_data and
+            "default_browser" in test_setup.config_data
+        ):
+            default_browser = test_setup.config_data['default_browser'].lower()
+
+        if default_browser == 'firefox':
+            cap = DesiredCapabilities.FIREFOX
+            cap['requireWindowFocus'] = True
+            cap['enablePersistentHover'] = False
+            profile = webdriver.FirefoxProfile()
+            profile.set_preference("dom.disable_beforeunload", True)
+            driver = webdriver.Firefox(capabilities=cap,
+                                       firefox_profile=profile)
+            driver.implicitly_wait(1)
+        else:
+            options = Options()
+            if test_setup.config_data:
+                if 'headless_chrome' in test_setup.config_data:
+                    if test_setup.config_data['headless_chrome']:
+                        options.add_argument("--headless")
+            options.add_argument("--window-size=1280,1024")
+            driver = webdriver.Chrome(chrome_options=options)
+
         app_starter = AppStarter(driver, config)
         app_starter.start_app()
 
@@ -182,7 +240,6 @@ def get_test_modules(arguments):
     # Sort module list so that test suite executes the test cases sequentially
     module_list = TestsGeneratorRegistry.registry.items()
     module_list = sorted(module_list, key=lambda module_tuple: module_tuple[0])
-
     return module_list
 
 
@@ -196,10 +253,19 @@ def add_arguments():
     """
 
     parser = argparse.ArgumentParser(description='Test suite for pgAdmin4')
-    parser.add_argument('--pkg', help='Executes the test cases of particular'
-                                      ' package and subpackages')
-    parser.add_argument('--exclude', help='Skips execution of the test '
-                                          'cases of particular package and sub-packages')
+    parser.add_argument(
+        '--pkg',
+        help='Executes the test cases of particular package and subpackages'
+    )
+    parser.add_argument(
+        '--exclude',
+        help='Skips execution of the test cases of particular package and '
+             'sub-packages'
+    )
+    parser.add_argument(
+        '--default_browser',
+        help='Executes the feature test in specific browser'
+    )
     arg = parser.parse_args()
 
     return arg
@@ -300,19 +366,23 @@ if __name__ == '__main__':
         signal.signal(getattr(signal, sig), sig_handler)
 
     # Set basic logging configuration for log file
-    logging.basicConfig(level=logging.DEBUG,
-                        format='%(asctime)s:%(levelname)s:%(name)s:%(message)s'
-                        ,
-                        filename=CURRENT_PATH + "/" + "regression.log",
-                        filemode='w'
-                        )
+    logging.basicConfig(
+        level=logging.DEBUG,
+        format='%(asctime)s:%(levelname)s:%(name)s:%(message)s',
+        filename=CURRENT_PATH + "/" + "regression.log",
+        filemode='w'
+    )
 
     # Create logger to write log in the logger file as well as on console
     stderr_logger = logging.getLogger('STDERR')
     sys.stderr = StreamToLogger(stderr_logger, logging.ERROR)
     args = vars(add_arguments())
     # Get test module list
-    test_module_list = get_test_modules(args)
+    try:
+        test_module_list = get_test_modules(args)
+    except Exception as e:
+        print(str(e))
+        sys.exit(1)
     # Login the test client
     test_utils.login_tester_account(test_client)
 
@@ -325,12 +395,40 @@ if __name__ == '__main__':
             print("\n=============Running the test cases for '%s'============="
                   % server['name'], file=sys.stderr)
             # Create test server
-            test_utils.create_parent_server_node(server)
+            server_information = test_utils.create_parent_server_node(server)
 
-            suite = get_suite(test_module_list, server, test_client)
+            # Create test database with random number to avoid conflict in
+            # parallel execution on different platforms. This database will be
+            # used across all feature tests.
+            test_db_name = "acceptance_test_db" + \
+                           str(random.randint(10000, 65535))
+            connection = test_utils.get_db_connection(
+                server['db'],
+                server['username'],
+                server['db_password'],
+                server['host'],
+                server['port'],
+                server['sslmode']
+            )
+
+            # Drop the database if already exists.
+            test_utils.drop_database(connection, test_db_name)
+            # Create database
+            test_utils.create_database(server, test_db_name)
+
+            if server['default_binary_paths'] is not None:
+                test_utils.set_preference(server['default_binary_paths'])
+
+            # Disable tree state saving
+            test_utils.disable_tree_state_save()
+
+            suite = get_suite(test_module_list,
+                              server,
+                              test_client,
+                              server_information, test_db_name)
             tests = unit_test.TextTestRunner(stream=sys.stderr,
-                                            descriptions=True,
-                                            verbosity=2).run(suite)
+                                             descriptions=True,
+                                             verbosity=2).run(suite)
 
             ran_tests, failed_cases, skipped_cases, passed_cases = \
                 get_tests_result(tests)
@@ -344,18 +442,27 @@ if __name__ == '__main__':
             if len(failed_cases) > 0:
                 failure = True
 
+            # Drop the testing database created initially
+            if connection:
+                test_utils.drop_database(connection, test_db_name)
+                connection.close()
+
             # Delete test server
             test_utils.delete_test_server(test_client)
     except SystemExit:
         if handle_cleanup:
             handle_cleanup()
 
-    print("\n==============================================================="
-          "=======", file=sys.stderr)
+    print(
+        "\n==============================================================="
+        "=======",
+        file=sys.stderr
+    )
     print("Test Result Summary", file=sys.stderr)
     print(
         "==================================================================="
-        "===\n", file=sys.stderr)
+        "===\n", file=sys.stderr
+    )
 
     test_result_json = {}
     for server_res in test_result:
@@ -390,10 +497,11 @@ if __name__ == '__main__':
             file=sys.stderr)
 
         temp_dict_for_server = {
-            server_res: {"tests_passed": [total_passed_cases, passed_cases],
-                         "tests_failed": [total_failed, failed_cases_json],
-                         "tests_skipped": [total_skipped, skipped_cases_json]
-                         }
+            server_res: {
+                "tests_passed": [total_passed_cases, passed_cases],
+                "tests_failed": [total_failed, failed_cases_json],
+                "tests_skipped": [total_skipped, skipped_cases_json]
+            }
         }
         test_result_json.update(temp_dict_for_server)
 
@@ -404,7 +512,9 @@ if __name__ == '__main__':
 
     print(
         "==================================================================="
-        "===\n", file=sys.stderr)
+        "===\n",
+        file=sys.stderr
+    )
 
     print("Please check output in file: %s/regression.log\n" % CURRENT_PATH)
 
