@@ -48,6 +48,8 @@ const providePlugin = new webpack.ProvidePlugin({
   Backbone: 'backbone',
   Backgrid: 'backgrid',
   pgAdmin: 'pgadmin',
+  'moment': 'moment',
+  'window.moment':'moment',
 });
 
 // Minify and omptimize JS/CSS to reduce bundle size. It is used in production
@@ -100,29 +102,54 @@ const sourceMapDevToolPlugin = new webpack.SourceMapDevToolPlugin({
   columns: false,
 });
 
+
+
+function cssToBeSkiped(curr_path) {
+  /** Skip all templates **/
+  if(curr_path.indexOf('template') > -1) {
+    return true;
+  }
+
+  for(let i=0; i< webpackShimConfig.css_bundle_skip.length; i++) {
+    if(path.join(__dirname, webpackShimConfig.css_bundle_skip[i]) === curr_path){
+      return true;
+    }
+  }
+  return false;
+}
+
 /* Get all the style files recursively and store in array to
- * give input to webpack. We are skipping pgadmin/static directory
- * and adding it to array manually
+ * give input to webpack.
  */
-let pgadminStyles = [path.join(__dirname, './pgadmin/static/scss/pgadmin.scss')];
-function pushPgadminStyles(curr_path) {
-  if(path.join(__dirname, './pgadmin/static') === curr_path ||
-      curr_path.indexOf('template') > -1) {
+function pushModulesCss(curr_path, pgadminStyles) {
+  /** Skip Directories */
+  if(cssToBeSkiped(curr_path)) {
     return;
   }
 
   fs.readdirSync(curr_path).map(function(curr_file) {
+    /** Skip Files */
+    if(cssToBeSkiped(path.join(curr_path, curr_file))) {
+      return;
+    }
+
     let stats = fs.statSync(path.join(curr_path, curr_file));
     /* if directory, dig further */
     if(stats.isDirectory()) {
-      pushPgadminStyles(path.join(curr_path, curr_file));
+      pushModulesCss(path.join(curr_path, curr_file), pgadminStyles);
     }
     else if(stats.isFile() && (curr_file.endsWith('.scss') || curr_file.endsWith('.css'))) {
       pgadminStyles.push(path.join(curr_path, curr_file));
     }
   });
 }
-pushPgadminStyles(path.join(__dirname,'./pgadmin'));
+
+let pgadminStyles = [];
+/* Include what is given in shim config */
+for(let i=0; i<webpackShimConfig.css_bundle_include.length; i++) {
+  pgadminStyles.push(path.join(__dirname, webpackShimConfig.css_bundle_include[i]));
+}
+pushModulesCss(path.join(__dirname,'./pgadmin'), pgadminStyles);
 
 module.exports = {
   stats: { children: false },
