@@ -14,6 +14,8 @@ module.exports = {
       _: 'underscore',
       'underscore.string': 'underscore.string',
       'window.jQuery': 'jquery',
+      'moment': 'moment',
+      'window.moment':'moment',
     }),
   ],
 
@@ -65,7 +67,7 @@ module.exports = {
       'jquery.ui': path.join(__dirname, './node_modules/slickgrid/lib/jquery-ui-1.11.3'),
       'spectrum': path.join(__dirname, './node_modules/spectrum-colorpicker/spectrum'),
       'bignumber': path.join(__dirname, './node_modules/bignumber.js/bignumber'),
-      'bootstrap.datetimepicker': path.join(__dirname, './node_modules/eonasdan-bootstrap-datetimepicker/build/js/bootstrap-datetimepicker.min'),
+      'bootstrap.datetimepicker': path.join(__dirname, './node_modules/tempusdominus-bootstrap-4/build/js/tempusdominus-bootstrap-4.min'),
       'bootstrap.switch': path.join(__dirname, './node_modules/bootstrap-switch/dist/js/bootstrap-switch'),
       'backbone': path.join(__dirname, './node_modules/backbone/backbone'),
       'backform': path.join(__dirname, './node_modules/backform/src/backform'),
@@ -78,6 +80,7 @@ module.exports = {
       'slickgrid.grid': nodeModulesDir + '/slickgrid/slick.grid',
       'bean': path.join(__dirname, './node_modules/flotr2/lib/bean'),
       'flotr2': path.join(__dirname, './node_modules/flotr2/flotr2.amd'),
+      'moment': path.join(__dirname, './node_modules/moment/moment'),
       'browser': path.resolve(__dirname, 'pgadmin/browser/static/js'),
       'pgadmin': sourcesDir + '/js/pgadmin',
       'pgadmin.sqlfoldcode': sourcesDir + '/js/codemirror/addon/fold/pgadmin-sqlfoldcode',
